@@ -1,5 +1,8 @@
 /* Loading of PEM encoded files with optional encryption
  * Copyright (C) 2001-2004 Andreas Steffen, Zuercher Hochschule Winterthur
+ * Copyright (C) 2004-2008  Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2004-2009  Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,10 +14,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pem.h,v 1.3 2004/06/14 01:46:03 mcr Exp $
  */
 
 #include "certs.h"
 
 extern err_t pemtobin(chunk_t *blob, prompt_pass_t *pass, const char* label
     , bool *pgp);
+
+#ifdef HAVE_LIBNSS
+extern void do_3des_nss(u_int8_t *buf, size_t buf_len, u_int8_t *key
+    , size_t key_size, u_int8_t *iv, bool enc);
+#endif
