@@ -12,7 +12,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pluto_constants.h,v 1.39 2005/10/03 19:58:12 mcr Exp $
  */
 
 /* Control and lock pathnames */
@@ -31,9 +30,8 @@ enum kernel_interface {
   USE_KLIPS = 3,
   USE_NETKEY= 4,
   USE_WIN2K = 5,
-  USE_FUNK  = 6,
-  USE_MASTKLIPS = 7,
-  USE_BSDKAME = 8,
+  USE_MASTKLIPS = 6,
+  USE_BSDKAME = 7,
 };
 
 /* RFC 3706 Dead Peer Detection */
@@ -44,6 +42,17 @@ enum dpd_action {
   DPD_ACTION_RESTART_BY_PEER = 3
 };
 
+/* Cisco interop: values remote_peer_type= */
+enum keyword_remotepeertype {
+    NON_CISCO = 0,
+    CISCO  = 1,
+};
+
+/* Network Manager support */
+enum keyword_nmconfigured {
+    NO = 0,
+    YES = 1,
+};
 
 /* Timer events */
 
@@ -178,7 +187,7 @@ typedef enum {
                                                 /* cause pluto to send all messages twice. */
 
 #define DBG_NONE	0	/* no options on, including impairments */
-#define DBG_ALL		LRANGES(DBG_RAW, DBG_X509)  /* all logging options on EXCEPT DBG_PRIVATE */
+#define DBG_ALL		LRANGES(DBG_RAW, DBG_OPPOINFO)  /* all logging options on EXCEPT DBG_PRIVATE and DBG_WHACKWATCH */
 #endif
 
 /* State of exchanges
@@ -535,21 +544,13 @@ enum pluto_policy {
 
 /* Encapsulation Mode attribute */
 
-#define ENCAPSULATION_MODE_UNSPECIFIED 0	/* not legal -- used internally */
-#define ENCAPSULATION_MODE_TUNNEL      1
-#define ENCAPSULATION_MODE_TRANSPORT   2
-
-#define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
+#define ENCAPSULATION_MODE_UNSPECIFIED             0	/* not legal -- used internally */
+#define ENCAPSULATION_MODE_TUNNEL                  1
+#define ENCAPSULATION_MODE_TRANSPORT               2
 #define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
 #define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
-
-#ifdef NAT_TRAVERSAL
 #define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
 #define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
-#define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
-#endif
 
 /* Oakley Lifetime Type attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
