@@ -126,9 +126,15 @@ struct whack_message {
     enum dpd_action dpd_action;
     int dpd_count;
 
+    /*Cisco interop:  remote peer type*/
+    enum keyword_remotepeertype remotepeertype;
+
     /* Force the use of NAT-T on a connection */
     bool forceencaps;
 
+    /*Checking if this connection is configured by Network Manager*/
+    enum keyword_nmconfigured nmconfigured;
+
     /*  note that each end contains string 2/5.id, string 3/6 cert,
      *  and string 4/7 updown
      */
