@@ -33,16 +33,16 @@
 #include "ipsecconf/oeconns.h"
 
 #ifdef HAVE_LIBNSS
-//#ifdef FIPS_CHECK
+/* #ifdef FIPS_CHECK */
 #include "oswconf.h"
 #endif
 
 static char _tmp_err[512];
 
-/** 
- * A policy only conn means that we load it, and do the appropriate firewalling to make
- * sure that no packets get out that this conn would apply to, but we refuse to negotiate
- * it in any way, either incoming or outgoing.
+/*
+ * A policy only conn means that we load it, and do the appropriate firewalling
+ * to make sure that no packets get out that this conn would apply to, but we
+ * refuse to negotiate it in any way, either incoming or outgoing.
  */
 #define POLICY_ONLY_CONN(conn) if(conn->options[KBF_AUTO] > STARTUP_ROUTE) { conn->options[KBF_AUTO]=STARTUP_POLICY; }
 
@@ -56,7 +56,7 @@ char **new_list(char *value);
  * @param cfg starter_config struct
  * @return void
  */
-static void default_values (struct starter_config *cfg)
+void ipsecconf_default_values(struct starter_config *cfg)
 {
 	if (!cfg) return;
 	memset(cfg, 0, sizeof(struct starter_config));
@@ -74,6 +74,14 @@ static void default_values (struct starter_config *cfg)
 #endif
 	cfg->conn_default.options[KBF_TYPE] = KS_TUNNEL;
 
+	/*Cisco interop: remote peer type*/
+	cfg->conn_default.options[KBF_REMOTEPEERTYPE] = NON_CISCO;
+
+	/*Network Manager support*/
+#ifdef HAVE_NM
+	cfg->conn_default.options[KBF_NMCONFIGURED] = NO;
+#endif
+
 	cfg->conn_default.policy = POLICY_RSASIG|POLICY_TUNNEL|POLICY_ENCRYPT|POLICY_PFS;
 	cfg->conn_default.policy |= POLICY_IKEV2_ALLOW;
 
@@ -356,19 +364,24 @@ static int validate_end(struct starter_conn *conn_st
 	break;
 
     case KH_IFACE:
-	assert(end->strings[KSCF_IP] != NULL);
-
-	if (end->iface) free(end->iface);
-	end->iface = xstrdup(end->strings[KNCF_IP]);
-	if (starter_iface_find(end->iface, family, &(end->addr),
-			       &(end->nexthop)) == -1) {
-	    conn_st->state = STATE_INVALID;
-	}
+	/* generally, this doesn't show up at this stage */
 	break;
 	
     case KH_IPADDR:
 	assert(end->strings[KSCF_IP] != NULL);
 
+	if (end->strings[KSCF_IP][0]=='%') {
+	    if (end->iface) free(end->iface);
+	    end->iface = xstrdup(end->strings[KSCF_IP]+1);
+	    if (starter_iface_find(end->iface, family, &(end->addr),
+				   &(end->nexthop)) == -1) {
+	        conn_st->state = STATE_INVALID;
+	    }
+	    /* not numeric, so set the type to the iface type */
+	    end->addrtype = KH_IFACE;
+	    break;
+	}
+
 	er = ttoaddr_num(end->strings[KNCF_IP], 0, family, &(end->addr));
 	if(er) {
 	    /* not numeric, so set the type to the string type */
@@ -976,7 +989,7 @@ static int load_conn (struct starter_config *cfg
 	conn->policy &= ~(POLICY_ID_AUTH_MASK);
 
 #ifdef HAVE_LIBNSS
-//#ifdef FIPS_CHECK
+/* #ifdef FIPS_CHECK */
         if(Pluto_IsFIPS()) {
 		if((conn->options[KBF_AUTHBY] & POLICY_PSK) == POLICY_PSK){
 		starter_log(LOG_LEVEL_INFO
@@ -1184,7 +1197,7 @@ struct starter_config *confread_load(const char *file
 	/**
 	 * Set default values
 	 */
-	default_values(cfg);
+	ipsecconf_default_values(cfg);
 
 	if(ctlbase) {
 	    pfree(cfg->ctlbase);
