@@ -171,13 +171,36 @@ struct keyword_enum_value kw_proto_stack_list[]={
     { "kame",         USE_BSDKAME },
     { "bsdkame",      USE_BSDKAME },
     { "win2k",        USE_WIN2K },
-    { "funk",         USE_FUNK },
 };
 
 struct keyword_enum_values kw_proto_stack=
     { kw_proto_stack_list, sizeof(kw_proto_stack_list)/sizeof(struct keyword_enum_value)};
 
 
+/*
+ *  Cisco interop: remote peer type
+ */
+
+struct keyword_enum_value kw_remote_peer_type_list[]={
+    { "cisco",         CISCO },
+};
+
+
+struct keyword_enum_values kw_remote_peer_type=
+    { kw_remote_peer_type_list, sizeof(kw_remote_peer_type_list)/sizeof(struct keyword_enum_value)};
+
+ /*
+  *  Network Manager support
+  */ 
+#ifdef HAVE_NM
+struct keyword_enum_value kw_nm_configured_list[]={
+    { "yes",         YES },
+};
+
+struct keyword_enum_values kw_nm_configured=
+    { kw_nm_configured_list, sizeof(kw_nm_configured_list)/sizeof(struct keyword_enum_value)};
+#endif
+
 /*
  * Values for right= and left=
  */
@@ -203,6 +226,7 @@ struct keyword_enum_value kw_plutodebug_values[]={
     { "x509",     DBG_X509 },
     { "dpd",      DBG_DPD }, 
     { "pfkey",    DBG_PFKEY }, 
+    { "natt",     DBG_NATT },
     { "nattraversal", DBG_NATT },
 
     { "impair-delay-adns-key-answer", IMPAIR_DELAY_ADNS_KEY_ANSWER },
@@ -341,6 +365,16 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"keylife",        kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"lifetime",       kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"salifetime",     kv_conn|kv_auto, kt_time,   KBF_SALIFETIME,NOT_ENUM},
+
+    /* Cisco interop: remote peer type*/
+    {"remote_peer_type", kv_conn|kv_auto, kt_enum, KBF_REMOTEPEERTYPE, &kw_remote_peer_type},
+
+    /* Network Manager support*/
+#ifdef HAVE_NM
+    {"nm_configured", kv_conn|kv_auto, kt_enum, KBF_NMCONFIGURED, &kw_nm_configured},
+#endif
+
+
 #ifdef NAT_TRAVERSAL
     {"forceencaps",    kv_conn|kv_auto, kt_bool,   KBF_FORCEENCAP, NOT_ENUM},
 #endif
