@@ -414,8 +414,7 @@ struct oe_conn oe_block = {
 	.oe_cn = "block",
 	.oe_sc = {
 		.policy = POLICY_TUNNEL|POLICY_PFS|
-		POLICY_GROUP|POLICY_GROUTED|POLICY_SHUNT_REJECT|
-		POLICY_IKEV2_ALLOW,
+		POLICY_GROUP|POLICY_GROUTED|POLICY_SHUNT_REJECT|POLICY_IKEV2_ALLOW,
 		
 		.desired_state = STARTUP_ROUTE,
 		
