@@ -12,7 +12,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: starterwhack.c,v 1.8 2004/12/01 07:33:14 ken Exp $
  */
 
 #include <sys/types.h>
@@ -119,12 +118,11 @@ int starter_whack_read_reply(int sock,
 		    }
 		    
 		    le++;	/* include NL in line */
-		    if(write(1, ls, le - ls) == -1) {
+		    if(write(STDOUT_FILENO, ls, le - ls) == -1) {
 			int e = errno;
 			starter_log(LOG_LEVEL_ERR, "whack: write() failed (%d %s), and ignored.\n",
 		    		e, strerror(e));
 		    }
-		    fsync(1);
 		    
 		    /* figure out prefix number
 		     * and how it should affect our exit status
@@ -283,6 +281,7 @@ static void set_whack_end(struct starter_config *cfg
 		break;
 		
 	case KH_IPADDR:
+	case KH_IFACE:
 		w->host_addr = l->addr;
 		break;
 
@@ -518,6 +517,18 @@ static int starter_whack_basic_add_conn(struct starter_config *cfg
 	}
 #endif
 
+	/*Cisco interop : remote peer type*/
+	if(conn->options_set[KBF_REMOTEPEERTYPE]) {
+		msg.remotepeertype=conn->options[KBF_REMOTEPEERTYPE];
+	}
+
+#ifdef HAVE_NM
+	/*Network Manager support*/
+	if(conn->options_set[KBF_NMCONFIGURED]) {
+		msg.nmconfigured=conn->options[KBF_NMCONFIGURED];
+	}
+#endif
+
 	set_whack_end(cfg, "left",  &msg.left, &conn->left);
 	set_whack_end(cfg, "right", &msg.right, &conn->right);
 
