@@ -536,24 +536,27 @@ static const char *const sa_lifetime_name[] = {
 enum_names sa_lifetime_names =
     { SA_LIFE_TYPE_SECONDS, SA_LIFE_TYPE_KBYTES, sa_lifetime_name, NULL };
 
+
 /* Encapsulation Mode attribute */
 
-static const char *const enc_mode_name[] = {
+static const char *const enc_rfc_mode_name[] = {
 	"ENCAPSULATION_MODE_TUNNEL",
 	"ENCAPSULATION_MODE_TRANSPORT",
-	"ENCAPSULATION_MODE_UDP_TUNNEL",
-	"ENCAPSULATION_MODE_UDP_TRANSPORT",
+	"ENCAPSULATION_MODE_UDP_TUNNEL_RFC",
+	"ENCAPSULATION_MODE_UDP_TRANSPORT_RFC",
     };
 
-static const char *const enc_udp_mode_name[] = {
-	"ENCAPSULATION_MODE_UDP_TUNNEL",
-	"ENCAPSULATION_MODE_UDP_TRANSPORT",
+static const char *const enc_draft_mode_name[] = {
+	"ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS",
+	"ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS",
     };
 
-static enum_names enc_udp_mode_names =
-    { ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS, ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS, enc_udp_mode_name, NULL };
+static enum_names enc_rfc_mode_names =
+    { ENCAPSULATION_MODE_TUNNEL, ENCAPSULATION_MODE_UDP_TRANSPORT_RFC, enc_rfc_mode_name, NULL };
+
 enum_names enc_mode_names =
-    { ENCAPSULATION_MODE_TUNNEL, ENCAPSULATION_MODE_UDP_TRANSPORT_RFC, enc_mode_name, &enc_udp_mode_names };
+    { ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS, ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS, enc_draft_mode_name, &enc_rfc_mode_names };
+
 
 /* Auth Algorithm attribute */
 
@@ -574,6 +577,27 @@ enum_names
     auth_alg_names =
 	{ AUTH_ALGORITHM_NONE, AUTH_ALGORITHM_AES_CBC , auth_alg_name, NULL };
 
+const char *const modecfg_cisco_attr_name[] = {
+        "CISCO_BANNER",
+        "CISCO_SAVE_PW",
+        "CISCO_DEF_DOMAIN",
+        "CISCO_SPLIT_DNS",
+        "CISCO_SPLIT_INC",
+        "CISCO_UDP_ENCAP_PORT",
+        "CISCO_UNKNOWN",
+        "CISCO_DO_PFS",
+        "CISCO_FW_TYPE",
+        "CISCO_BACKUP_SERVER",
+        "CISCO_DDNS_HOSTNAME",
+        NULL
+    };
+
+enum_names modecfg_cisco_attr_names_tv =
+    { CISCO_BANNER + ISAKMP_ATTR_AF_TV , CISCO_DDNS_HOSTNAME + ISAKMP_ATTR_AF_TV, modecfg_cisco_attr_name , NULL };
+
+enum_names modecfg_cisco_attr_names =
+    { CISCO_BANNER, CISCO_DDNS_HOSTNAME, modecfg_cisco_attr_name , &modecfg_cisco_attr_names_tv };
+
 /* From draft-beaulieu-ike-xauth */
 const char *const xauth_attr_name[] = {
 	"XAUTH-TYPE",
@@ -590,7 +614,7 @@ const char *const xauth_attr_name[] = {
     };
 
 enum_names xauth_attr_names_tv =
-    { XAUTH_TYPE + ISAKMP_ATTR_AF_TV , XAUTH_ANSWER + ISAKMP_ATTR_AF_TV, xauth_attr_name , NULL };
+    { XAUTH_TYPE + ISAKMP_ATTR_AF_TV , XAUTH_ANSWER + ISAKMP_ATTR_AF_TV, xauth_attr_name , &modecfg_cisco_attr_names };
 
 enum_names xauth_attr_names =
     { XAUTH_TYPE , XAUTH_ANSWER, xauth_attr_name , &xauth_attr_names_tv };
@@ -772,9 +796,29 @@ static const char *const oakley_group_name_rfc3526[] = {
 	"OAKLEY_GROUP_MODP6144",
 	"OAKLEY_GROUP_MODP8192"
 };
+
+#ifdef USE_MODP_RFC5114
+static const char *const oakley_group_name_rfc5114[] = {
+        "OAKLEY_GROUP_DH22",
+        "OAKLEY_GROUP_DH23",
+        "OAKLEY_GROUP_DH24"
+};
+#endif
+
+#ifdef USE_MODP_RFC5114
+enum_names oakley_group_names_rfc5114 =
+    { OAKLEY_GROUP_DH22, OAKLEY_GROUP_DH24,
+            oakley_group_name_rfc5114, NULL };
+#endif
+
 enum_names oakley_group_names_rfc3526 =
     { OAKLEY_GROUP_MODP2048, OAKLEY_GROUP_MODP8192,
-	    oakley_group_name_rfc3526, NULL };
+	    oakley_group_name_rfc3526,
+#ifdef USE_MODP_RFC5114
+	    &oakley_group_names_rfc5114 };
+#else
+	    NULL };
+#endif
 
 enum_names oakley_group_names =
     { OAKLEY_GROUP_MODP768, OAKLEY_GROUP_MODP1536, 
@@ -842,9 +886,38 @@ static const char *const notification_dpd_name[] = {
         "R_U_THERE_ACK",
 };
 
+static const char *const notification_cisco_chatter_name[] = {
+	"ISAKMP_N_CISCO_HELLO", /* 30000 */
+	"ISAKMP_N_CISCO_WWTEBR",
+	"ISAKMP_N_CISCO_SHUT_UP",
+    };
+
+static const char *const notification_ios_alives_name[] = {
+	"ISAKMP_N_IOS_KEEP_ALIVE_REQ", /* 32768*/
+	"ISAKMP_N_IOS_KEEP_ALIVE_ACK",
+    };
+
+static const char *const notification_cisco_more_name[] = {
+        "ISAKMP_N_CISCO_LOAD_BALANCE", /* 40501 */
+	"ISAKMP_N_CISCO_UNKNOWN_40502",
+	"ISAKMP_N_CISCO_PRESHARED_KEY_HASH",
+    };
+
+enum_names notification_cisco_more_names = 
+    {  ISAKMP_N_CISCO_LOAD_BALANCE, ISAKMP_N_CISCO_PRESHARED_KEY_HASH,
+      notification_cisco_more_name, NULL };
+
+enum_names notification_ios_alives_names =
+    { ISAKMP_N_IOS_KEEP_ALIVE_REQ, ISAKMP_N_IOS_KEEP_ALIVE_ACK,
+      notification_ios_alives_name, &notification_cisco_more_names };
+
+enum_names notification_cisco_chatter_names =
+    { ISAKMP_N_CISCO_HELLO, ISAKMP_N_CISCO_SHUT_UP,
+      notification_cisco_chatter_name, &notification_ios_alives_names };
+
 enum_names notification_dpd_names =
     { R_U_THERE, R_U_THERE_ACK,
-      notification_dpd_name, NULL };
+      notification_dpd_name, &notification_cisco_chatter_names };
 
 enum_names notification_names =
     { INVALID_PAYLOAD_TYPE, UNEQUAL_PAYLOAD_LENGTHS,
