@@ -1393,9 +1393,9 @@ decrypt_sig(chunk_t sig, int alg, const x509cert_t *issuer_cert,
 	    if(PK11_VerifyRecover(publicKey,&signature,&dsig,osw_return_nss_password_file_info()) == SECSuccess )
 	    {
             DBG(DBG_PARSING,
-                DBG_dump("NSS decrypted sig: ", dsig.data, dsig.len)
-            )
-            DBG_log("NSS: length of decrypted sig = %d", dsig.len);
+                DBG_dump("NSS decrypted sig: ", dsig.data, dsig.len);
+                DBG_log("NSS: length of decrypted sig = %d", dsig.len);
+	    );
 	    }
 
             pfree(nc.ptr);
@@ -1406,12 +1406,14 @@ decrypt_sig(chunk_t sig, int alg, const x509cert_t *issuer_cert,
 	   if(memcmp(dsig.data+dsig.len-digest->len,digest->ptr, digest->len)==0)
 	   {
             pfree(dsigc.ptr);
-            DBG_log("NSS : RSA Signature verified, hash values matched");
+	    DBG(DBG_PARSING,
+		DBG_log("NSS: RSA Signature verified, hash values matched")
+	    );
 	    return TRUE;
 	   }
 
            pfree(dsigc.ptr);
-	   DBG_log("NSS : RSA Signature NOT verified");
+	   DBG(DBG_PARSING, DBG_log("NSS: RSA Signature NOT verified"));
 	   return FALSE;
 	}
 	default:
@@ -1870,10 +1872,10 @@ parse_authorityInfoAccess(chunk_t blob, int level0, chunk_t *accessLocation)
 			    return;
 			}
 		    }
-		    openswan_log("warning: ignoring OCSP InfoAccessLocation with unkown protocol");
+		    openswan_log("warning: ignoring OCSP InfoAccessLocation with unknown protocol");
 		    break;
 		default:
-		    /* unkown accessMethod, ignoring */
+		    /* unknown accessMethod, ignoring */
 		    break;
 		}
 	    }
@@ -2176,7 +2178,6 @@ parse_x509crl(chunk_t blob, u_int level0, x509crl_t *crl)
 		revokedCert_t *revokedCert = alloc_thing(revokedCert_t, "revokedCert");
 		/* since it is assumed here that CRL_OBJ_USER_CERTIFICATE is reached
 		   before CRL_OBJ_REVOCATION_DATE */
-		// Paul passert(userCertificate.len == 0);
 		revokedCert->userCertificate = userCertificate;
 		revokedCert->revocationDate = parse_time(object, level);
 		revokedCert->next = crl->revokedCertificates;
