@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * RCSID $Id: packet.c,v 1.49 2005/01/23 18:53:56 mcr Exp $
  */
 
 #include <stdio.h>
@@ -1658,34 +1656,44 @@ out_struct(const void *struct_ptr, struct_desc *sd
     return FALSE;
 }
 
+/* Find last complete top-level payload and change its np
+ *  * Note: we must deal with payloads already formatted for the network.
+ *  _*_Note:_we_don't_think_a_FALSE_return_should_happen_but_old_routine_did.
+ *   */
 bool
 out_modify_previous_np(u_int8_t np, pb_stream *outs)
 {
-	size_t len = (outs->cur - outs->start), offset;
-	if (len < sizeof(struct isakmp_hdr)) {
-		return FALSE;
-	}
-	else if (len == sizeof(struct isakmp_hdr)) {
-		struct isakmp_hdr *hdr = (struct isakmp_hdr *)outs->start;
-		hdr->isa_np = np;
-		return TRUE;
-	}
-	else {
-		struct isakmp_generic *hdr;
-		for (offset = sizeof(struct isakmp_hdr); offset < len ;
-			offset += ntohs(hdr->isag_length)) {
-			if ((len - offset) < sizeof(struct isakmp_generic))
-				return FALSE;
-			hdr = (struct isakmp_generic *)(outs->start+offset);
-			if ((len - offset) < ntohs(hdr->isag_length))
-				return FALSE;
-			if ((len - offset) == ntohs(hdr->isag_length)) {
-				hdr->isag_np = np;
-				return TRUE;
-			}
+    u_int8_t *pl = outs->start;
+    size_t left = outs->cur - outs->start;
+
+    passert(left >= NSIZEOF_isakmp_hdr);    /* not even room for isakmp_hdr! */
+    if (left == NSIZEOF_isakmp_hdr) {
+	/* no payloads, just the isakmp_hdr: insert np here */
+	passert(pl[NOFFSETOF_isa_np] == ISAKMP_NEXT_NONE ||
+		pl[NOFFSETOF_isa_np] == ISAKMP_NEXT_HASH);
+	pl[NOFFSETOF_isa_np] = np;
+    } else {
+	pl += NSIZEOF_isakmp_hdr;       /* skip over isakmp_hdr */
+	left -= NSIZEOF_isakmp_hdr;
+	for (;;) {
+		size_t pllen;
+
+		passert(left >= NSIZEOF_isakmp_generic);
+		pllen = (pl[NOFFSETOF_isag_length] << 8)
+			| pl[NOFFSETOF_isag_length + 1];
+		passert(left >= pllen);
+		if (left == pllen) {
+			/* found last top-level payload */
+			pl[NOFFSETOF_isag_np] = np;
+			break;  /* done */
+		} else {
+			/* this payload is not the last: scan forward */
+			pl += pllen;
+			left -= pllen;
 		}
 	}
-	return FALSE;
+	}
+	return TRUE;
 }
 
 bool
