@@ -3,6 +3,7 @@
 #include "constants.h"
 #include "oswalloc.h"
 #include "whack.h"
+#include "oswlog.h"
 
 void readwhackmsg(char *infile)
 {
@@ -17,7 +18,8 @@ void readwhackmsg(char *infile)
     }
 
     /* okay, eat first line, it's a comment, but log it. */
-    fgets(b1, sizeof(b1), record);
+    if(fgets(b1, sizeof(b1), record)==NULL)
+	DBG(DBG_PARSING, DBG_log("readwhackmsg: fgets returned NULL"));
     printf("Pre-amble: %s", b1);
     
     plen=0;
@@ -28,7 +30,8 @@ void readwhackmsg(char *infile)
 	struct whack_message m1;
 	int abuflen;
 
-	fread(&a, 4, 2, record);  /* eat time stamp */
+	if(fread(&a, 4, 2, record) == 0) ; /* eat time stamp */
+		DBG(DBG_PARSING, DBG_log( "readwhackmsg: fread returned 0"));
 	
 	/* account for this header we just consumed */
 	plen -= 12;
