@@ -237,7 +237,7 @@ whack_get_value(char *buf, size_t bufsize)
     len = 0;
     while(try > 0 && len==0)
     {
-	fprintf(stderr, "Name enter:   ");
+	fprintf(stderr, "Enter username:   ");
 	
 	memset(buf, 0, bufsize);
 	
@@ -274,7 +274,7 @@ whack_get_secret(char *buf, size_t bufsize)
 
     fflush(stdout);
     usleep(20000); /* give fflush time for flushing */
-    secret = getpass("Enter secret: ");
+    secret = getpass("Enter passphrase: ");
     secret = (secret == NULL) ? "" : secret;
 
     strncpy(buf, secret, bufsize);
