@@ -3,7 +3,8 @@
  * header file for Openswan library functions
  * Copyright (C) 1998, 1999, 2000  Henry Spencer.
  * Copyright (C) 1999, 2000, 2001  Richard Guy Briggs
- * Copyright (C) 2003 - 2008  Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2003 - 2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008 - 2009 David McCullough <david_mccullough@securecomputing.com>
  * 
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published by
@@ -38,6 +39,14 @@
 # error "KLIPS is no longer supported on Linux 2.0. Sorry"
 #endif
 
+/*
+ * We use a lot of config defines,  on older kernels that means we
+ * need to include config.h
+ */
+#ifndef AUTOCONF_INCLUDED
+#include <linux/config.h>
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
 # define SPINLOCK
 # define PROC_FS_21
@@ -106,6 +115,10 @@
 # define NEED_SPINLOCK_TYPES
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+# define HAVE_NETDEV_HEADER_OPS 1
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,8)
 # define NEED_INET_PROTOCOL
 #endif
@@ -256,10 +269,12 @@
 /* type of sock.sk_stamp changed from timeval to ktime  */
 #else
 /* internals of struct skbuff changed */
-/* but RedHat ported some of this back to their RHEL kernel, so check for that */
+/* but RedHat/SUSE ported some of this back to their RHEL kernel, so check for that */
 # if !defined(RHEL_MAJOR) || !defined(RHEL_MINOR) || !(RHEL_MAJOR == 5 && RHEL_MINOR >= 2)
 #  define        HAVE_DEV_NEXT
-#  define ip_hdr(skb)  ((skb)->nh.iph)
+#  if defined(CONFIG_SLE_VERSION) && defined(CONFIG_SLE_SP) && (CONFIG_SLE_VERSION == 10 && CONFIG_SLE_SP <= 2)
+#   define ip_hdr(skb)  ((skb)->nh.iph)
+#  endif
 #  define skb_tail_pointer(skb)  ((skb)->tail)
 #  define skb_end_pointer(skb)  ((skb)->end)
 #  define skb_network_header(skb)  ((skb)->nh.raw)
@@ -272,7 +287,7 @@
 #  define skb_mac_header(skb)  ((skb)->mac.raw)
 #  define skb_set_mac_header(skb,off)  ((skb)->mac.raw = (skb)->data + (off))
 # endif
-# if defined(CONFIG_SLE_VERSION) && defined(CONFIG_SLE_SP) && (CONFIG_SLE_VERSION == 10 && CONFIG_SLE_SP >= 2)
+# if defined(CONFIG_SLE_VERSION) && defined(CONFIG_SLE_SP) && (CONFIG_SLE_VERSION == 10 && CONFIG_SLE_SP == 2)
 # define ip_hdr(skb) ((skb)->nh.iph)
 # endif
 #endif
@@ -309,6 +324,7 @@
  */
 # define HAVE_PROC_DIR_ENTRY
 # define        PROC_NET        init_net.proc_net
+# define	PROC_EOF_DATA
 
 # define __ipsec_dev_get(x) __dev_get_by_name(&init_net, x)
 # define ipsec_dev_get(x) dev_get_by_name(&init_net, x)
@@ -322,16 +338,22 @@
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
 # define ip_chk_addr(a) inet_addr_type(&init_net, a)
-
 # define l_inet_addr_type(a)	inet_addr_type(&init_net, a)
-
 #else
 # define ip_chk_addr inet_addr_type
-
 #define l_inet_addr_type	inet_addr_type
-
 #endif
 
+/* not sure when network name spaces got introduced, but it is in 2.6.26 */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
+# define HAVE_NETWORK_NAMESPACE 1
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
+# define HAVE_CURRENT_UID
+#endif
+
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,31)
 #ifndef NETDEV_TX_BUSY
 # ifdef NETDEV_XMIT_CN
 #  define NETDEV_TX_BUSY NETDEV_XMIT_CN
@@ -339,7 +361,30 @@
 #  define NETDEV_TX_BUSY 1
 # endif
 #endif
+#endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
+# define USE_NETDEV_OPS
+#else
+# define skb_dst_drop(s)	({ \
+					if ((s)->dst) \
+						dst_release((s)->dst); \
+					(s)->dst = NULL; \
+				})
+# define skb_dst_set(s,p)	(s)->dst = (p)
+# define skb_dst(s)		(s)->dst
+#endif
+
+#if 0
+/* nicely, latest netdevice.h includes this define */
+#ifndef HAVE_NETDEV_PRIV
+#define netdev_priv(dev) (dev->priv)
+#endif
+#endif
+
+#if !defined(HAVE_CURRENT_UID)
+#define current_uid() (current->uid)
+#endif
 
 #ifdef NET_21
 # define ipsec_kfree_skb(a) kfree_skb(a)
@@ -404,12 +449,16 @@
 	printk(sevlevel "%s: " format , netdev->name , ## arg)
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
-#define	PROC_NET	init_net.proc_net
-#define	PROC_EOF_DATA
-#else
-#define	PROC_NET	proc_net
-#endif
+#ifdef NETDEV_23
+# define ipsec_dev_put(x) dev_put(x)
+# define __ipsec_dev_put(x) __dev_put(x)
+# define ipsec_dev_hold(x) dev_hold(x)
+#else /* NETDEV_23 */
+# define ipsec_dev_get dev_get
+# define __ipsec_dev_put(x) 
+# define ipsec_dev_put(x)
+# define ipsec_dev_hold(x) 
+#endif /* NETDEV_23 */
 
 #ifndef late_initcall
 # include <linux/init.h>
@@ -443,6 +492,14 @@
 # define NF_INET_LOCAL_OUT NF_IP_LOCAL_OUT
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
+#define	inet_sport	sport
+#define	inet_dport	dport
+#define	CTL_NAME(n)
+#else
+#define	CTL_NAME(n)	.ctl_name = n,
+#endif
+
 #if __KERNEL__
 # if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,0)
 #  if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) 
@@ -450,6 +507,9 @@
 #  else
 #   error "kernels before 2.4 are not supported at this time"
 #  endif
+# else
+#   define KLIPS_INC_USE /* nothing */
+#   define KLIPS_DEC_USE /* nothing */
 # endif
 #endif
 
