@@ -110,12 +110,13 @@ enum ipsec_cipher_algo {
   ESP_AES_CCM_8=14,
   ESP_AES_CCM_12=15,
   ESP_AES_CCM_16=16,
-  /* unassigned=17 */
+  ESP_ID17=17, /* unassigned=17 */
   ESP_AES_GCM_8=18,
   ESP_AES_GCM_12=19,
   ESP_AES_GCM_16=20,
   ESP_SEED_CBC=21,
   ESP_CAMELLIA=22,
+  ESP_NULL_AUTH_AES_GMAC=23 /* [RFC4543][Errata1821] */
   /* 249-255 reserved for private use */
 };
 
