@@ -47,9 +47,11 @@ struct ipsectunnelconf
 #define IPSEC_SET_DEV	(SIOCDEVPRIVATE)
 #define IPSEC_DEL_DEV	(SIOCDEVPRIVATE + 1)
 #define IPSEC_CLR_DEV	(SIOCDEVPRIVATE + 2)
-#define IPSEC_UDP_ENCAP_CONVERT	(SIOCDEVPRIVATE + 3)
 #endif
 
+/* This is used for KLIPS as well as NETKEY */
+#define IPSEC_UDP_ENCAP_CONVERT	(SIOCDEVPRIVATE + 3)
+
 #ifdef __KERNEL__
 #include <linux/version.h>
 #ifndef KERNEL_VERSION
@@ -64,7 +66,7 @@ struct ipsecpriv
 	char locked;
 	int  (*hard_start_xmit) (struct sk_buff *skb,
 		struct net_device *dev);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+#ifdef HAVE_NETDEV_HEADER_OPS
 	const struct header_ops *header_ops;
 #else
 
@@ -85,6 +87,10 @@ struct ipsecpriv
 				 unsigned short htype, __u32 daddr);
 #endif /* !NET_21 */
 	void (*header_cache_update)(struct hh_cache *hh, struct net_device *dev, unsigned char *  haddr);
+#endif
+#ifdef USE_NETDEV_OPS
+	const struct net_device_ops *saved_netdev_ops;
+	struct net_device_ops netdev_ops;
 #endif
 	int  (*set_mac_address)(struct net_device *dev, void *addr);
 	struct net_device_stats *(*get_stats)(struct net_device *dev);
