@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: addrtypeof.c,v 1.10 2004-07-10 07:43:47 mcr Exp $
+ * RCSID $Id: addrtypeof.c,v 1.10 2004/07/10 07:43:47 mcr Exp $
  */
 #include "openswan.h"
 
@@ -51,6 +51,36 @@ const unsigned char **dstp;	/* NULL means just a size query */
 		break;
 	}
 
+	if (dstp != NULL)
+		*dstp = p;
+	return n;
+
+}
+/*
+ - addrbytesptr - get pointer to the address bytes of an ip_address
+ */
+size_t				/* 0 for error */
+addrbytesptr_write(src, dstp)
+ip_address *src;
+unsigned char **dstp;	/* NULL means just a size query */
+{
+	unsigned char *p;
+	size_t n;
+
+	switch (src->u.v4.sin_family) {
+	case AF_INET:
+		p = (unsigned char *)&src->u.v4.sin_addr.s_addr;
+		n = 4;
+		break;
+	case AF_INET6:
+		p = (unsigned char *)&src->u.v6.sin6_addr;
+		n = 16;
+		break;
+	default:
+		return 0;
+		break;
+	}
+
 	if (dstp != NULL)
 		*dstp = p;
 	return n;
