@@ -44,6 +44,8 @@
 #include <linux/random.h>       /* get_random_bytes() */
 #include <net/protocol.h>
 
+#include "openswan/ipsec_param2.h"
+
 #include <openswan.h>
 
 #ifdef SPINLOCK
@@ -247,7 +249,7 @@ ipsec_klips_init(void)
         if (error)
                 goto error_xfrm_register;
 
-#else // CONFIG_XFRM_ALTERNATE_STACK
+#else /* CONFIG_XFRM_ALTERNATE_STACK */
 
 #ifdef CONFIG_KLIPS_ESP
 	error |= openswan_inet_add_protocol(&esp_protocol, IPPROTO_ESP,"ESP");
@@ -271,13 +273,21 @@ ipsec_klips_init(void)
 #endif /* CONFIG_KLIPS_IPCOMP */
 #endif
 
-#endif // CONFIG_XFRM_ALTERNATE_STACK
+#endif /* CONFIG_XFRM_ALTERNATE_STACK */
 
 	error |= ipsec_tunnel_init_devices();
         if (error)
                 goto error_tunnel_init_devices;
 
 	error |= ipsec_mast_init_devices();
+	if (error)
+		goto error_mast_init_devices;
+
+#ifdef CONFIG_INET_IPSEC_SAREF
+	error = ipsec_mast_init_saref();
+	if (error)
+		goto error_mast_init_saref;
+#endif
 
 /* This is no longer needed for >= 2.6.23. We use HAVE_UDP_ENCAP_CONVERT */
 #if defined(NET_26) && defined(CONFIG_IPSEC_NAT_TRAVERSAL)
@@ -306,14 +316,20 @@ ipsec_klips_init(void)
 	prng_init(&ipsec_prng, seed, sizeof(seed));
 	return error;
 
-        // undo ipsec_sysctl_register
+        /* undo ipsec_sysctl_register */
 error_sysctl_register:
+#ifdef CONFIG_INET_IPSEC_SAREF
+	ipsec_mast_cleanup_saref();
+error_mast_init_saref:
+#endif
+	ipsec_mast_cleanup_devices();
+error_mast_init_devices:
 	ipsec_tunnel_cleanup_devices();
 error_tunnel_init_devices:
 #ifdef CONFIG_XFRM_ALTERNATE_STACK
         xfrm_deregister_alternate_rcv(ipsec_rcv);
 error_xfrm_register:
-#else // CONFIG_XFRM_ALTERNATE_STACK
+#else /* CONFIG_XFRM_ALTERNATE_STACK */
 #ifdef IPCOMP_USED_ALONE
 #ifdef CONFIG_KLIPS_IPCOMP
 error_openswan_inet_add_protocol_comp:
@@ -337,8 +353,10 @@ error_radijinit:
 	ipsec_sadb_free();
 error_sadb_init:
 error_proc_init:
-        // ipsec_proc_init() does not cleanup after itself, so we have to do it here
-        // TODO: ipsec_proc_init() should roll back what it chaned on failure
+        /* ipsec_proc_init() does not cleanup after itself, so we have to do
+	 * it here
+	 * TODO: ipsec_proc_init() should roll back what it chaned on failure
+	 */
 	ipsec_proc_cleanup();
         ipsec_rcv_state_cache_cleanup ();
 error_rcv_state_cache:
@@ -373,6 +391,10 @@ ipsec_cleanup(void)
 # endif
 #endif
 
+#ifdef CONFIG_INET_IPSEC_SAREF
+	ipsec_mast_cleanup_saref();
+#endif
+
 	error |= ipsec_mast_cleanup_devices();
 
 	KLIPS_PRINT(debug_netlink, /* debug_tunnel & DB_TN_INIT, */
@@ -386,7 +408,7 @@ ipsec_cleanup(void)
 
         xfrm_deregister_alternate_rcv(ipsec_rcv);
 
-#else // CONFIG_XFRM_ALTERNATE_STACK
+#else /* CONFIG_XFRM_ALTERNATE_STACK */
 
 /* we never actually link IPCOMP to the stack */
 #ifdef IPCOMP_USED_ALONE
@@ -409,7 +431,7 @@ ipsec_cleanup(void)
 		       "esp close: can't remove protocol\n");
 #endif /* CONFIG_KLIPS_ESP */
 
-#endif // CONFIG_XFRM_ALTERNATE_STACK
+#endif /* CONFIG_XFRM_ALTERNATE_STACK */
 
 	error |= unregister_netdevice_notifier(&ipsec_dev_notifier);
 
