@@ -13,7 +13,7 @@
  * for more details.
  */
 
-char ipsec_mast_c_version[] = "RCSID $Id: $";
+char ipsec_mast_c_version[] = "Please use ipsec --version instead";
 
 #define __NO_VERSION__
 #include <linux/module.h>
@@ -43,7 +43,9 @@ char ipsec_mast_c_version[] = "RCSID $Id: $";
 #include <linux/etherdevice.h> /* eth_type_trans */
 #include <linux/ip.h>          /* struct iphdr */
 #include <linux/skbuff.h>
+#ifdef NETDEV_25	/* 2.6 kernels */
 #include <net/xfrm.h>
+#endif
 
 #include <openswan.h>
 
@@ -69,7 +71,6 @@ char ipsec_mast_c_version[] = "RCSID $Id: $";
 #include "openswan/ipsec_ipe4.h"
 #include "openswan/ipsec_ah.h"
 #include "openswan/ipsec_esp.h"
-#include "openswan/ipsec_kern24.h"
 
 #include <openswan/pfkeyv2.h>
 #include <openswan/pfkey.h>
@@ -111,34 +112,33 @@ ipsec_mast_close(struct net_device *dev)
 
 static inline int ipsec_mast_xmit2(struct sk_buff *skb)
 {
+#ifdef NETDEV_25	/* 2.6 kernels */
 	return dst_output(skb);
+#else
+	return ip_send(skb);
+#endif
 }
 
-#ifdef HAVE_IPSEC_SAREF
-int ip_cmsg_send_ipsec(struct cmsghdr *cmsg, struct ipcm_cookie *ipc)
+#ifdef CONFIG_INET_IPSEC_SAREF
+static int klips_set_ipc_saref(struct ipcm_cookie *ipc,
+		xfrm_sec_unique_t ref)
 {
 	struct ipsec_sa *sa1;
-	xfrm_sec_unique_t *ref;
 	struct sec_path *sp;
 
-	if(cmsg->cmsg_len != CMSG_LEN(sizeof(xfrm_sec_unique_t))) {
-		return -EINVAL;
-	}
-
-	ref = (xfrm_sec_unique_t *)CMSG_DATA(cmsg);
-
 	sp = secpath_dup(NULL);
-	if(!sp) {
+	if(!sp)
 		return -EINVAL;
-	}
 
-	sp->ref = *ref;
-	KLIPS_PRINT(debug_mast, "sending with saref=%u\n", sp->ref);
+	sp->ref = ref;
+	KLIPS_PRINT(debug_mast, "klips_debug:klips_set_ipc_saref: "
+			"sending with saref=%u\n", sp->ref);
 		
 	sa1 = ipsec_sa_getbyref(sp->ref, IPSEC_REFOTHER);
 	if(sa1 && sa1->ips_out) {
 		ipc->oif = sa1->ips_out->ifindex;
-		KLIPS_PRINT(debug_mast, "setting oif: %d\n", ipc->oif);
+		KLIPS_PRINT(debug_mast, "klips_debug:klips_set_ipc_saref: "
+			"setting oif: %d\n", ipc->oif);
 	}
 	ipsec_sa_put(sa1, IPSEC_REFOTHER);
 	
@@ -146,6 +146,40 @@ int ip_cmsg_send_ipsec(struct cmsghdr *cmsg, struct ipcm_cookie *ipc)
 
 	return 0;
 }
+
+static void klips_get_secpath_refs(struct sec_path *sp,
+		xfrm_sec_unique_t *refme, xfrm_sec_unique_t *refhim)
+{
+	struct ipsec_sa *sa1;
+
+	if(sp==NULL) return;
+
+	KLIPS_PRINT(debug_rcv, "klips_debug:klips_get_secpath_refs: "
+			"retrieving saref=%u from sp=%p\n",
+		    sp->ref, sp);
+
+	*refme = sp->ref;
+
+	sa1 = ipsec_sa_getbyref(sp->ref, IPSEC_REFOTHER);
+	*refhim = sa1 ? sa1->ips_refhim : 0;
+
+	if(sa1)
+		ipsec_sa_put(sa1, IPSEC_REFOTHER);
+}
+
+static struct ipsec_secpath_saref_ops klips_saref_ops = {
+	.set_ipc_saref = klips_set_ipc_saref,
+	.get_secpath_sarefs = klips_get_secpath_refs,
+};
+
+int ipsec_mast_init_saref(void)
+{
+	return register_ipsec_secpath_saref_ops(&klips_saref_ops);
+}
+void ipsec_mast_cleanup_saref(void)
+{
+	unregister_ipsec_secpath_saref_ops(&klips_saref_ops);
+}
 #endif
 
 #if 0
@@ -230,7 +264,8 @@ ipsec_mast_xsm_complete(
 {
 	if (stat != IPSEC_XMIT_OK) {
 		KLIPS_PRINT(debug_mast,
-				"klips_debug:ipsec_mast_xsm_complete: ipsec_xsm failed: %d\n",
+				"klips_debug:ipsec_mast_xsm_complete: "
+				"ipsec_xsm failed: %d\n",
 				stat);
 		goto cleanup;
 	}
@@ -243,6 +278,7 @@ ipsec_mast_xsm_complete(
 	}
 #endif
 
+#ifdef NETDEV_25	/* 2.6 kernels */
 	/* now send the packet again */
 	{
 		struct flowi fl;
@@ -250,6 +286,9 @@ ipsec_mast_xsm_complete(
 		memset(&fl, 0, sizeof(fl));
 		ipsec_xmit_send(ixs, &fl);
 	}
+#else
+	ipsec_xmit_send(ixs, NULL);
+#endif
 
 cleanup:
 	ipsec_xmit_cleanup(ixs);
@@ -275,46 +314,69 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	struct ipsec_xmit_state *ixs;
 	IPsecSAref_t SAref;
 
+	KLIPS_PRINT(debug_mast, "klips_debug:ipsec_mast_start_xmit: skb=%p\n", skb);
 	if(skb == NULL) {
-		printk("mast start_xmit passed NULL\n");
+		printk("ipsec_mast_start_xmit: "
+			"passed NULL\n");
 		return 0;
 	}
 		
 	ixs = ipsec_xmit_state_new();
 	if(ixs == NULL) {
-		printk("mast failed to allocate IXS\n");
+		printk("ipsec_mast_start_xmit: "
+			"failed to allocate IXS\n");
 		return 0;
 	}
 
 	ixs->skb = skb;
 	SAref = 0;
+#ifdef NETDEV_25
 	if(skb->nfmark & 0x80000000) {
 		SAref = NFmark2IPsecSAref(skb->nfmark);
-		KLIPS_PRINT(debug_mast, "getting SAref=%d from nfmark\n",
-			    SAref);
+		KLIPS_PRINT(debug_mast, "klips_debug:ipsec_mast_start_xmit: "
+				"getting SAref=%d from nfmark\n",
+				SAref);
 	}
+#endif
 
-#ifdef HAVE_IPSEC_SAREF
+#ifdef CONFIG_INET_IPSEC_SAREF
 	if(skb->sp && skb->sp->ref != IPSEC_SAREF_NULL) {
 		SAref = skb->sp->ref;
-		KLIPS_PRINT(debug_mast, "getting SAref=%d from sec_path\n",
-			    SAref);
+		KLIPS_PRINT(debug_mast, "klips_debug:ipsec_mast_start_xmit: "
+				"getting SAref=%d from sec_path\n",
+				SAref);
 	}
 #endif
-	KLIPS_PRINT(debug_mast, "skb=%p\n", skb);
 
 	ipsec_xmit_sanity_check_skb(ixs);
 
 	ixs->ipsp = ipsec_sa_getbyref(SAref, IPSEC_REFOTHER);
 	if(ixs->ipsp == NULL) {
-		KLIPS_ERROR(debug_mast, "%s: no SA for saref=%d\n",
-			    dev->name, SAref);
-		ipsec_kfree_skb(skb);
+		KLIPS_ERROR(debug_mast, "klips_debug:ipsec_mast_start_xmit: "
+				"%s: no SA for saref=%d\n",
+				dev->name, SAref);
 		ipsec_xmit_cleanup(ixs);
 		ipsec_xmit_state_delete(ixs);
 		return 0;
 	}
 
+	/* fill in outgoing_said using the ipsp we have */
+	ixs->outgoing_said = ixs->ipsp->ips_said;
+
+#ifdef NETDEV_25
+	/* prevent recursion through the saref route */
+	if(skb->nfmark & 0x80000000) {
+		skb->nfmark = 0;
+	}
+#endif
+#if 0
+	/* TODO: do we have to also have to do this? */
+	if(skb->sp && skb->sp->ref != IPSEC_SAREF_NULL) {
+		secpath_put(skb->sp);
+		skb->sp = NULL;
+	}
+#endif
+
 	/*
 	 * we should be calculating the MTU by looking up a route
 	 * based upon the destination in the SA, and then cache
@@ -323,6 +385,7 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	ixs->cur_mtu = 1460;
 	ixs->physmtu = 1460;
 
+	ixs->mast_mode = 1;
 	ixs->xsm_complete = ipsec_mast_xsm_complete;
 	ixs->state = IPSEC_XSM_INIT2;	/* we start later in the process */
 
@@ -495,7 +558,9 @@ ipsec_mast_neigh_setup_dev(struct net_device *dev, struct neigh_parms *p)
 DEBUG_NO_STATIC int
 ipsec_mast_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
 {
-	struct ipsecmastconf *cf = (struct ipsecmastconf *)&ifr->ifr_data;
+	/* struct ipsecmastconf *cf = (struct ipsecmastconf *)&ifr->ifr_data;*/
+	/* overlay our struct ipsecmast onto ifr.ifr_ifru union (hope it fits!) */
+	struct ipsecmastconf *cf=(struct ipsecmastconf *)ifr->ifr_ifru.ifru_newname;       
 	struct ipsecpriv *prv = netdev_priv(dev);
 
 	cf = cf;
@@ -651,18 +716,25 @@ int
 ipsec_mast_probe(struct net_device *dev)
 {
 	int i;
+	struct ipsecpriv *ipriv;
 
 	KLIPS_PRINT(debug_mast,
-		    "klips_debug:ipsec_mast_init: "
+		    "klips_debug:ipsec_mast_probe: "
 		    "allocating %lu bytes initialising device: %s\n",
 		    (unsigned long) sizeof(struct mastpriv),
 		    dev->name ? dev->name : "NULL");
 
+#ifndef USE_NETDEV_OPS
 	/* Add our mast functions to the device */
 	dev->open		= ipsec_mast_open;
 	dev->stop		= ipsec_mast_close;
 	dev->hard_start_xmit	= ipsec_mast_start_xmit;
 	dev->get_stats		= ipsec_mast_get_stats;
+	dev->set_multicast_list = NULL;
+	dev->do_ioctl		= ipsec_mast_ioctl;
+	dev->set_mac_address 	= NULL;
+	dev->neigh_setup        = ipsec_mast_neigh_setup_dev;
+#endif
 #ifdef alloc_netdev
 	dev->destructor         = free_netdev;
 #endif
@@ -672,28 +744,28 @@ ipsec_mast_probe(struct net_device *dev)
 	if (dev->priv == NULL)
 		return -ENOMEM;
 #endif
+	ipriv = netdev_priv(dev);
 	memset(netdev_priv(dev), 0, sizeof(struct mastpriv));
 
 	for(i = 0; i < sizeof(zeroes); i++) {
 		((__u8*)(zeroes))[i] = 0;
 	}
 	
-	dev->set_multicast_list = NULL;
-	dev->do_ioctl		= ipsec_mast_ioctl;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+#ifdef HAVE_NETDEV_HEADER_OPS
 	dev->header_ops = NULL;
 #else
 	dev->hard_header	= NULL;
 	dev->rebuild_header 	= NULL;
 	dev->header_cache_update= NULL;
 #endif
-	dev->set_mac_address 	= NULL;
-	dev->neigh_setup        = ipsec_mast_neigh_setup_dev;
 	dev->hard_header_len 	= 8+20+20+8;
 	dev->mtu		= 0;
 	dev->addr_len		= 0;
 	dev->type		= ARPHRD_NONE;
 	dev->tx_queue_len	= 10;		
+#ifdef IFF_XMIT_DST_RELEASE
+	dev->priv_flags	       &= ~IFF_XMIT_DST_RELEASE;
+#endif
 	memset((caddr_t)(dev->broadcast),0xFF, ETH_ALEN);	/* what if this is not attached to ethernet? */
 
 	/* New-style flags. */
@@ -714,6 +786,18 @@ static void ipsec_mast_netdev_setup(struct net_device *dev)
 struct net_device *mastdevices[IPSEC_NUM_IFMAX];
 int mastdevices_max=-1;
 
+#ifdef USE_NETDEV_OPS
+static const struct net_device_ops ipsec_mast_ops = {
+	.ndo_init		= ipsec_mast_probe,
+	.ndo_open		= ipsec_mast_open,
+	.ndo_stop		= ipsec_mast_close,
+	.ndo_start_xmit		= ipsec_mast_start_xmit,
+	.ndo_get_stats		= ipsec_mast_get_stats,
+	.ndo_do_ioctl		= ipsec_mast_ioctl,
+	.ndo_neigh_setup	= ipsec_mast_neigh_setup_dev,
+};
+#endif
+
 int ipsec_mast_createnum(int vifnum) 
 {
 	struct net_device *im;
@@ -751,7 +835,11 @@ int ipsec_mast_createnum(int vifnum)
 	memcpy(im->name, name, IFNAMSIZ);
 #endif
 		
+#ifdef USE_NETDEV_OPS
+	im->netdev_ops = &ipsec_mast_ops;
+#else
 	im->init = ipsec_mast_probe;
+#endif
 
 	if(register_netdev(im) != 0) {
 		printk(KERN_ERR "ipsec_mast: failed to register %s\n",
@@ -759,7 +847,7 @@ int ipsec_mast_createnum(int vifnum)
 		return -EIO;
 	}
 
-	dev_hold(im);
+	ipsec_dev_hold(im);
 	mastdevices[vifentry]=im;
 
 	return 0;
@@ -819,7 +907,7 @@ ipsec_mast_get_device(int vifnum)
 		if(vifnum <= mastdevices_max) {
 			nd = mastdevices[vifnum];
 
-			if(nd) dev_hold(nd);
+			if(nd) ipsec_dev_hold(nd);
 			return nd;
 		} else {
 			KLIPS_ERROR(debug_tunnel,
@@ -829,6 +917,16 @@ ipsec_mast_get_device(int vifnum)
 	}
 }
 
+int
+ipsec_is_mast_device(const struct net_device *dev)
+{
+#ifndef USE_NETDEV_OPS
+	return dev && (dev->hard_start_xmit == ipsec_mast_start_xmit);
+#else
+	return dev && (dev->netdev_ops == &ipsec_mast_ops);
+#endif
+}
+
 unsigned int
 ipsec_mast_is_transport(int vifnum)
 {
@@ -867,7 +965,7 @@ ipsec_mast_cleanup_devices(void)
 			kfree(dev_mast->priv);
 			dev_mast->priv=NULL;
 #endif
-			dev_put(mastdevices[i]);
+			ipsec_dev_put(mastdevices[i]);
 			mastdevices[i]=NULL;
 		}
 	}
