@@ -81,7 +81,6 @@
 #include "openswan/ipsec_rcv.h"
 #include "openswan/ipsec_ah.h"
 #include "openswan/ipsec_esp.h"
-#include "openswan/ipsec_kern24.h"
 
 #ifdef CONFIG_KLIPS_IPCOMP
 #include "openswan/ipcomp.h"
@@ -112,7 +111,6 @@ struct ipsec_birth_reply ipsec_ipv6_birth_packet;
 int debug_esp = 0;
 int debug_ah = 0;
 int sysctl_ipsec_inbound_policy_check = 1;
-int debug_tunnel = 0;
 int debug_xmit = 0;
 int debug_xform = 0;
 int debug_eroute = 0;
