@@ -68,7 +68,7 @@
 
 #include <net/ip.h>
 
-#include "openswan/ipsec_kern24.h"
+#include "openswan/ipsec_kversion.h"
 #include "openswan/radij.h"
 #include "openswan/ipsec_encap.h"
 #include "openswan/ipsec_sa.h"
@@ -96,7 +96,6 @@
 
 #include "openswan/ipsec_proto.h"
 #include "openswan/ipsec_alg.h"
-#include "openswan/ipsec_kern24.h"
 
 #ifdef CONFIG_KLIPS_OCF
 #include "ipsec_ocf.h"
@@ -439,37 +438,6 @@ struct sk_buff *ipsec_rcv_natt_decap(struct sk_buff *skb
 }
 #endif
 
-#ifdef HAVE_IPSEC_SAREF
-void ip_cmsg_recv_ipsec(struct msghdr *msg, struct sk_buff *skb)
-{
-	struct ipsec_sa *sa1;
-	struct sec_path *sp;
-	xfrm_sec_unique_t refs[2];
-
-	sp = skb->sp;
-
-	if(sp==NULL) return;
-
-	KLIPS_PRINT(debug_rcv, "retrieving saref=%u from skb=%p\n",
-		    sp->ref, skb);
-
-	sa1 = ipsec_sa_getbyref(sp->ref, IPSEC_REFOTHER);
-	if(sa1) {
-		refs[1]= sa1->ips_refhim;
-	} else {
-		refs[1]= 0;
-	}
-	refs[0]=sp->ref;
-
-	put_cmsg(msg, SOL_IP, IP_IPSEC_REFINFO,
-		 sizeof(xfrm_sec_unique_t)*2, &refs);
-	if(sa1) {
-		ipsec_sa_put(sa1, IPSEC_REFOTHER);
-	}
-}
-#endif
-
-
 void ipsec_rcv_setoutif(struct ipsec_rcv_state *irs)
 {
 	struct sk_buff *skb = irs->skb;
@@ -483,10 +451,17 @@ void ipsec_rcv_setoutif(struct ipsec_rcv_state *irs)
 		}
 		skb->dev = irs->ipsp->ips_out;
 		
+#ifdef USE_NETDEV_OPS
+		if(skb->dev && skb->dev->netdev_ops->ndo_get_stats) {
+			struct net_device_stats *stats = skb->dev->netdev_ops->ndo_get_stats(skb->dev);
+			irs->stats = stats;
+		}
+#else
 		if(skb->dev && skb->dev->get_stats) {
 			struct net_device_stats *stats = skb->dev->get_stats(skb->dev);
 			irs->stats = stats;
 		}
+#endif
 	} 
 }
 
@@ -522,7 +497,10 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 		if((ipsnext = ipsp->ips_next)) {
 			char sa2[SATOT_BUF];
 			size_t sa_len2;
-			sa_len2 = satot(&ipsnext->ips_said, 0, sa2, sizeof(sa2));
+			if (debug_rcv)
+				sa_len2 = satot(&ipsnext->ips_said, 0, sa2, sizeof(sa2));
+			else
+				sa_len2 = 0;
 			KLIPS_PRINT(debug_rcv,
 				    "klips_debug:ipsec_rcv: "
 				    "unexpected SA:%s after IPIP SA:%s\n",
@@ -600,7 +578,8 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 		
 		/* re-do any strings for debugging */
 		irs->ipp = ipp;
-		ipsec_rcv_redodebug(irs);
+		if (debug_rcv)
+			ipsec_rcv_redodebug(irs);
 
 		skb->protocol = htons(ETH_P_IP);
 		skb->ip_summed = 0;
@@ -645,7 +624,7 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 		goto rcvleave;
 	}
 #ifdef CONFIG_NETFILTER
-	skb->nfmark = (skb->nfmark & (~(IPsecSAref2NFmark(IPSEC_SA_REF_TABLE_MASK))))
+	skb->nfmark = (skb->nfmark & (~(IPsecSAref2NFmark(IPSEC_SA_REF_MASK))))
 		| IPsecSAref2NFmark(IPsecSA2SAref(ipsp));
 	KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
 		    "klips_debug:ipsec_rcv: "
@@ -845,7 +824,7 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 
 		if(ipsecdev) {
 			KLIPS_PRINT(debug_rcv && prvdev,
-					"klips_debug:klips26_rcv_encap: "
+					"klips_debug:ipsec_rcv_init: "
 					"assigning packet ownership to "
 					"virtual device %s from "
 					"physical device %s.\n",
@@ -853,13 +832,14 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 			skb->dev = ipsecdev;
 		} else {
 			KLIPS_PRINT(debug_rcv,
-					"klips_debug:klips26_rcv_encap: "
+					"klips_debug:ipsec_rcv_init: "
 					"assigning packet ownership to "
 					"virtual device mast0 from "
 					"physical device %s.\n",
 				skb->dev->name);
+
 			skb->dev = ipsec_mast_get_device(0);
-			
+
 			/* ipsec_mast_get takes the device */
 			if(skb->dev) dev_put(skb->dev);
 		}
@@ -868,7 +848,7 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 			stats = (struct net_device_stats *) &(prvdev->mystats);
 		}
 	} else {
-		KLIPS_PRINT(debug_rcv, "klips_debug:klips26_rcv_encap: "
+		KLIPS_PRINT(debug_rcv, "klips_debug:ipsec_rcv_init: "
 				"device supplied with skb is NULL\n");
 	}
 
@@ -1600,11 +1580,34 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 {
 	struct sk_buff *skb;
 	struct iphdr *ipp;
-	struct ipsec_sa *ipsp = NULL;
 
 	KLIPS_PRINT(debug_rcv, "klips_debug: %s(st=%d,nxt=%d)\n", __FUNCTION__,
 			irs->state, irs->next_state);
 
+	/* set up for decap loop */
+	ipp = irs->ipp;
+	skb = irs->skb;
+
+#ifdef NAT_TRAVERSAL
+	if ((irs->natt_type) && (ipp->protocol != IPPROTO_IPIP)) {
+	       /*
+		* NAT-Traversal and Transport Mode:
+		*   we need to correct TCP/UDP checksum
+		* If we've got NAT-OA, we can fix checksum without recalculation.
+		*/
+		__u32 natt_oa = (irs->ipsp && irs->ipsp->ips_natt_oa) ?
+			((struct sockaddr_in*)(irs->ipsp->ips_natt_oa))->sin_addr.s_addr : 0;
+
+		if (natt_oa != 0) {
+			/* reset source address to what it was before NAT */
+			ipp->saddr = natt_oa;
+			ipp->check = 0;
+			ipp->check = ip_fast_csum((unsigned char *)ipp, ipp->ihl);
+			KLIPS_PRINT(debug_rcv, "csum: %04x\n", ipp->check);
+		}
+	}
+#endif
+
 	/* okay, acted on all SA's, so free the last SA, and move to the next */
 	if(irs->ipsp) {
 		struct ipsec_sa *newipsp;
@@ -1619,11 +1622,6 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 		irs->ipsp=newipsp;
 	}
 
-	/* set up for decap loop */
-	ipp  = irs->ipp;
-	ipsp = irs->ipsp;
-	skb = irs->skb;
-
 #ifdef CONFIG_KLIPS_IPCOMP
 	/* if there is an IPCOMP, but we don't have an IPPROTO_COMP,
 	 * then we can just skip it
@@ -1643,27 +1641,6 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 	}
 #endif /* CONFIG_KLIPS_IPCOMP */
 
-#ifdef NAT_TRAVERSAL
-	if ((irs->natt_type) && (ipp->protocol != IPPROTO_IPIP)) {
-	  /**
-	   * NAT-Traversal and Transport Mode:
-	   *   we need to correct TCP/UDP checksum
-	   *
-	   * If we've got NAT-OA, we can fix checksum without recalculation.
-	   */
-		__u32 natt_oa = ipsp->ips_natt_oa ?
-			((struct sockaddr_in*)(ipsp->ips_natt_oa))->sin_addr.s_addr : 0;
-
-		if(natt_oa != 0) {
-			/* reset source address to what it was before NAT */
-			ipp->saddr = natt_oa;
-			ipp->check = 0;
-			ipp->check = ip_fast_csum((unsigned char *)ipp, ipp->ihl);
-			KLIPS_PRINT(debug_rcv, "csum: %04x\n", ipp->check);
-		}
-	}
-#endif
-
 	/*
 	 * the SA is still locked from the loop
 	 */
@@ -1693,7 +1670,7 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 		/* STUFF */
 	}
 
-#ifdef HAVE_IPSEC_SAREF	
+#ifdef CONFIG_INET_IPSEC_SAREF
 	if(skb->sp) {
 		secpath_put(skb->sp);
 	}
@@ -1704,10 +1681,7 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 	/* release the dst that was attached, since we have likely
 	 * changed the actual destination of the packet.
 	 */
-	if(skb->dst) {
-		dst_release(skb->dst);
-		skb->dst = NULL;
-	}
+	skb_dst_drop(skb);
 	skb->pkt_type = PACKET_HOST;
 	if(irs->hard_header_len &&
 	   (skb_mac_header(skb) != (skb_network_header(skb) - irs->hard_header_len)) &&
@@ -2130,8 +2104,9 @@ int klips26_rcv_encap(struct sk_buff *skb, __u16 encap_type)
 }
 #endif
 
-// ------------------------------------------------------------------------
-// this handles creating and managing state for recv path
+/* ------------------------------------------------------------------------
+ * this handles creating and managing state for recv path
+ */
 
 static spinlock_t irs_cache_lock = SPIN_LOCK_UNLOCKED;
 #ifdef HAVE_KMEM_CACHE_MACRO
@@ -2205,7 +2180,7 @@ ipsec_rcv_state_new (void)
         if (unlikely (NULL == irs))
                 goto bail;
 
-        // initialize the object
+        /* initialize the object */
 #if 1
         memset((caddr_t)irs, 0, sizeof(*irs));
 #else
