@@ -113,7 +113,12 @@ typedef struct {
 
 #define IPS_HASH(said) (((said)->spi + (said)->dst.u.v4.sin_addr.s_addr + (said)->proto) % SADB_HASHMOD)
 
-int
+static int ipsec_saref_verify_slot(IPsecSAref_t ref);
+static int ipsec_SArefSubTable_alloc(unsigned table);
+static int ipsec_saref_freelist_init(void);
+static IPsecSAref_t ipsec_SAref_alloc(int*erorr); /* pass in error var by pointer */
+
+static int
 ipsec_SAref_recycle(void)
 {
 	int table, i;
@@ -191,7 +196,7 @@ ipsec_SAref_recycle(void)
 	return 0;
 }
 
-int
+static int
 ipsec_SArefSubTable_alloc(unsigned table)
 {
 	unsigned entry;
@@ -200,12 +205,11 @@ ipsec_SArefSubTable_alloc(unsigned table)
 	KLIPS_PRINT(debug_xform,
 		    "klips_debug:ipsec_SArefSubTable_alloc: "
 		    "allocating %lu bytes for table %u of %u.\n",
-		    (unsigned long) (IPSEC_SA_REF_SUBTABLE_NUM_ENTRIES * sizeof(struct ipsec_sa *)),
-		    table,
-		    IPSEC_SA_REF_MAINTABLE_NUM_ENTRIES);
+		    (unsigned long) IPSEC_SA_REF_SUBTABLE_SIZE,
+		    table, IPSEC_SA_REF_MAINTABLE_NUM_ENTRIES);
 
 	/* allocate another sub-table */
-	SArefsub = vmalloc(IPSEC_SA_REF_SUBTABLE_NUM_ENTRIES * sizeof(struct ipsec_sa *));
+	SArefsub = kmalloc(IPSEC_SA_REF_SUBTABLE_SIZE, GFP_ATOMIC);
 	if(SArefsub == NULL) {
 		KLIPS_PRINT(debug_xform,
 			    "klips_debug:ipsec_SArefSubTable_alloc: "
@@ -232,7 +236,7 @@ ipsec_SArefSubTable_alloc(unsigned table)
 	return 0;
 }
 
-int
+static int
 ipsec_saref_verify_slot(IPsecSAref_t ref)
 {
 	int ref_table=IPsecSAref2table(ref);
@@ -300,7 +304,7 @@ ipsec_sadb_init(void)
 	return error;
 }
 
-IPsecSAref_t
+static IPsecSAref_t
 ipsec_SAref_alloc(int*error) /* pass in error var by pointer */
 {
 	IPsecSAref_t SAref;
@@ -353,6 +357,7 @@ ipsec_SAref_alloc(int*error) /* pass in error var by pointer */
 	return SAref;
 }
 
+#if 0
 int
 ipsec_sa_print(struct ipsec_sa *ips)
 {
@@ -424,6 +429,7 @@ ipsec_sa_print(struct ipsec_sa *ips)
 	printk("\n");
 	return 0;
 }
+#endif
 
 struct ipsec_sa*
 ipsec_sa_alloc(int*error) /* pass in error var by pointer */
@@ -873,9 +879,10 @@ ipsec_sadb_cleanup(__u8 proto)
 	unsigned i;
 	int error = 0;
 	struct ipsec_sa *ips;
-	//struct ipsec_sa *ipsnext, **ipsprev;
-        //char sa[SATOT_BUF];
-	//size_t sa_len;
+	/* struct ipsec_sa *ipsnext, **ipsprev; */
+	/* char sa[SATOT_BUF]; */
+	/* size_t sa_len; */
+	 
 
 	KLIPS_PRINT(debug_xform,
 		    "klips_debug:ipsec_sadb_cleanup: "
@@ -896,7 +903,7 @@ ipsec_sadb_cleanup(__u8 proto)
 		}
 	}
 
-//errlab:
+/* errlab: */
 
 	spin_unlock_bh(&tdb_lock);
 
@@ -976,7 +983,7 @@ ipsec_sadb_free(void)
 					ipsec_sadb.refTable[table]->entry[entry] = NULL;
 				}
 			}
-			vfree(ipsec_sadb.refTable[table]);
+			kfree(ipsec_sadb.refTable[table]);
 			ipsec_sadb.refTable[table] = NULL;
 		}
 	}
