@@ -29,7 +29,6 @@
 
 #include "openswan/ipsec_param.h"
 
-
 #ifdef MALLOC_SLAB
 # include <linux/slab.h> /* kmalloc() */
 #else /* MALLOC_SLAB */
@@ -49,6 +48,9 @@
 
 #include <asm/uaccess.h>
 #include <asm/checksum.h>
+
+#include "openswan/ipsec_param2.h"
+
 #include <openswan.h>
 #ifdef NET_21
 # define MSS_HACK_		/* experimental */
@@ -68,7 +70,7 @@
 # include <net/tcp.h>		/* TCP options */
 #endif	/* MSS_HACK */
 
-#include "openswan/ipsec_kern24.h"
+#include "openswan/ipsec_kversion.h"
 #include "openswan/radij.h"
 #include "openswan/ipsec_life.h"
 #include "openswan/ipsec_xform.h"
@@ -81,6 +83,7 @@
 #include "openswan/ipsec_ipe4.h"
 #include "openswan/ipsec_ah.h"
 #include "openswan/ipsec_esp.h"
+#include "openswan/ipsec_mast.h"
 
 #ifdef CONFIG_KLIPS_IPCOMP
 #include "openswan/ipcomp.h"
@@ -93,37 +96,6 @@
 #include "openswan/ipsec_alg.h"
 #include "ipsec_ocf.h"
 
-
-/* 
- * Stupid kernel API differences in APIs. Not only do some
- * kernels not have ip_select_ident, but some have differing APIs,
- * and SuSE has one with one parameter, but no way of checking to
- * see what is really what.
- */
-
-#ifdef SUSE_LINUX_2_4_19_IS_STUPID
-#define KLIPS_IP_SELECT_IDENT(iph, skb) ip_select_ident(iph)
-#else
-
-/* simplest case, nothing */
-#if !defined(IP_SELECT_IDENT)
-#define KLIPS_IP_SELECT_IDENT(iph, skb)  do { iph->id = htons(ip_id_count++); } while(0)
-#endif
-
-/* kernels > 2.3.37-ish */
-#if defined(IP_SELECT_IDENT) && !defined(IP_SELECT_IDENT_NEW)
-#define KLIPS_IP_SELECT_IDENT(iph, skb) ip_select_ident(iph, skb->dst)
-#endif
-
-/* kernels > 2.4.2 */
-#if defined(IP_SELECT_IDENT) && defined(IP_SELECT_IDENT_NEW)
-#define KLIPS_IP_SELECT_IDENT(iph, skb) ip_select_ident(iph, skb->dst, NULL)
-#endif
-
-#endif /* SUSE_LINUX_2_4_19_IS_STUPID */
-
-
-
 #if defined(CONFIG_KLIPS_AH)
 #if defined(CONFIG_KLIPS_AUTH_HMAC_MD5) || defined(CONFIG_KLIPS_AUTH_HMAC_SHA1)
 static __u32 zeroes[64];
@@ -261,6 +233,22 @@ void
 ipsec_print_ip(struct iphdr *ip)
 {
 	char buf[ADDRTOA_BUF];
+	struct tcphdr *tcphdr = NULL;
+
+	if (!ip)
+		return;
+
+	/* we are taking some liberties here assuming that the IP and TCP
+	 * headers are contiguous in memory */
+	switch (ip->protocol) {
+	case IPPROTO_TCP:
+	case IPPROTO_UDP:
+		/* NOTE: we only use this for getting port numbers, and they
+		 * are at the same offsets for both tcp and udp headers
+		 */
+		tcphdr = (struct tcphdr*)((caddr_t)ip + (ip->ihl << 2));
+		break;
+	}
 
 	printk(KERN_INFO "klips_debug:   IP:");
 	printk(" ihl:%d", ip->ihl << 2);
@@ -290,24 +278,29 @@ ipsec_print_ip(struct iphdr *ip)
 	printk(" chk:%d", ntohs(ip->check));
 	addrtoa(*((struct in_addr*)(&ip->saddr)), 0, buf, sizeof(buf));
 	printk(" saddr:%s", buf);
-	if(ip->protocol == IPPROTO_UDP)
+	if(tcphdr)
 		printk(":%d",
-		       ntohs(((struct udphdr*)((caddr_t)ip + (ip->ihl << 2)))->source));
-	if(ip->protocol == IPPROTO_TCP)
-		printk(":%d",
-		       ntohs(((struct tcphdr*)((caddr_t)ip + (ip->ihl << 2)))->source));
+		       ntohs(tcphdr->source));
 	addrtoa(*((struct in_addr*)(&ip->daddr)), 0, buf, sizeof(buf));
 	printk(" daddr:%s", buf);
-	if(ip->protocol == IPPROTO_UDP)
+	if(tcphdr)
 		printk(":%d",
-		       ntohs(((struct udphdr*)((caddr_t)ip + (ip->ihl << 2)))->dest));
-	if(ip->protocol == IPPROTO_TCP)
-		printk(":%d",
-		       ntohs(((struct tcphdr*)((caddr_t)ip + (ip->ihl << 2)))->dest));
+		       ntohs(tcphdr->dest));
 	if(ip->protocol == IPPROTO_ICMP)
 		printk(" type:code=%d:%d",
 		       ((struct icmphdr*)((caddr_t)ip + (ip->ihl << 2)))->type,
 		       ((struct icmphdr*)((caddr_t)ip + (ip->ihl << 2)))->code);
+	if(ip->protocol == IPPROTO_TCP) {
+		printk(" seq=%u ack=%u", tcphdr->seq, tcphdr->ack_seq);
+		if (tcphdr->fin) printk(" FIN");
+		if (tcphdr->syn) printk(" SYN");
+		if (tcphdr->rst) printk(" RST");
+		if (tcphdr->psh) printk(" PSH");
+		if (tcphdr->ack) printk(" ACK");
+		if (tcphdr->urg) printk(" URG");
+		if (tcphdr->ece) printk(" ECE");
+		if (tcphdr->cwr) printk(" CWR");
+	}
 	printk("\n");
 
 	if(sysctl_ipsec_debug_verbose) {
@@ -369,7 +362,7 @@ ipsec_adjust_mss(struct sk_buff *skb, struct tcphdr *tcph, u_int16_t mtu)
 }
 #endif	/* MSS_HACK */
 
-DEBUG_NO_STATIC char *
+DEBUG_NO_STATIC const char *
 ipsec_xmit_err(int err)
 {
 	static char tmp[32];
@@ -475,7 +468,8 @@ ipsec_xmit_sanity_check_skb(struct ipsec_xmit_state *ixs)
 			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 				    "klips_error:ipsec_xmit_sanity_check_skb: "
 				    "skb_cow failed to allocate buffer, dropping.\n" );
-			ixs->stats->tx_dropped++;
+			if (ixs->stats)
+				ixs->stats->tx_dropped++;
 			return IPSEC_XMIT_ERRSKBALLOC;
 		}
 	}
@@ -488,7 +482,8 @@ ipsec_xmit_sanity_check_skb(struct ipsec_xmit_state *ixs)
 			    "klips_debug:ipsec_xmit_sanity_check_skb: "
 			    "found IP Version %d but cannot process other IP versions than v4.\n",
 			    ixs->iph->version); /* XXX */
-		ixs->stats->tx_dropped++;
+		if (ixs->stats)
+			ixs->stats->tx_dropped++;
 		return IPSEC_XMIT_NOIPV6;
 	}
 	
@@ -497,7 +492,8 @@ ipsec_xmit_sanity_check_skb(struct ipsec_xmit_state *ixs)
 		KLIPS_PRINT(debug_tunnel,
 			    "klips_debug:ipsec_xmit_sanity_check_skb: "
 			    "cannot process IP header options yet.  May be mal-formed packet.\n"); /* XXX */
-		ixs->stats->tx_dropped++;
+		if (ixs->stats)
+			ixs->stats->tx_dropped++;
 		return IPSEC_XMIT_NOIPOPTIONS;
 	}
 #endif /* IPSEC_DISALLOW_IPOPTIONS */
@@ -509,7 +505,8 @@ ipsec_xmit_sanity_check_skb(struct ipsec_xmit_state *ixs)
 
 		KLIPS_PRINT(debug_tunnel, "klips_debug:ipsec_xmit_sanity_check_skb: "
 			    "TTL=0, too many hops!\n");
-		ixs->stats->tx_dropped++;
+		if (ixs->stats)
+			ixs->stats->tx_dropped++;
 		return IPSEC_XMIT_TTLEXPIRED;
 	}
 #endif /* !NET_21 */
@@ -563,7 +560,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 				ixs->headroom += ESP_HEADER_LEN + 16 /* ivsize */;
 				break;
 			default:
-				ixs->stats->tx_errors++;
+				if (ixs->stats)
+					ixs->stats->tx_errors++;
 				return IPSEC_XMIT_ESP_BADALG;
 			}
 		} else
@@ -575,7 +573,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 		} else
 #endif /* CONFIG_KLIPS_ALG */
 		{
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			return IPSEC_XMIT_ESP_BADALG;
 		}
 #ifdef CONFIG_KLIPS_OCF
@@ -611,7 +610,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 		case AH_NONE:
 			break;
 		default:
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			return IPSEC_XMIT_ESP_BADALG;
 		}		
 		ixs->tailroom += ixs->blocksize != 1 ?
@@ -631,7 +631,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 		break;
 #endif /* CONFIG_KLIPS_IPCOMP */
 	default:
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_BADPROTO;
 	}
 	
@@ -644,7 +645,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 		       "klips_error:ipsec_xmit_encap_once: "
 		       "tried to skb_push headroom=%d, %d available.  This should never happen, please report.\n",
 		       ixs->headroom, skb_headroom(ixs->skb));
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_ESP_PUSHPULLERR;
 	}
 
@@ -655,7 +657,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 		       "klips_error:ipsec_xmit_encap_once: "
 		       "tried to skb_put %d, %d available.  This should never happen, please report.\n",
 		       ixs->tailroom, skb_tailroom(ixs->skb));
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_ESP_PUSHPULLERR;
 	}
 	skb_put(ixs->skb, ixs->tailroom);
@@ -668,7 +671,8 @@ ipsec_xmit_encap_init(struct ipsec_xmit_state *ixs)
 		printk(KERN_WARNING "klips_error:ipsec_xmit_encap_once: "
 		       "tot_len (%d) > 65520.  This should never happen, please report.\n",
 		       ixs->len);
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_BADLEN;
 	}
 	memmove((void *)ixs->dat, (void *)(ixs->dat + ixs->headroom), ixs->iphlen);
@@ -708,7 +712,8 @@ ipsec_xmit_encap_select(struct ipsec_xmit_state *ixs)
 		break;
 #endif
 	default:
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_BADPROTO;
 	}
 	return IPSEC_XMIT_OK;
@@ -746,13 +751,18 @@ ipsec_xmit_esp(struct ipsec_xmit_state *ixs)
 	ixs->iph->protocol = IPPROTO_ESP;
 	
 #ifdef CONFIG_KLIPS_OCF
-	if (ixs->ipsp->ocf_in_use)
+	if (ixs->ipsp->ocf_in_use) {
+		/* handle the IV code here for now,  near the similar code below */
+		prng_bytes(&ipsec_prng,
+			   (char *)ixs->espp->esp_iv, ixs->ipsp->ips_iv_size);
 		return(ipsec_ocf_xmit(ixs));
+	}
 #endif
 
 #ifdef CONFIG_KLIPS_ALG
 	if (!ixs->ixt_e) {
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_ESP_BADALG;
 	}
 	
@@ -805,7 +815,8 @@ ipsec_xmit_esp_ah(struct ipsec_xmit_state *ixs)
 #ifdef CONFIG_KLIPS_OCF
 	if (ixs->ipsp->ocf_in_use) {
 		/* we should never be here using OCF */
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_AH_BADALG;
 	} else
 #endif
@@ -858,7 +869,8 @@ ipsec_xmit_esp_ah(struct ipsec_xmit_state *ixs)
 	case AH_NONE:
 		break;
 	default:
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_AH_BADALG;
 	}
 	return IPSEC_XMIT_OK;
@@ -962,7 +974,8 @@ ipsec_xmit_ah(struct ipsec_xmit_state *ixs)
 		break;
 #endif /* CONFIG_KLIPS_AUTH_HMAC_SHA1 */
 	default:
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		return IPSEC_XMIT_AH_BADALG;
 	}
 	return IPSEC_XMIT_OK;
@@ -998,11 +1011,13 @@ ipsec_xmit_ipip(struct ipsec_xmit_state *ixs)
 	ixs->iph->protocol = IPPROTO_IPIP;
 	ixs->iph->ihl      = sizeof(struct iphdr) >> 2;
 
-	KLIPS_IP_SELECT_IDENT(ixs->iph, ixs->skb);
-
 	ixs->newdst = (__u32)ixs->iph->daddr;
 	ixs->newsrc = (__u32)ixs->iph->saddr;
 	
+	/* newer kernels require skb->dst to be set in KLIPS_IP_SELECT_IDENT */
+	/* we need to do this before any HASH generation is done */
+	KLIPS_IP_SELECT_IDENT(ixs->iph, ixs->skb);
+
 #ifdef NET_21
 	skb_set_transport_header(ixs->skb, ipsec_skb_offset(ixs->skb, ip_hdr(ixs->skb)));
 #endif /* NET_21 */
@@ -1073,8 +1088,16 @@ ipsec_xmit_cont(struct ipsec_xmit_state *ixs)
 #else /* NET_21 */
 	ixs->skb->ip_hdr = ixs->skb->h.iph = (struct iphdr *) ixs->skb->data;
 #endif /* NET_21 */
-	ixs->iph->check = 0;
-	ixs->iph->check = ip_fast_csum((unsigned char *)ixs->iph, ixs->iph->ihl);
+
+	/*
+	 * if we have more work to do,  it's likely this checksum is getting
+	 * encapsulated,  and we must do it.  Otherwise,  we do a final one
+	 * just before the ip_send/nf hook in ipsec_xmit_send.
+	 */
+	if (ixs->ipsp->ips_next) {
+		ixs->iph->check = 0;
+		ixs->iph->check = ip_fast_csum((unsigned char *)ixs->iph, ixs->iph->ihl);
+	}
 			
 	KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 		    "klips_debug:ipsec_xmit_encap_once: "
@@ -1270,7 +1293,8 @@ ipsec_xmit_init1(struct ipsec_xmit_state *ixs)
 			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 				    "klips_debug:ipsec_xmit_encap_bundle: "
 				    "shunt SA of DROP or no eroute: dropping.\n");
-			ixs->stats->tx_dropped++;
+			if (ixs->stats)
+				ixs->stats->tx_dropped++;
 			break;
 				
 		case SPI_REJECT:
@@ -1282,7 +1306,8 @@ ipsec_xmit_init1(struct ipsec_xmit_state *ixs)
 				  ICMP_PKT_FILTERED,
 				  0,
 				  ixs->physdev);
-			ixs->stats->tx_dropped++;
+			if (ixs->stats)
+				ixs->stats->tx_dropped++;
 			break;
 				
 		case SPI_PASS:
@@ -1298,7 +1323,8 @@ ipsec_xmit_init1(struct ipsec_xmit_state *ixs)
 			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 				    "klips_debug:ipsec_xmit_encap_bundle: "
 				    "shunt SA of HOLD: this does not make sense here, dropping.\n");
-			ixs->stats->tx_dropped++;
+			if (ixs->stats)
+				ixs->stats->tx_dropped++;
 			break;
 
 		case SPI_TRAP:
@@ -1384,7 +1410,8 @@ ipsec_xmit_init1(struct ipsec_xmit_state *ixs)
 				/* whether or not the above succeeded, we continue */
 				
 			}
-			ixs->stats->tx_dropped++;
+			if (ixs->stats)
+				ixs->stats->tx_dropped++;
 		}
 		default:
 			/* XXX what do we do with an unknown shunt spi? */
@@ -1463,7 +1490,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 				    "ipsec_sa in dead state for SA:<%s%s%s> %s, can no longer be used, dropping packet.\n",
 				    IPS_XFORM_NAME(ixs->ipsp),
 				    ixs->sa_len ? ixs->sa_txt : " (error)");
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_SAIDNOTLIVE;
 			goto cleanup;
 		}
@@ -1477,7 +1505,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 				    IPS_XFORM_NAME(ixs->ipsp),
 				    ixs->sa_len ? ixs->sa_txt : " (error)");
 			ipsec_sa_rm(ixs->ipsp);
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_REPLAYROLLED;
 			goto cleanup;
 		}
@@ -1505,7 +1534,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 					ipsec_life_countbased, ipsec_outgoing, ixs->ipsp) == ipsec_life_harddied) {
 				
 			ipsec_sa_rm(ixs->ipsp);
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_LIFETIMEFAILED;
 			goto cleanup;
 		}
@@ -1542,7 +1572,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 					ixs->headroom += ESP_HEADER_LEN + 16 /* ivsize */;
 					break;
 				default:
-					ixs->stats->tx_errors++;
+					if (ixs->stats)
+						ixs->stats->tx_errors++;
 					bundle_stat = IPSEC_XMIT_ESP_BADALG;
 					goto cleanup;
 				}
@@ -1555,7 +1586,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 			} else
 #endif /* CONFIG_KLIPS_ALG */
 			{
-				ixs->stats->tx_errors++;
+				if (ixs->stats)
+					ixs->stats->tx_errors++;
 				bundle_stat = IPSEC_XMIT_ESP_BADALG;
 				goto cleanup;
 			}
@@ -1590,7 +1622,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 			case AH_NONE:
 				break;
 			default:
-				ixs->stats->tx_errors++;
+				if (ixs->stats)
+					ixs->stats->tx_errors++;
 				bundle_stat = IPSEC_XMIT_AH_BADALG;
 				goto cleanup;
 			}			
@@ -1648,7 +1681,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 			break;
 #endif /* CONFIG_KLIPS_IPCOMP */
 		default:
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_BADPROTO;
 			goto cleanup;
 		}
@@ -1723,7 +1757,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 				  ICMP_FRAG_NEEDED,
 				  ixs->cur_mtu,
 				  ixs->physdev);
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		bundle_stat = IPSEC_XMIT_CANNOTFRAG;
 		goto cleanup;
 #else /* IPSEC_obey_DF */
@@ -1754,7 +1789,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 				printk(KERN_WARNING
 				       "klips_warning:ipsec_xmit_init2: "
 				       "ipsec_adjust_mss() failed\n");
-				ixs->stats->tx_errors++;
+				if (ixs->stats)
+					ixs->stats->tx_errors++;
 				bundle_stat = IPSEC_XMIT_MSSERR;
 				goto cleanup;
 			}
@@ -1793,7 +1829,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 			printk(KERN_WARNING "klips_debug:ipsec_xmit_init2: "
 			       "Failed, tried to allocate %d bytes for temp hard_header.\n", 
 			       ixs->hard_header_len);
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_ERRMEMALLOC;
 			goto cleanup;
 		}
@@ -1807,7 +1844,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 			printk(KERN_WARNING "klips_error:ipsec_xmit_init2: "
 			       "tried to skb_pull hhlen=%d, %d available.  This should never happen, please report.\n",
 			       ixs->hard_header_len, (int)(ixs->skb->len));
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_ESP_PUSHPULLERR;
 			goto cleanup;
 		}
@@ -1863,7 +1901,8 @@ ipsec_xmit_init2(struct ipsec_xmit_state *ixs)
 			       "klips_debug:ipsec_xmit_init2: "
 			       "Failed, tried to allocate %d head and %d tailroom\n", 
 			       ixs->max_headroom, ixs->max_tailroom);
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_ERRSKBALLOC;
 			goto cleanup;
 		}
@@ -1927,6 +1966,17 @@ static inline int ipsec_xmit_send2(struct sk_buff *skb)
 	return ip_send(skb);
 #endif
 }
+
+static inline int ipsec_xmit_send2_mast(struct sk_buff *skb)
+{
+#ifdef NETDEV_25	/* 2.6 kernels */
+	/* prevent recursion through the saref route */
+	if(skb->nfmark & 0x80000000)
+		skb->nfmark = 0;
+#endif
+	return ipsec_xmit_send2(skb);
+
+}
 #endif /* NETDEV_23 */
 
 #ifdef NAT_TRAVERSAL
@@ -1949,7 +1999,8 @@ enum ipsec_xmit_value ipsec_nat_encap(struct ipsec_xmit_state *ixs)
 				"This should never happen, please report.\n",
 				ixs->natt_head,
 				skb_tailroom(ixs->skb));
-			ixs->stats->tx_errors++;
+			if (ixs->stats)
+				ixs->stats->tx_errors++;
 			return IPSEC_XMIT_ESPUDP;
 		}
 		skb_put(ixs->skb, ixs->natt_head);
@@ -1996,12 +2047,17 @@ enum ipsec_xmit_value
 ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl)
 {
 	int error;
-  
+	int is_mast_packet;
+
+	/* check if this packet is sent from the mast, before we route */
+	is_mast_packet = ipsec_is_mast_device(ixs->skb->dev);
+
 #ifdef NETDEV_25
 	fl->nl_u.ip4_u.daddr = ip_hdr(ixs->skb)->daddr;
 	fl->nl_u.ip4_u.saddr = ixs->pass ? 0 : ip_hdr(ixs->skb)->saddr;
 	fl->nl_u.ip4_u.tos = RT_TOS(ip_hdr(ixs->skb)->tos);
 	fl->proto = ip_hdr(ixs->skb)->protocol;
+	fl->mark = ixs->skb->mark;
 #if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,24)
 	error = ip_route_output_key(&ixs->route, fl);
 #else
@@ -2018,7 +2074,8 @@ ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl)
                                     /* mcr->rgb: should this be 0 instead? */
 				    ixs->physdev->ifindex))) {
 #endif
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 			    "klips_debug:ipsec_xmit_send: "
 			    "ip_route_output failed with error code %d, dropped\n",
@@ -2029,7 +2086,8 @@ ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl)
 	if(ixs->dev == ixs->route->u.dst.dev) {
 		ip_rt_put(ixs->route);
 		/* This is recursion, drop it. */
-		ixs->stats->tx_errors++;
+		if (ixs->stats)
+			ixs->stats->tx_errors++;
 		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 			    "klips_debug:ipsec_xmit_send: "
 			    "suspect recursion, dev=rt->u.dst.dev=%s, dropped\n",
@@ -2037,8 +2095,8 @@ ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl)
 		return IPSEC_XMIT_RECURSDETECT;
 	}
 
-	dst_release(ixs->skb->dst);
-	ixs->skb->dst = &ixs->route->u.dst;
+	skb_dst_drop(ixs->skb);
+	skb_dst_set(ixs->skb, &ixs->route->u.dst);
 	if(ixs->stats) {
 		ixs->stats->tx_bytes += ixs->skb->len;
 	}
@@ -2059,6 +2117,11 @@ ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl)
 		ipsec_nf_reset(ixs->skb);
 	}
 
+	/* fix up the checksum after changes to the header */
+	ip_hdr(ixs->skb)->check = 0;
+	ip_hdr(ixs->skb)->check =
+		ip_fast_csum((unsigned char *)ip_hdr(ixs->skb), ip_hdr(ixs->skb)->ihl);
+
 	KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 		    "klips_debug:ipsec_xmit_send: "
 		    "...done, calling ip_send() on device:%s\n",
@@ -2067,10 +2130,16 @@ ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl)
 #ifdef NETDEV_23	/* 2.4 kernels */
 	{
 		int err;
+		if (is_mast_packet)
+			/* skip filtering on mast devices, since it resets our
+			 * route, nfmark, and causes nasty reentrancy. */
+			err = ipsec_xmit_send2_mast(ixs->skb);
+
+		else
+			err = NF_HOOK(PF_INET, NF_INET_LOCAL_OUT, ixs->skb, NULL,
+					ixs->route->u.dst.dev,
+					ipsec_xmit_send2);
 
-		err = NF_HOOK(PF_INET, NF_INET_LOCAL_OUT, ixs->skb, NULL,
-			      ixs->route->u.dst.dev,
-			      ipsec_xmit_send2);
 		if(err != NET_XMIT_SUCCESS && err != NET_XMIT_CN) {
 			if(net_ratelimit())
 				printk(KERN_ERR
@@ -2170,6 +2239,7 @@ void
 ipsec_xsm(struct ipsec_xmit_state *ixs)
 {
 	enum ipsec_xmit_value stat = IPSEC_XMIT_ENCAPFAIL;
+	unsigned more_allowed;
 
 	if (ixs == NULL) {
 		KLIPS_PRINT(debug_tunnel, "klips_debug:ipsec_xsm: ixs == NULL.\n");
@@ -2183,10 +2253,12 @@ ipsec_xsm(struct ipsec_xmit_state *ixs)
 
 	/*
 	 * if we have a valid said,  then we must check it here to ensure it
-	 * hasn't gone away while we were waiting for a task to complete
+	 * hasn't gone away while we were waiting for a task to complete.
+	 *
+	 * If the said was found via saref in mast code, skip this check.
 	 */
 
-	if (ixs->ipsp) {
+	if (ixs->ipsp && !ixs->mast_mode) {
 		struct ipsec_sa *ipsp;
 		ipsp = ipsec_sa_getbyid(&ixs->outgoing_said, IPSEC_REFTX);
 		if (unlikely(ipsp == NULL)) {
@@ -2207,7 +2279,8 @@ ipsec_xsm(struct ipsec_xmit_state *ixs)
 		}
 	}
 
-	while (ixs->state != IPSEC_XSM_DONE) {
+	more_allowed = 1000;
+	while (ixs->state != IPSEC_XSM_DONE && --more_allowed) {
 
 		ixs->next_state = xmit_state_table[ixs->state].next_state;
 
