@@ -66,6 +66,8 @@
 #endif
 
 #include <linux/types.h>
+
+#include "openswan/ipsec_param2.h"
  
 #include <openswan.h>
 
@@ -77,16 +79,17 @@
 #include <openswan/pfkey.h>
 
 #include "openswan/ipsec_proto.h"
-#include "openswan/ipsec_kern24.h"
+#include "openswan/ipsec_kversion.h"
 #include "openswan/ipsec_sysctl.h"
 
 #define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
-#if 0
 #ifndef SOCKOPS_WRAPPED
 #define SOCKOPS_WRAPPED(name) name
 #endif /* SOCKOPS_WRAPPED */
-#endif
+#ifndef SOCKOPS_WRAP
+#define	SOCKOPS_WRAP(name, fam)
+#endif /* SOCKOPS_WRAP */
 
 extern struct proto_ops SOCKOPS_WRAPPED(pfkey_ops);
 
@@ -492,7 +495,7 @@ pfkey_destroy_socket(struct sock *sk)
 			printk(" h:0p%p", skb_transport_header(skb));
 			printk(" nh:0p%p", skb_network_header(skb));
 			printk(" mac:0p%p", skb_mac_header(skb));
-			printk(" dst:0p%p", skb->dst);
+			printk(" dst:0p%p", skb_dst(skb));
 			if(sysctl_ipsec_debug_verbose) {
 				int i;
 				
@@ -675,7 +678,7 @@ pfkey_create(struct socket *sock, int protocol)
 	if(!capable(CAP_NET_ADMIN)) {
 		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_create: "
-			    "must be root to open pfkey sockets.\n");
+			    "must be root (or have net_admin capability) to open pfkey sockets.\n");
 		return -EACCES;
 	}
 
@@ -716,7 +719,7 @@ pfkey_create(struct socket *sock, int protocol)
 	sk->sk_family = PF_KEY;
 /*	sk->num = protocol; */
 	sk->sk_protocol = protocol;
-	key_pid(sk) = current->pid;
+	key_pid(sk) = current_uid();
 	KLIPS_PRINT(debug_pfkey,
 		    "klips_debug:pfkey_create: "
 		    "sock->fasync_list=0p%p sk->sleep=0p%p.\n",
@@ -875,7 +878,7 @@ pfkey_sendmsg(struct socket *sock, struct msghdr *msg, int len, struct scm_cooki
 	if(!capable(CAP_NET_ADMIN)) {
 		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_sendmsg: "
-			    "must be root to send messages to pfkey sockets.\n");
+			    "must be root (or have net_admin capability) to send messages to pfkey sockets.\n");
 		SENDERR(EACCES);
 	}
 
