@@ -155,9 +155,6 @@ pfkey_sa_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* extr)
 #ifdef CONFIG_KLIPS_OCF
 		if (ipsec_ocf_sa_init(ipsp, ipsp->ips_authalg, ipsp->ips_encalg))
 		    break;
-#endif
-#ifdef CONFIG_KLIPS_ALG
-		ipsec_alg_sa_init(ipsp);
 #endif
 		break;
 	case IPPROTO_IPIP:
@@ -716,7 +713,7 @@ errlab:
 }
 
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+#ifdef NAT_TRAVERSAL
 int
 pfkey_x_nat_t_type_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* extr)
 {
