@@ -13,8 +13,6 @@
  * for more details.
  */
 
-char addconn_c_version[] = "RCSID $Id: spi.c,v 1.114 2005/08/18 14:04:40 ken Exp $";
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
@@ -139,7 +137,7 @@ main(int argc, char *argv[])
     while((opt = getopt_long(argc, argv, "", longopts, 0)) != EOF) {
 	switch(opt) {
 	case 'h':
-	    //usage:
+	    /* usage: */
 	    usage();
 	    break;
 
@@ -261,9 +259,10 @@ main(int argc, char *argv[])
 	char b[ADDRTOT_BUF];
 	e = ttoaddr(defaultroute, strlen(defaultroute), AF_INET, &cfg->dr);
 	if(e) {
-	    printf("invalid default route: %s\n", e);
-	    exit(4);
-	}
+	    printf("ignoring invalid defaultroute: %s\n", e);
+	    defaultroute = NULL;
+	    /* exit(4); */
+	} else
 
 	if(verbose) {
 	    addrtot(&cfg->dr, 0, b, sizeof(b));
@@ -276,9 +275,10 @@ main(int argc, char *argv[])
 	char b[ADDRTOT_BUF];
 	e = ttoaddr(defaultnexthop, strlen(defaultnexthop), AF_INET, &cfg->dnh);
 	if(e) {
-	    printf("invalid default route: %s\n", e);
-	    exit(4);
-	}
+	    printf("ignoring invalid defaultnexthop: %s\n", e);
+	    defaultnexthop = NULL;
+	    /* exit(4); */
+	} else
 
 	if(verbose) {
 	    addrtot(&cfg->dnh, 0, b, sizeof(b));
