@@ -150,11 +150,17 @@ main(int argc, char **argv)
 			em_db_tn=em_db_nl=em_db_xf=em_db_er=em_db_sp=0;
 			em_db_rj=em_db_es=em_db_ah=em_db_rx=em_db_ky=0;
 			em_db_gz=em_db_vb=0;
-			if(strcmp(optarg, "tunnel") == 0) {
+			if(strcmp(optarg, "all") == 0) {
+				em_db_tn=em_db_nl=em_db_xf=em_db_er=em_db_sp=-1;
+				em_db_rj=em_db_es=em_db_ah=em_db_rx=em_db_ky=-1;
+				em_db_gz=-1;
+				em_db_vb= 0;
+			} else if(strcmp(optarg, "tunnel") == 0) {
 				em_db_tn = -1L;
 			} else if(strcmp(optarg, "tncfg") == 0) {
 			        em_db_tn = DB_TN_REVEC;
-			} else if(strcmp(optarg, "tunnel-xmit") == 0) {
+			} else if(strcmp(optarg, "xmit") == 0
+					|| strcmp(optarg, "tunnel-xmit") == 0) {
 				em_db_tn = DB_TN_XMIT;
 			} else if(strcmp(optarg, "netlink") == 0) {
 				em_db_nl = -1L;
@@ -179,6 +185,8 @@ main(int argc, char **argv)
 			} else if(strcmp(optarg, "verbose") == 0) {
 				em_db_vb = -1L;
 			} else {
+				fprintf(stdout, "%s: unknown set argument '%s'\n",
+						program_name, optarg);
 				usage(program_name);
 			}
 			em_db_nl |= 1 << (sizeof(em_db_nl) * 8 -1);
@@ -194,7 +202,11 @@ main(int argc, char **argv)
 			em_db_gz=em_db_vb=-1;
 
 			action = 'c';
-			if(strcmp(optarg, "tunnel") == 0) {
+			if(strcmp(optarg, "all") == 0) {
+				em_db_tn=em_db_nl=em_db_xf=em_db_er=em_db_sp=0;
+				em_db_rj=em_db_es=em_db_ah=em_db_rx=em_db_ky=0;
+				em_db_gz=em_db_vb=0;
+			} else if(strcmp(optarg, "tunnel") == 0) {
 				em_db_tn = 0;
 			} else if(strcmp(optarg, "tunnel-xmit") == 0
 				  || strcmp(optarg, "xmit") == 0) {
@@ -222,6 +234,8 @@ main(int argc, char **argv)
 			} else if(strcmp(optarg, "verbose") == 0) {
 				em_db_vb = 0;
 			} else {
+				fprintf(stdout, "%s: unknown clear argument '%s'\n",
+						program_name, optarg);
 				usage(program_name);
 			}
 			em_db_nl &= ~(1 << (sizeof(em_db_nl) * 8 -1));
@@ -272,6 +286,8 @@ main(int argc, char **argv)
 			/* no return on error */
 			break;
 		default:
+			fprintf(stdout, "%s: unknown option '%s'\n",
+					program_name, argv[optind]);
 			break;
 		}
 		previous = c;
