@@ -58,8 +58,9 @@
 # include <keyhi.h>
 # include "oswconf.h"
 
-#define PK11_Derive(base, mechanism, param, target, operation, keysize)  \
-	PK11_Derive_osw(base, mechanism, param, target, operation, keysize)
+/* #define PK11_Derive(base, mechanism, param, target, operation, keysize) \
+ *	PK11_Derive_osw(base, mechanism, param, target, operation, keysize)
+ */
 
 static PK11SymKey *pk11_extract_derive_wrapper_osw(PK11SymKey *base, CK_EXTRACT_PARAMS bs
 		, CK_MECHANISM_TYPE target , CK_ATTRIBUTE_TYPE operation, int keySize)
@@ -68,7 +69,7 @@ static PK11SymKey *pk11_extract_derive_wrapper_osw(PK11SymKey *base, CK_EXTRACT_
       param.data = (unsigned char*)&bs;
       param.len = sizeof (bs);
 
-    return PK11_Derive(base, CKM_EXTRACT_KEY_FROM_KEY, &param, target, operation, keySize);
+    return PK11_Derive_osw(base, CKM_EXTRACT_KEY_FROM_KEY, &param, target, operation, keySize);
 }
 /*
 static CK_MECHANISM_TYPE nss_hmac_mech(const struct hash_desc *hasher)
@@ -308,7 +309,7 @@ skeyid_preshared(const chunk_t pss
         buf2[k] ^= HMAC_OPAD;
     }
 
-    //pfree(nir.ptr);
+    /* pfree(nir.ptr); */
 
     mechanism=nss_key_derivation_mech(hasher);
     buf1_chunk.ptr=buf1;
@@ -318,23 +319,23 @@ skeyid_preshared(const chunk_t pss
     buf2_chunk.len=HMAC_BUFSIZE;
 
     PK11SymKey *tkey4 = pk11_derive_wrapper_osw(shared, CKM_CONCATENATE_DATA_AND_BASE, buf1_chunk, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
-    //nss_symkey_log(tkey4, "pss+ipad+shared");
+    /* nss_symkey_log(tkey4, "pss+ipad+shared"); */
 
     CK_EXTRACT_PARAMS bs=0;
     PK11SymKey *tkey5 = pk11_extract_derive_wrapper_osw(tkey4, bs, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, HMAC_BUFSIZE);
-    //nss_symkey_log(tkey5, "pss+ipad");
+    /* nss_symkey_log(tkey5, "pss+ipad"); */
 
     PK11SymKey *tkey6 = pk11_derive_wrapper_osw(tkey5, CKM_CONCATENATE_BASE_AND_DATA, nir, mechanism, CKA_DERIVE, 0);
     pfree(nir.ptr);
-    //nss_symkey_log(tkey6, "pss+ipad+nir");
+    /* nss_symkey_log(tkey6, "pss+ipad+nir"); */
 
-    //PK11SymKey *tkey1 = pk11_derive_wrapper_osw(shared, CKM_CONCATENATE_DATA_AND_BASE, buf1_chunk, mechanism, CKA_DERIVE, 0);
-    PK11SymKey *tkey2 = PK11_Derive(tkey6, mechanism, NULL, CKM_CONCATENATE_DATA_AND_BASE, CKA_DERIVE, 0);
-    //nss_symkey_log(tkey2, "pss : tkey2");
+    /* PK11SymKey *tkey1 = pk11_derive_wrapper_osw(shared, CKM_CONCATENATE_DATA_AND_BASE, buf1_chunk, mechanism, CKA_DERIVE, 0); */
+    PK11SymKey *tkey2 = PK11_Derive_osw(tkey6, mechanism, NULL, CKM_CONCATENATE_DATA_AND_BASE, CKA_DERIVE, 0);
+    /* nss_symkey_log(tkey2, "pss : tkey2"); */
 
     PK11SymKey *tkey3 = pk11_derive_wrapper_osw(tkey2, CKM_CONCATENATE_DATA_AND_BASE, buf2_chunk, mechanism, CKA_DERIVE, 0);
-    skeyid = PK11_Derive(tkey3, mechanism, NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
-    //nss_symkey_log(tkey2, "pss : tkey3");
+    skeyid = PK11_Derive_osw(tkey3, mechanism, NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    /* nss_symkey_log(tkey2, "pss : tkey3"); */
 
     skeyid_chunk->len = sizeof(PK11SymKey *);
     skeyid_chunk->ptr = alloc_bytes(skeyid_chunk->len, "calculated skeyid(pss)");
@@ -434,9 +435,9 @@ skeyid_digisig(const chunk_t ni
     buf2_chunk.len=HMAC_BUFSIZE;
 
     PK11SymKey *tkey1 = pk11_derive_wrapper_osw(shared, CKM_CONCATENATE_DATA_AND_BASE, buf1_chunk, mechanism, CKA_DERIVE, 0);
-    PK11SymKey *tkey2 = PK11_Derive(tkey1, mechanism, NULL, CKM_CONCATENATE_DATA_AND_BASE, CKA_DERIVE, 0);
+    PK11SymKey *tkey2 = PK11_Derive_osw(tkey1, mechanism, NULL, CKM_CONCATENATE_DATA_AND_BASE, CKA_DERIVE, 0);
     PK11SymKey *tkey3 = pk11_derive_wrapper_osw(tkey2, CKM_CONCATENATE_DATA_AND_BASE, buf2_chunk, mechanism, CKA_DERIVE, 0);
-    skeyid = PK11_Derive(tkey3, mechanism, NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    skeyid = PK11_Derive_osw(tkey3, mechanism, NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
 
     skeyid_chunk->len = sizeof(PK11SymKey *);
     skeyid_chunk->ptr = alloc_bytes(skeyid_chunk->len, "calculated skeyid");
@@ -560,7 +561,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
     param.data=(unsigned char *) &keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey3 = PK11_Derive(tkey2, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    PK11SymKey *tkey3 = PK11_Derive_osw(tkey2, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(tkey3!=NULL);
 
     PK11SymKey *tkey4 = pk11_derive_wrapper_osw(tkey3, CKM_CONCATENATE_BASE_AND_DATA
@@ -577,7 +578,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
 
     PR_ASSERT(tkey6!=NULL);
 
-    PK11SymKey *tkey7 = PK11_Derive(tkey6, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    PK11SymKey *tkey7 = PK11_Derive_osw(tkey6, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(tkey7!=NULL);
 
     PK11SymKey *tkey8 = pk11_derive_wrapper_osw(tkey1, CKM_XOR_BASE_AND_DATA
@@ -589,10 +590,10 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey9 = PK11_Derive(tkey8, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+    PK11SymKey *tkey9 = PK11_Derive_osw(tkey8, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
     PR_ASSERT(tkey9!=NULL);
 
-    skeyid_d = PK11_Derive(tkey9, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    skeyid_d = PK11_Derive_osw(tkey9, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(skeyid_d!=NULL);
     nss_symkey_log(skeyid_d, "skeyid_d");
      /*****End of SKEYID_d derivation***************************************/
@@ -603,14 +604,14 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey10 = PK11_Derive(tkey2, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+    PK11SymKey *tkey10 = PK11_Derive_osw(tkey2, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
     PR_ASSERT(tkey10!=NULL);
 
     keyhandle=PK11_GetSymKeyHandle(shared);
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey11 = PK11_Derive(tkey10, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    PK11SymKey *tkey11 = PK11_Derive_osw(tkey10, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(tkey11!=NULL);
 
     PK11SymKey *tkey12 = pk11_derive_wrapper_osw(tkey11, CKM_CONCATENATE_BASE_AND_DATA
@@ -625,17 +626,17 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                                                 , hmac_val1, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
     PR_ASSERT(tkey14!=NULL);
 
-    PK11SymKey *tkey15 = PK11_Derive(tkey14, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    PK11SymKey *tkey15 = PK11_Derive_osw(tkey14, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(tkey15!=NULL);
 
     keyhandle=PK11_GetSymKeyHandle(tkey15);
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey16 = PK11_Derive(tkey8, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+    PK11SymKey *tkey16 = PK11_Derive_osw(tkey8, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
     PR_ASSERT(tkey16!=NULL);
 
-    skeyid_a = PK11_Derive(tkey16, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    skeyid_a = PK11_Derive_osw(tkey16, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(skeyid_a!=NULL);
     nss_symkey_log(skeyid_a, "skeyid_a");
     /*****End of SKEYID_a derivation***************************************/
@@ -646,14 +647,14 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey17 = PK11_Derive(tkey2, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+    PK11SymKey *tkey17 = PK11_Derive_osw(tkey2, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
     PR_ASSERT(tkey17!=NULL);
 
     keyhandle=PK11_GetSymKeyHandle(shared);
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey18 = PK11_Derive(tkey17, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+    PK11SymKey *tkey18 = PK11_Derive_osw(tkey17, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
     PR_ASSERT(tkey18!=NULL);
 
     PK11SymKey *tkey19 = pk11_derive_wrapper_osw(tkey18, CKM_CONCATENATE_BASE_AND_DATA
@@ -668,14 +669,14 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                                                 , hmac_val2, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
     PR_ASSERT(tkey21!=NULL);
 
-    PK11SymKey *tkey22 = PK11_Derive(tkey21, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA , CKA_DERIVE, 0);
+    PK11SymKey *tkey22 = PK11_Derive_osw(tkey21, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA , CKA_DERIVE, 0);
     PR_ASSERT(tkey22!=NULL);
 
     keyhandle=PK11_GetSymKeyHandle(tkey22);
     param.data=(unsigned char*)&keyhandle;
     param.len=sizeof(keyhandle);
 
-    PK11SymKey *tkey23 = PK11_Derive(tkey8, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+    PK11SymKey *tkey23 = PK11_Derive_osw(tkey8, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
     PR_ASSERT(tkey23!=NULL);
 
     DBG(DBG_CRYPT, DBG_log("NSS: enc keysize=%d\n",(int)keysize));
@@ -690,7 +691,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
       param1.len = sizeof (bitstart);
 
        if(keysize <= hasher->hash_digest_len){
-       skeyid_e = PK11_Derive(tkey23, nss_key_derivation_mech(hasher), NULL, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
+       skeyid_e = PK11_Derive_osw(tkey23, nss_key_derivation_mech(hasher), NULL, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
        PR_ASSERT(skeyid_e!=NULL);
 
 
@@ -706,7 +707,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
         size_t i = 0;
        PK11SymKey *keymat;
 
-        skeyid_e = PK11_Derive(tkey23, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+        skeyid_e = PK11_Derive_osw(tkey23, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
         PR_ASSERT(skeyid_e!=NULL);
 
         PK11SymKey *tkey25 = pk11_derive_wrapper_osw(skeyid_e, CKM_CONCATENATE_BASE_AND_DATA
@@ -721,7 +722,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                                                 , hmac_zerobyte, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
         PR_ASSERT(tkey27!=NULL);
 
-        PK11SymKey *tkey28 = PK11_Derive(tkey27, nss_key_derivation_mech(hasher), NULL
+        PK11SymKey *tkey28 = PK11_Derive_osw(tkey27, nss_key_derivation_mech(hasher), NULL
                                          , CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
         PR_ASSERT(tkey28!=NULL);
 
@@ -733,10 +734,10 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
         param.data=(unsigned char*)&keyhandle;
         param.len=sizeof(keyhandle);
 
-        PK11SymKey *tkey30 = PK11_Derive(tkey29, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+        PK11SymKey *tkey30 = PK11_Derive_osw(tkey29, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
         PR_ASSERT(tkey30!=NULL);
 
-       PK11SymKey *tkey31 = PK11_Derive(tkey30, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+       PK11SymKey *tkey31 = PK11_Derive_osw(tkey30, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
        PR_ASSERT(tkey31!=NULL);
 
        keymat = tkey31;
@@ -761,10 +762,10 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                param.data=(unsigned char*)&keyhandle;
                param.len=sizeof(keyhandle);
 
-               PK11SymKey *tkey34 = PK11_Derive(tkey33, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+               PK11SymKey *tkey34 = PK11_Derive_osw(tkey33, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
                PR_ASSERT(tkey34!=NULL);
 
-               PK11SymKey *tkey35 = PK11_Derive(tkey34, nss_key_derivation_mech(hasher), NULL
+               PK11SymKey *tkey35 = PK11_Derive_osw(tkey34, nss_key_derivation_mech(hasher), NULL
                                          , CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
                PR_ASSERT(tkey35!=NULL);
 
@@ -772,10 +773,10 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                param.data=(unsigned char*)&keyhandle;
                param.len=sizeof(keyhandle);
 
-               PK11SymKey *tkey37 = PK11_Derive(tkey36, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+               PK11SymKey *tkey37 = PK11_Derive_osw(tkey36, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
                PR_ASSERT(tkey37!=NULL);
 
-               PK11SymKey *tkey38 = PK11_Derive(tkey37, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+               PK11SymKey *tkey38 = PK11_Derive_osw(tkey37, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
                PR_ASSERT(tkey38!=NULL);
 
                i += hasher->hash_digest_len;
@@ -787,7 +788,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                        param.data=(unsigned char*)&keyhandle;
                        param.len=sizeof(keyhandle);
 
-                       PK11SymKey *tkey39 = PK11_Derive(keymat, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
+                       PK11SymKey *tkey39 = PK11_Derive_osw(keymat, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
                        PR_ASSERT(tkey39!=NULL);
 
                        enc_key = PK11_DeriveWithFlags(tkey39, CKM_EXTRACT_KEY_FROM_KEY, &param1
@@ -822,7 +823,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
                        param.data=(unsigned char*)&keyhandle;
                        param.len=sizeof(keyhandle);
 
-                       PK11SymKey *tkey39=PK11_Derive(keymat,CKM_CONCATENATE_BASE_AND_KEY, &param,CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+                       PK11SymKey *tkey39=PK11_Derive_osw(keymat,CKM_CONCATENATE_BASE_AND_KEY, &param,CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
                        PR_ASSERT(tkey39!=NULL);
 
 
@@ -1235,11 +1236,15 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 	unsigned int keybytes;
 	unsigned char *kb;
 
-	//if(vpss.prf_hasher->hash_key_size == 0) {
+#if 0
+	if(vpss.prf_hasher->hash_key_size == 0) {
+#endif
 	keybytes = vpss.ni.len + vpss.nr.len;
-	//} else {
-	//keybytes = vpss.prf_hasher->hash_key_size;
-	//}
+#if 0
+	} else {
+	    keybytes = vpss.prf_hasher->hash_key_size;
+	}
+#endif
 
 	kb = alloc_bytes(keybytes, "skeyseed prf key");
 	memset(kb, 0, keybytes);
@@ -1321,7 +1326,7 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 		param.data=(unsigned char*)&keyhandle;
 		param.len=sizeof(keyhandle);
 
-		PK11SymKey *tkey12 = PK11_Derive(tkey2, CKM_CONCATENATE_BASE_AND_KEY
+		PK11SymKey *tkey12 = PK11_Derive_osw(tkey2, CKM_CONCATENATE_BASE_AND_KEY
 			, &param, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
 		PR_ASSERT(tkey12!=NULL);
 
@@ -1355,7 +1360,7 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 	   PR_ASSERT(tkey7!=NULL);
 
 
-	   PK11SymKey *tkey8 = PK11_Derive(tkey7, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
+	   PK11SymKey *tkey8 = PK11_Derive_osw(tkey7, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_DATA, CKA_DERIVE, 0);
 	   PR_ASSERT(tkey8!=NULL);
 
 
@@ -1368,19 +1373,19 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 	   param.data=(unsigned char*)&keyhandle;
 	   param.len=sizeof(keyhandle);
 
-	   PK11SymKey *tkey10 = PK11_Derive(tkey9, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
+	   PK11SymKey *tkey10 = PK11_Derive_osw(tkey9, CKM_CONCATENATE_BASE_AND_KEY, &param, nss_key_derivation_mech(hasher), CKA_DERIVE, 0);
 	   PR_ASSERT(tkey10!=NULL);
 
 
 	   if(vpss.counter[0]== 0x01) {
-		finalkey = PK11_Derive(tkey10, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+		finalkey = PK11_Derive_osw(tkey10, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
 		PR_ASSERT(finalkey!=NULL);
 
 
-		tkey11 = PK11_Derive(tkey10, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+		tkey11 = PK11_Derive_osw(tkey10, nss_key_derivation_mech(hasher), NULL, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
 		PR_ASSERT(tkey11!=NULL);
 	   } else {
-		tkey11 = PK11_Derive(tkey10, nss_key_derivation_mech(hasher), NULL, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
+		tkey11 = PK11_Derive_osw(tkey10, nss_key_derivation_mech(hasher), NULL, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
 		PR_ASSERT(tkey11!=NULL);
 
 
@@ -1389,10 +1394,10 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 		param.len=sizeof(keyhandle);
 
 		if( total_keysize <= (PK11_GetKeyLength(finalkey)+PK11_GetKeyLength(tkey11)) ) {
-		   finalkey = PK11_Derive(finalkey, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
+		   finalkey = PK11_Derive_osw(finalkey, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_EXTRACT_KEY_FROM_KEY, CKA_DERIVE, 0);
 		   PR_ASSERT(finalkey!=NULL);
 		} else {
-		   finalkey = PK11_Derive(finalkey, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
+		   finalkey = PK11_Derive_osw(finalkey, CKM_CONCATENATE_BASE_AND_KEY, &param, CKM_CONCATENATE_BASE_AND_KEY, CKA_DERIVE, 0);
 		   PR_ASSERT(finalkey!=NULL);
 		}
 	   }
