@@ -59,7 +59,20 @@ static MP_INT
     modp6144_modulus,
     modp8192_modulus;
 
-MP_INT groupgenerator;	/* MODP group generator (2) */
+#ifdef USE_MODP_RFC5114
+static MP_INT
+    dh22_modulus,
+    dh23_modulus,
+    dh24_modulus;
+#endif
+
+MP_INT groupgenerator;  /* MODP group generator (2) */
+
+#ifdef USE_MODP_RFC5114
+MP_INT generator_dh22,
+       generator_dh23,
+       generator_dh24;
+#endif
 
 #ifdef IKE_ALG
 
@@ -113,13 +126,14 @@ static struct hash_desc crypto_hasher_md5 =
     hash_ctx_size: sizeof(MD5_CTX),
     hash_key_size:   MD5_DIGEST_SIZE,
     hash_digest_len: MD5_DIGEST_SIZE,
+    hash_integ_len: 0,				/*Not applicable*/
     hash_init: (void (*)(void *)) osMD5Init,
     hash_update: (void (*)(void *, const u_int8_t *, size_t)) osMD5Update,
     hash_final: (void (*)(u_char *, void *)) osMD5Final,
 };
 
 static struct hash_desc crypto_integ_md5 =
-{ 	
+{ 
     common: {name: "oakley_md5",
 	     officname: "md5",
 	     algo_type: IKE_ALG_INTEG,
@@ -129,13 +143,14 @@ static struct hash_desc crypto_integ_md5 =
     hash_ctx_size: sizeof(MD5_CTX),
     hash_key_size:   MD5_DIGEST_SIZE,
     hash_digest_len: MD5_DIGEST_SIZE,
+    hash_integ_len: MD5_DIGEST_SIZE_96,	
     hash_init: (void (*)(void *)) osMD5Init,
     hash_update: (void (*)(void *, const u_int8_t *, size_t)) osMD5Update,
     hash_final: (void (*)(u_char *, void *)) osMD5Final,
 };
 
 static struct hash_desc crypto_hasher_sha1 =
-{ 	
+{ 
     common: {name: "oakley_sha",
 	     officname: "sha1",
 	     algo_type: IKE_ALG_HASH,
@@ -145,6 +160,7 @@ static struct hash_desc crypto_hasher_sha1 =
     hash_ctx_size: sizeof(SHA1_CTX),
     hash_key_size:   SHA1_DIGEST_SIZE,
     hash_digest_len: SHA1_DIGEST_SIZE,
+    hash_integ_len: 0,                          /*Not applicable*/
     hash_init: (void (*)(void *)) SHA1Init,
     hash_update: (void (*)(void *, const u_int8_t *, size_t)) SHA1Update,
     hash_final: (void (*)(u_char *, void *)) SHA1Final,
@@ -161,6 +177,7 @@ static struct hash_desc crypto_integ_sha1 =
     hash_ctx_size: sizeof(SHA1_CTX),
     hash_key_size:   SHA1_DIGEST_SIZE,
     hash_digest_len: SHA1_DIGEST_SIZE,
+    hash_integ_len: SHA1_DIGEST_SIZE_96,
     hash_init: (void (*)(void *)) SHA1Init,
     hash_update: (void (*)(void *, const u_int8_t *, size_t)) SHA1Update,
     hash_final: (void (*)(u_char *, void *)) SHA1Final,
@@ -170,6 +187,11 @@ void
 init_crypto(void)
 {
     if (mpz_init_set_str(&groupgenerator, MODP_GENERATOR, 10) != 0
+#ifdef USE_MODP_RFC5114
+    ||  mpz_init_set_str(&generator_dh22, MODP_GENERATOR_DH22, 16) != 0
+    ||  mpz_init_set_str(&generator_dh23, MODP_GENERATOR_DH23, 16) != 0
+    ||  mpz_init_set_str(&generator_dh24, MODP_GENERATOR_DH24, 16) != 0
+#endif
 #if defined(USE_VERYWEAK_DH1)	                        /* modp768 not sufficiently strong */
     || mpz_init_set_str(&modp768_modulus, MODP768_MODULUS, 16) != 0
 #endif
@@ -179,8 +201,15 @@ init_crypto(void)
     || mpz_init_set_str(&modp3072_modulus, MODP3072_MODULUS, 16) != 0
     || mpz_init_set_str(&modp4096_modulus, MODP4096_MODULUS, 16) != 0
     || mpz_init_set_str(&modp6144_modulus, MODP6144_MODULUS, 16) != 0
-    || mpz_init_set_str(&modp8192_modulus, MODP8192_MODULUS, 16) != 0)
+    || mpz_init_set_str(&modp8192_modulus, MODP8192_MODULUS, 16) != 0
+#ifdef USE_MODP_RFC5114
+    || mpz_init_set_str(&dh22_modulus, MODP1024_MODULUS_DH22, 16) != 0
+    || mpz_init_set_str(&dh23_modulus, MODP2048_MODULUS_DH23, 16) != 0
+    || mpz_init_set_str(&dh24_modulus, MODP2048_MODULUS_DH24, 16) != 0
+#endif
+      ) {
 	exit_log("mpz_init_set_str() failed in init_crypto()");
+     }
 #ifdef IKE_ALG
 	{
 #ifdef USE_TWOFISH
@@ -244,6 +273,7 @@ init_crypto(void)
  * See RFC2409 "The Internet key exchange (IKE)" 6.
  */
 
+#ifndef USE_MODP_RFC5114
 const struct oakley_group_desc unset_group = {0, NULL, 0};	/* magic signifier */
 
 const struct oakley_group_desc oakley_group[] = {
@@ -258,6 +288,26 @@ const struct oakley_group_desc oakley_group[] = {
     { OAKLEY_GROUP_MODP6144, &modp6144_modulus, BYTES_FOR_BITS(6144) },
     { OAKLEY_GROUP_MODP8192, &modp8192_modulus, BYTES_FOR_BITS(8192) },
 };
+#else
+const struct oakley_group_desc unset_group = {0, NULL, NULL, 0};      /* magic signifier */
+
+const struct oakley_group_desc oakley_group[] = {
+#if defined(USE_VERYWEAK_DH1)           /* modp768 not sufficiently strong */
+    { OAKLEY_GROUP_MODP768, &groupgenerator, &modp768_modulus, BYTES_FOR_BITS(768) },
+#endif
+    { OAKLEY_GROUP_MODP1024, &groupgenerator, &modp1024_modulus, BYTES_FOR_BITS(1024) },
+    { OAKLEY_GROUP_MODP1536, &groupgenerator, &modp1536_modulus, BYTES_FOR_BITS(1536) },
+    { OAKLEY_GROUP_MODP2048, &groupgenerator, &modp2048_modulus, BYTES_FOR_BITS(2048) },
+    { OAKLEY_GROUP_MODP3072, &groupgenerator, &modp3072_modulus, BYTES_FOR_BITS(3072) },
+    { OAKLEY_GROUP_MODP4096, &groupgenerator, &modp4096_modulus, BYTES_FOR_BITS(4096) },
+    { OAKLEY_GROUP_MODP6144, &groupgenerator, &modp6144_modulus, BYTES_FOR_BITS(6144) },
+    { OAKLEY_GROUP_MODP8192, &groupgenerator, &modp8192_modulus, BYTES_FOR_BITS(8192) },
+    { OAKLEY_GROUP_DH22, &generator_dh22, &dh22_modulus, BYTES_FOR_BITS(1024) },
+    { OAKLEY_GROUP_DH23, &generator_dh23, &dh23_modulus, BYTES_FOR_BITS(2048) },
+    { OAKLEY_GROUP_DH24, &generator_dh24, &dh24_modulus, BYTES_FOR_BITS(2048) },
+
+};
+#endif
 
 const unsigned int oakley_group_size = elemsof(oakley_group);
 
