@@ -54,11 +54,12 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <sys/types.h>
 
 #include <openswan.h>
 
+#include "sysdep.h"
 #include "constants.h"
 #include "defs.h"
 #include "state.h"
@@ -190,7 +191,9 @@ db_trans_expand(struct db_context *ctx, int delta_trans)
 	}
 	/* update elem count */
 	ctx->max_trans = max_trans;
-	PFREE_ST(old_trans, db_trans_st);
+	if(old_trans) {
+		PFREE_ST(old_trans, db_trans_st);
+	}
 	ret = 0;
 out:
 	return ret;
@@ -205,7 +208,7 @@ db_attrs_expand(struct db_context *ctx, int delta_attrs)
 	int ret = -1;
 	struct db_attr *new_attrs, *old_attrs;
 	struct db_trans *t;
-	int ti;
+	unsigned int ti;
 	int max_attrs = ctx->max_attrs + delta_attrs;
 	int offset;
 
@@ -242,7 +245,7 @@ db_attrs_expand(struct db_context *ctx, int delta_attrs)
 	}
 	/* update elem count */
 	ctx->max_attrs = max_attrs;
-	PFREE_ST(old_attrs, db_attrs_st);
+	if(old_attrs) PFREE_ST(old_attrs, db_attrs_st);
 	ret = 0;
 out:
 	return ret;
@@ -324,7 +327,7 @@ int
 db_attr_add_values(struct db_context *ctx,  u_int16_t type, u_int16_t val)
 {
 	struct db_attr attr;
-	attr.type = type;
+	attr.type.oakley = type;
 	attr.val = val;
 	return db_attr_add (ctx, &attr);
 }
@@ -332,7 +335,7 @@ db_attr_add_values(struct db_context *ctx,  u_int16_t type, u_int16_t val)
 int
 db_ops_show_status(void)
 {
-	whack_log(RC_COMMENT, "stats " __FILE__ ": " 
+	whack_log(RC_COMMENT, "stats db_ops: " 
 			DB_OPS_STATS_DESC " :"
 			DB_OPS_STATS_STR("context")
 			DB_OPS_STATS_STR("trans")
@@ -347,33 +350,41 @@ db_ops_show_status(void)
 /* 
  * From below to end just testing stuff ....
  */
-#ifdef TEST
+#if defined(TEST)
 static void db_prop_print(struct db_prop *p)
 {
 	struct db_trans *t;
 	struct db_attr *a;
 	int ti, ai;
 	enum_names *n, *n_at, *n_av;
-	printf("protoid=\"%s\"\n", enum_name(&protocol_names, p->protoid));
+
+	DBG_log("protoid=\"%s\"\n", enum_name(&protocol_names, p->protoid));
 	for (ti=0, t=p->trans; ti< p->trans_cnt; ti++, t++) {
-		switch( t->transid) {
+		switch( p->protoid) {
 			case PROTO_ISAKMP:
-				n=&isakmp_transformid_names;break;
+				n=&isakmp_transformid_names;
+				break;
 			case PROTO_IPSEC_ESP:
-				n=&esp_transformid_names;break;
+				n=&esp_transformid_names;
+				break;
+			case PROTO_IPSEC_AH:
+				n=&ah_transformid_names;
+				break;
 			default:
 				continue;
 		}
-		printf("  transid=\"%s\"\n", 
-			enum_name(n, t->transid));
+		DBG_log("  transid=\"%s\"\n", enum_name(n, t->transid));
+
 		for (ai=0, a=t->attrs; ai < t->attr_cnt; ai++, a++) {
 			int i;
-			switch( t->transid) {
+			switch( p->protoid) {
 				case PROTO_ISAKMP:
 					n_at=&oakley_attr_names;
 					i=a->type|ISAKMP_ATTR_AF_TV;
 					n_av=oakley_attr_val_descs[(i)&ISAKMP_ATTR_RTYPE_MASK];
 					break;
+
+				case PROTO_IPSEC_AH:
 				case PROTO_IPSEC_ESP:
 					n_at=&ipsec_attr_names;
 					i=a->type|ISAKMP_ATTR_AF_TV;
@@ -382,20 +393,24 @@ static void db_prop_print(struct db_prop *p)
 				default:
 					continue;
 			}
-			printf("    type=\"%s\" value=\"%s\"\n", 
+			DBG_log("    type=\"%s\" value=\"%s\"\n", 
 				enum_name(n_at, i),
 				enum_name(n_av, a->val));
 		}
 	}
 
 }
-static void db_print(struct db_context *ctx) 
+
+void db_print(struct db_context *ctx) 
 {
-	printf("trans_cur diff=%d, attrs_cur diff=%d\n", 
+	DBG_log("trans_cur diff=%d, attrs_cur diff=%d\n", 
 			ctx->trans_cur - ctx->trans0,
 			ctx->attrs_cur - ctx->attrs0);
 	db_prop_print(&ctx->prop);
 }
+#endif
+
+#if defined(TEST)
 
 void
 passert_fail(const char *pred_str, const char *file_str, unsigned long line_no);
