@@ -1,10 +1,9 @@
+#ifndef _IKEv1_CONTINUATIONS_H
+#define _IKEv1_CONTINUATIONS_H
 /*
  * continuations used
  */
-struct ke_continuation {
-    struct pluto_crypto_req_cont ke_pcrc;
-    struct msg_digest           *md;
-};
+#include "ike_continuations.h"
 
 struct qke_continuation {
     struct pluto_crypto_req_cont qke_pcrc;
@@ -14,14 +13,6 @@ struct qke_continuation {
     struct msg_digest           *md;            /* used in responder */
 };
 
-struct dh_continuation {
-	struct pluto_crypto_req_cont dh_pcrc;
-	struct msg_digest           *md;
-	so_serial_t                  serialno;  /* used for inter state
-						 * calculations on responder */
-};
-
-
 typedef stf_status initiator_function(int whack_sock
 				      , struct connection *c
 				      , struct state *predecessor
@@ -39,3 +30,4 @@ typedef stf_status initiator_function(int whack_sock
 
 #define RETURN_STF_FAILURE(f) RETURN_STF_FAILURE2(f, NULL)
 
+#endif /* _IKEv1_CONTINUATIONS */
