@@ -29,9 +29,9 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
 #include <sys/time.h>		/* for gettimeofday */
 #include <gmp.h>
+#include <resolv.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
@@ -2527,7 +2527,7 @@ send_notification(struct state *sndst, u_int16_t type, struct state *encst,
 	}
 
 	if(spisize > 0) {
-	    if(out_raw(spi, spisize, &not_pbs, "spi")) {
+	    if(!out_raw(spi, spisize, &not_pbs, "spi")) {
 		openswan_log("failed to build notification for spisize=%d\n", (int)spisize);
 		return;
 	    }
