@@ -27,8 +27,8 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
 #include <sys/time.h>		/* for gettimeofday */
+#include <resolv.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
@@ -1673,12 +1673,15 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 #ifdef NAT_TRAVERSAL
 #ifdef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
     if( (p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED)
-       && !(p1st->st_policy & POLICY_TUNNEL)
-       && (p1st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))
-       && (p == NULL) )
+	&& !(p1st->st_policy & POLICY_TUNNEL)
+	&&  (p1st->hidden_variables.st_nat_traversal & (LELEM(NAT_TRAVERSAL_NAT_BHND_ME) | LELEM(NAT_TRAVERSAL_NAT_BHND_PEER)) )
+	&& (p == NULL) )
         {
-          p = c;
-          DBG(DBG_CONTROL, DBG_log("using (something - hopefully the IP we are NAT'ed too) for transport mode connection \"%s\"", p->name));
+	    p = c;
+	    DBG(DBG_CONTROL
+		, DBG_log("using (something - hopefully the IP we or they are"
+			  " NAT'ed to) for transport mode connection \"%s\""
+			  , p->name));
         }
 #endif
 #endif
@@ -1830,7 +1833,7 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 
 
 
-	else if (is_virtual_connection(c))
+	if (is_virtual_connection(c))
 	{
 	    char cthat[END_BUF];
 
@@ -1858,10 +1861,12 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
      */
 
     hv = p1st->hidden_variables;
+#ifdef NAT_TRAVERSAL
     if ((p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
 	(p1st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
 	nat_traversal_natoa_lookup(md, &hv);
     }
+#endif
 
     /* now that we are sure of our connection, create our new state */
     {
@@ -2250,21 +2255,6 @@ quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 	p->isaiid_np = ISAKMP_NEXT_NONE;
     }
 
-#ifdef NAT_TRAVERSAL
-#if 0
-    DBG_log("NAT-OA: %d tunnel: %d \n"
-		,(st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)
-    		,(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT));
-    if ((st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA) &&
-	(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT)) {
-	/** Send NAT-OA if our address is NATed and if we use Transport Mode */
-	if (!nat_traversal_add_natoa(ISAKMP_NEXT_NONE, &md->rbody, md->st, FALSE)) {
-	    return STF_INTERNAL_ERROR;
-	}
-    }
-#endif
-#endif
-
 #ifdef TPM
     {
 	pb_stream *pbs = &md->rbody;
