@@ -368,7 +368,7 @@ ikev2_parent_outI1_common(struct msg_digest *md
 	{
 		chunk_t child_spi;
 		memset(&child_spi, 0, sizeof(child_spi));
-		ship_v2N (ISAKMP_NEXT_v2SA, ISAKMP_PAYLOAD_CRITICAL, PROTO_ISAKMP,
+		ship_v2N (ISAKMP_NEXT_v2SA, ISAKMP_PAYLOAD_NONCRITICAL, PROTO_ISAKMP,
 				    &child_spi, 
 					COOKIE, &st->st_dcookie, &md->rbody);
     }
@@ -414,7 +414,7 @@ ikev2_parent_outI1_common(struct msg_digest *md
 	
 	memset(&in, 0, sizeof(in));
 	in.isag_np = np;
-	in.isag_critical = ISAKMP_PAYLOAD_CRITICAL;
+	in.isag_critical = ISAKMP_PAYLOAD_NONCRITICAL;
 
 	if(!out_struct(&in, &ikev2_nonce_desc, &md->rbody, &pb) ||
 	   !out_raw(st->st_ni.ptr, st->st_ni.len, &pb, "IKEv2 nonce"))
@@ -514,7 +514,7 @@ stf_status ikev2parent_inI1outR1(struct msg_digest *md)
 	 * are under DOS
 	 */
 	DBG_log("no connection found\n");
-	//SEND_NOTIFICATION(NO_PROPOSAL_CHOSEN);
+	/* SEND_NOTIFICATION(NO_PROPOSAL_CHOSEN); */
 	return STF_FAIL + NO_PROPOSAL_CHOSEN;
     }
 	
@@ -757,7 +757,7 @@ ikev2_parent_inI1outR1_tail(struct pluto_crypto_req_cont *pcrc
 	
 	memset(&in, 0, sizeof(in));
 	in.isag_np = np;
-	in.isag_critical = ISAKMP_PAYLOAD_CRITICAL;
+	in.isag_critical = ISAKMP_PAYLOAD_NONCRITICAL;
 
 	if(!out_struct(&in, &ikev2_nonce_desc, &md->rbody, &pb) ||
 	   !out_raw(st->st_nr.ptr, st->st_nr.len, &pb, "IKEv2 nonce"))
@@ -816,7 +816,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 stf_status ikev2parent_inR1outI2(struct msg_digest *md)
 {
     struct state *st = md->st;
-    //struct connection *c = st->st_connection;
+    /* struct connection *c = st->st_connection; */
     pb_stream *keyex_pbs;
 	
     /* check if the responder replied with v2N with DOS COOKIE */
@@ -840,7 +840,7 @@ stf_status ikev2parent_inR1outI2(struct msg_digest *md)
 
 		change_state(st, STATE_PARENT_I1);
     	st->st_msgid_lastack = INVALID_MSGID;
-	 	md->msgid_received = INVALID_MSGID;  //AAA hack 
+	md->msgid_received = INVALID_MSGID;  /* AAA hack  */
     	st->st_msgid_nextuse = 0;
 
 		return ikev2_parent_outI1_common(md, st);  
@@ -989,13 +989,22 @@ static void ikev2_padup_pre_encrypt(struct msg_digest *md
     }
 }
 
-static unsigned char *ikev2_authloc(struct msg_digest *md UNUSED
+static unsigned char *ikev2_authloc(struct msg_digest *md
 				    , pb_stream *e_pbs)
 {
     unsigned char *b12;
+    struct state *st = md->st;
+    struct state *pst = st;
+
+    if(st->st_clonedfrom != 0) {
+	pst = state_with_serialno(st->st_clonedfrom);
+	if( pst == NULL) {
+		return NULL;
+	}
+    }
 	
     b12 = e_pbs->cur;
-    if(!out_zero(12, e_pbs, "96-bits of truncated HMAC"))
+    if(!out_zero(pst->st_oakley.integ_hasher->hash_integ_len, e_pbs, "length of truncated HMAC"))
 	return NULL;
 
     return b12;
@@ -1058,7 +1067,7 @@ static stf_status ikev2_encrypt_msg(struct msg_digest *md,
 	
 	if(DBGP(DBG_PARSING)) {
 	    DBG_dump("data being hmac:", authstart, authloc-authstart);
-	    DBG_dump("out calculated auth:", authloc, 12); 
+	    DBG_dump("out calculated auth:", authloc, pst->st_oakley.integ_hasher->hash_integ_len); 
 	}
     }
     
@@ -1108,12 +1117,12 @@ stf_status ikev2_decrypt_msg(struct msg_digest *md
 	
 	if(DBGP(DBG_PARSING)) {
 	    DBG_dump("data being hmac:", authstart, encend-authstart);
-	    DBG_dump("R2 calculated auth:", b12, 12); 
-	    DBG_dump("R2  provided  auth:", encend, 12);
+	    DBG_dump("R2 calculated auth:", b12, pst->st_oakley.integ_hasher->hash_integ_len); 
+	    DBG_dump("R2  provided  auth:", encend, pst->st_oakley.integ_hasher->hash_integ_len);
 	}
 	
 	/* compare first 96 bits == 12 bytes */
-	if(memcmp(b12, encend, 12)!=0) {
+	if(memcmp(b12, encend, pst->st_oakley.integ_hasher->hash_integ_len)!=0) {
 	    openswan_log("R2 failed to match authenticator");
 	    return STF_FAIL;
 	}
@@ -1180,7 +1189,7 @@ static stf_status ikev2_send_auth(struct connection *c
     }
 
     
-    a.isaa_critical = ISAKMP_PAYLOAD_CRITICAL;
+    a.isaa_critical = ISAKMP_PAYLOAD_NONCRITICAL;
     a.isaa_np = np;
     
     if(c->policy & POLICY_RSASIG) {
@@ -1277,7 +1286,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 
     /* insert an Encryption payload header */
     e.isag_np = ISAKMP_NEXT_v2IDi;
-    e.isag_critical = ISAKMP_PAYLOAD_CRITICAL;
+    e.isag_critical = ISAKMP_PAYLOAD_NONCRITICAL;
 
     if(!out_struct(&e, &ikev2_e_desc, &md->rbody, &e_pbs)) {
 	return STF_INTERNAL_ERROR;
@@ -1309,7 +1318,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 
 	hmac_init_chunk(&id_ctx, pst->st_oakley.prf_hasher, pst->st_skey_pi);
 	build_id_payload((struct isakmp_ipsec_id *)&r_id, &id_b, &c->spd.this);
-	r_id.isai_critical = ISAKMP_PAYLOAD_CRITICAL;
+	r_id.isai_critical = ISAKMP_PAYLOAD_NONCRITICAL;
 	{  /* decide to send CERT payload */
 	    send_cert = doi_send_ikev2_cert_thinking(st);
 	    
@@ -1366,16 +1375,27 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 
 	/*
 	 * now, find an eligible child SA from the pending list, and emit
-	 * SA2i, TSi and TSr for it.
+	 * SA2i, TSi and TSr and (USE_TRANSPORT_MODE notification in transport mode) for it .
 	 */
 	if(c0) {
 	    st->st_connection = c0;
+
 	    ikev2_emit_ipsec_sa(md,&e_pbs_cipher,ISAKMP_NEXT_v2TSi,c0, policy);
 	    
 	    st->st_ts_this = ikev2_subnettots(&c0->spd.this);
 	    st->st_ts_that = ikev2_subnettots(&c0->spd.that);
 	    
 	    ikev2_calc_emit_ts(md, &e_pbs_cipher, INITIATOR, c0, policy);
+
+	    if( !(st->st_connection->policy & POLICY_TUNNEL) ) {
+		DBG_log("Initiator child policy is transport mode, sending USE_TRANSPORT_MODE");
+		chunk_t child_spi, notifiy_data;
+		memset(&child_spi, 0, sizeof(child_spi));
+		memset(&notifiy_data, 0, sizeof(notifiy_data));
+		ship_v2N (ISAKMP_NEXT_NONE, ISAKMP_PAYLOAD_NONCRITICAL, 0,
+				&child_spi,
+				USE_TRANSPORT_MODE, &notifiy_data, &e_pbs_cipher);
+	    }
 	} else {
 	    openswan_log("no pending SAs found, PARENT SA keyed only");
 	}
@@ -1445,7 +1465,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 stf_status ikev2parent_inI2outR2(struct msg_digest *md)
 {
     struct state *st = md->st;
-    //struct connection *c = st->st_connection;
+    /* struct connection *c = st->st_connection; */
 
     /*
      * the initiator sent us an encrypted payload. We need to calculate
@@ -1461,6 +1481,12 @@ stf_status ikev2parent_inI2outR2(struct msg_digest *md)
 	return STF_FATAL;
     }
 
+    /* process AUTH payload */
+    if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
+	openswan_log("no authentication payload found");
+	return STF_FATAL;
+    }
+
     /* now. we need to go calculate the g^xy */
     {
 	struct dh_continuation *dh = alloc_thing(struct dh_continuation
@@ -1600,12 +1626,6 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    ikev2_decode_cr(md, &st->st_connection->requested_ca);
     }
 
-    /* process AUTH payload */
-    if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
-	openswan_log("no authentication payload found");
-	return STF_FAIL;
-    }
-
     /* now check signature from RSA key */
     switch(md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2a.isaa_type)
     {
@@ -1680,7 +1700,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	
 	/* insert an Encryption payload header */
 	e.isag_np = ISAKMP_NEXT_v2IDr;
-	e.isag_critical = ISAKMP_PAYLOAD_CRITICAL;
+	e.isag_critical = ISAKMP_PAYLOAD_NONCRITICAL;
 
 	if(!out_struct(&e, &ikev2_e_desc, &md->rbody, &e_pbs)) {
 	    return STF_INTERNAL_ERROR;
@@ -1717,7 +1737,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 			    , st->st_skey_pr);
 	    build_id_payload((struct isakmp_ipsec_id *)&r_id, &id_b,
 			     &c->spd.this);
-	    r_id.isai_critical = ISAKMP_PAYLOAD_CRITICAL;
+	    r_id.isai_critical = ISAKMP_PAYLOAD_NONCRITICAL;
 
 	    if(send_cert) 
 		r_id.isai_np = ISAKMP_NEXT_v2CERT;
@@ -1970,6 +1990,37 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
 	if(rn != NOTHING_WRONG)
 	    return STF_FAIL + rn;
     }
+
+    {
+	if ( md->chain[ISAKMP_NEXT_v2N] ) {
+	    if ( (md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == USE_TRANSPORT_MODE )) {
+		if ( st->st_connection->policy & POLICY_TUNNEL) {
+		/*This means we did not send USE_TRANSPORT, however responder is sending it in now (inR2), seems incorrect*/
+			DBG(DBG_CONTROLMORE,
+			DBG_log("Initiator policy is tunnel, responder sends USE_TRANSPORT_MODE notification in inR2, ignoring it"));
+		}
+		else {
+			DBG(DBG_CONTROLMORE,
+			DBG_log("Initiator policy is transport, responder sends USE_TRANSPORT_MODE, setting CHILD SA to transport mode"));
+			if (st->st_esp.present == TRUE) { 
+			/*openswan supports only "esp" with ikev2 it seems, look at ikev2_parse_child_sa_body handling*/
+			st->st_esp.attrs.encapsulation = ENCAPSULATION_MODE_TRANSPORT;
+			}
+		}
+	    }
+	}
+
+        if ( md->chain[ISAKMP_NEXT_v2N] == NULL ||
+               !(md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == USE_TRANSPORT_MODE ) ) {
+                if ( !(st->st_connection->policy & POLICY_TUNNEL) ) {
+                /*This means we sent USE_TRANSPORT, however responder did not send it or did not agree with that*/
+                        DBG(DBG_CONTROLMORE,
+			DBG_log("Initiator policy is transport, responder did not send USE_TRANSPORT_MODE, so falling back to tunnel mode (rfc 4306)"));
+                }
+        }
+
+    }
+
 	
     ikev2_derive_child_keys(st, md->role);
 
@@ -2095,7 +2146,7 @@ send_v2_notification(struct state *p1st, u_int16_t type
 	child_spi.len = 0;
 
 	/* build and add v2N payload to the packet */
-	ship_v2N (ISAKMP_NEXT_NONE, ISAKMP_PAYLOAD_CRITICAL, PROTO_ISAKMP,
+	ship_v2N (ISAKMP_NEXT_NONE, ISAKMP_PAYLOAD_NONCRITICAL, PROTO_ISAKMP,
 				    &child_spi, 
 					type, n_data, &rbody);
 
