@@ -106,7 +106,7 @@ ikev2_send_cert( struct state *st, struct msg_digest *md
 		 	, DBG_log("  no known CA for the other end"));}
 	     else 
 		{ DBG(DBG_CONTROL, 
-	    	      DBG_log(" we reached an unexecpted state - a bad day? I don't feel like sending a certificate request (CERTREQ)"));}
+	    	      DBG_log(" we reached an unexpected state - a bad day? I don't feel like sending a certificate request (CERTREQ)"));}
         }
     cert.isac_critical = ISAKMP_PAYLOAD_NONCRITICAL;
     cert.isac_enc = mycert.type;
