@@ -23,6 +23,7 @@
 #include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <resolv.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
@@ -85,6 +86,11 @@ orient(struct connection *c)
 
 	for (sr = &c->spd; sr; sr = sr->next)
 	{
+	    /* There can be more then 1 spd policy associated - required
+	     * for cisco split networking when remote_peer_type=cisco
+	     */
+	    if(c->remotepeertype == CISCO && sr != &c->spd ) continue;
+
 	    /* Note: this loop does not stop when it finds a match:
 	     * it continues checking to catch any ambiguity.
 	     */
@@ -444,7 +450,7 @@ cannot_oppo(struct connection *c
 	{
 	    char state_buf[LOG_WIDTH];
 	    char state_buf2[LOG_WIDTH];
-	    time_t n = now();
+	    const time_t n = now();
 
 	    fmt_state(st, n, state_buf, sizeof(state_buf)
 		      , state_buf2, sizeof(state_buf2));
@@ -488,10 +494,10 @@ cannot_oppo(struct connection *c
 #endif
 }
 
-static void initiate_ondemand_body(struct find_oppo_bundle *b
+static int initiate_ondemand_body(struct find_oppo_bundle *b
     , struct adns_continuation *ac, err_t ac_ugh);	/* forward */
 
-void
+int
 initiate_ondemand(const ip_address *our_client
 , const ip_address *peer_client
 , int transport_proto
@@ -511,7 +517,7 @@ initiate_ondemand(const ip_address *our_client
     b.failure_shunt = 0;
     b.whackfd = whackfd;
     b.step = fos_start;
-    initiate_ondemand_body(&b, NULL, NULL);
+    return initiate_ondemand_body(&b, NULL, NULL);
 }
 
 static void
@@ -580,7 +586,7 @@ continue_oppo(struct adns_continuation *acr, err_t ugh)
     }
     else
     {
-	initiate_ondemand_body(&cr->b, &cr->ac, ugh);
+	(void)initiate_ondemand_body(&cr->b, &cr->ac, ugh);
 	whackfd = NULL_FD;	/* was handed off */
     }
 
@@ -702,7 +708,8 @@ check_txt_recs(enum myid_state try_state
 
 
 /* note: gateways_from_dns must be NULL iff this is the first call */
-static void
+/* return true if we did something */
+static int
 initiate_ondemand_body(struct find_oppo_bundle *b
 , struct adns_continuation *ac
 , err_t ac_ugh)
@@ -715,8 +722,11 @@ initiate_ondemand_body(struct find_oppo_bundle *b
     int hisport;
     char demandbuf[256];
     bool loggedit = FALSE;
+    int work = 0;
+
+    /* on klips/mast assume we will do something */
+    work = (kern_interface == USE_KLIPS || kern_interface == USE_MASTKLIPS);
 
-    
     /* What connection shall we use?
      * First try for one that explicitly handles the clients.
      */
@@ -741,6 +751,7 @@ initiate_ondemand_body(struct find_oppo_bundle *b
     if (isanyaddr(&b->our_client) || isanyaddr(&b->peer_client))
     {
 	cannot_oppo(NULL, b, "impossible IP address");
+	work = 0;
     }
     else if ((c = find_connection_for_clients(&sr
 					      , &b->our_client
@@ -753,12 +764,14 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 	 */
 	if(!loggedit) { openswan_log("%s", demandbuf); loggedit=TRUE; }
 	cannot_oppo(NULL, b, "no routed template covers this pair");
+	work = 0;
     }
     else if (c->kind == CK_TEMPLATE && (c->policy & POLICY_OPPO)==0)
     {
 	if(!loggedit) { openswan_log("%s", demandbuf); loggedit=TRUE; }
 	loglog(RC_NOPEERIP, "cannot initiate connection for packet %s:%d -> %s:%d proto=%d - template conn"
 	       , ours, ourport, his, hisport, b->transport_proto);
+	work = 0;
     }
     else if (c->kind != CK_TEMPLATE)
     {
@@ -777,7 +790,7 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 	{
 	    char cib[CONN_INST_BUF];
 	    /* there is already an instance being negotiated, do nothing */
-	    openswan_log("rekeing existing instance \"%s\"%s, due to acquire"
+	    openswan_log("rekeying existing instance \"%s\"%s, due to acquire"
 			 , c->name
 			 , (fmt_conn_instance(c, cib), cib));
 
@@ -1435,6 +1448,7 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 	}
     }
     close_any(b->whackfd);
+    return work;
 }
 
 /* check nexthop safety
@@ -1443,43 +1457,44 @@ initiate_ondemand_body(struct find_oppo_bundle *b
  * not process a packet output by an eroute.
  */
 #ifdef NEVER
-//bool
-//check_nexthop(const struct connection *c)
-//{
-//    struct connection *d;
-//
-//    if (addrinsubnet(&c->spd.this.host_nexthop, &c->spd.that.client))
-//    {
-//	loglog(RC_LOG_SERIOUS, "cannot perform routing for connection \"%s\""
-//	    " because nexthop is within peer's client network",
-//	    c->name);
-//	return FALSE;
-//    }
-//
-//    for (d = connections; d != NULL; d = d->next)
-//    {
-//	if (d->routing != RT_UNROUTED)
-//	{
-//	    if (addrinsubnet(&c->spd.this.host_nexthop, &d->spd.that.client))
-//	    {
-//		loglog(RC_LOG_SERIOUS, "cannot do routing for connection \"%s\"
-//		    " because nexthop is contained in"
-//		    " existing routing for connection \"%s\"",
-//		    c->name, d->name);
-//		return FALSE;
-//	    }
-//	    if (addrinsubnet(&d->spd.this.host_nexthop, &c->spd.that.client))
-//	    {
-//		loglog(RC_LOG_SERIOUS, "cannot do routing for connection \"%s\"
-//		    " because it contains nexthop of"
-//		    " existing routing for connection \"%s\"",
-//		    c->name, d->name);
-//		return FALSE;
-//	    }
-//	}
-//    }
-//    return TRUE;
-//}
+/* bool
+ * check_nexthop(const struct connection *c)
+ * {
+ *     struct connection *d;
+ * 
+ *     if (addrinsubnet(&c->spd.this.host_nexthop, &c->spd.that.client))
+ *     {
+ * 	loglog(RC_LOG_SERIOUS, "cannot perform routing for connection \"%s\""
+ * 	    " because nexthop is within peer's client network",
+ * 	    c->name);
+ * 	return FALSE;
+ *     }
+ * 
+ *     for (d = connections; d != NULL; d = d->next)
+ *     {
+ * 	if (d->routing != RT_UNROUTED)
+ * 	{
+ * 	    if (addrinsubnet(&c->spd.this.host_nexthop, &d->spd.that.client))
+ * 	    {
+ * 		loglog(RC_LOG_SERIOUS, "cannot do routing for connection \"%s\"
+ * 		    " because nexthop is contained in"
+ * 		    " existing routing for connection \"%s\"",
+ * 		    c->name, d->name);
+ * 		return FALSE;
+ * 	    }
+ * 	    if (addrinsubnet(&d->spd.this.host_nexthop, &c->spd.that.client))
+ * 	    {
+ * 		loglog(RC_LOG_SERIOUS, "cannot do routing for connection \"%s\"
+ * 		    " because it contains nexthop of"
+ * 		    " existing routing for connection \"%s\"",
+ * 		    c->name, d->name);
+ * 		return FALSE;
+ * 	    }
+ * 	}
+ *     }
+ *     return TRUE;
+ * }
+ */
 #endif /* NEVER */
 
 /* an ISAKMP SA has been established.
