@@ -29,9 +29,9 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
 #include <sys/time.h>		/* for gettimeofday */
 #include <gmp.h>
+#include <resolv.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
@@ -518,7 +518,7 @@ extract_peer_id(struct id *peer, const pb_stream const* id_pbs)
 	    memcpy(idbuf, id_pbs->cur, len-1);
 	    idbuf[len]='\0';
 	    loglog(RC_LOG_SERIOUS, "peer's ID_USER_FQDN contains no @: %s", idbuf);
-	    //return FALSE;
+	    /* return FALSE; */
 	}
 	/* FALLTHROUGH */
     case ID_FQDN:
@@ -870,6 +870,7 @@ void fmt_isakmp_sa_established(struct state *st, char *sadetails, int sad_len)
     char *b = sadetails;
     const char *authname;
     const char *integstr, *integname;
+    char integname_tmp[20];
     
     passert(st->st_oakley.encrypter != NULL);
     passert(st->st_oakley.prf_hasher != NULL);
@@ -878,7 +879,9 @@ void fmt_isakmp_sa_established(struct state *st, char *sadetails, int sad_len)
     if(st->st_ikev2) {
 	authname="IKEv2";
 	integstr=" integ=";
-	integname=st->st_oakley.integ_hasher->common.officname;
+	snprintf(integname_tmp, sizeof(integname_tmp), "%s_%zu", st->st_oakley.integ_hasher->common.officname
+		, st->st_oakley.integ_hasher->hash_integ_len*BITS_PER_BYTE);
+	integname=(const char*)integname_tmp;
     } else {
 	authname = enum_show(&oakley_auth_names, st->st_oakley.auth);
 	integstr="";
