@@ -49,6 +49,7 @@
 #include "kernel_netlink.h"
 #include "kernel_pfkey.h"
 #include "kernel_noklips.h"
+#include "kernel_bsdkame.h"
 #include "packet.h"
 #include "x509.h"
 #include "log.h"
@@ -98,6 +99,8 @@ static struct bare_shunt *bare_shunts = NULL;
 static int num_ipsec_eroute = 0;
 #endif
 
+static void free_bare_shunt(struct bare_shunt **pp);
+
 #ifdef DEBUG
 void
 DBG_bare_shunt_log(const char *op, const struct bare_shunt *bs)
@@ -161,7 +164,15 @@ record_and_initiate_opportunistic(const ip_subnet *ours
 
         networkof(ours, &src);
         networkof(his, &dst);
-        initiate_ondemand(&src, &dst, transport_proto, TRUE, NULL_FD, "acquire");
+        if (initiate_ondemand(&src, &dst, transport_proto
+				, TRUE, NULL_FD, "acquire") == 0) {
+			/* if we didn't do any ondemand stuff the shunt is not needed */
+			struct bare_shunt **bspp = bare_shunt_ptr(ours,his,transport_proto);
+			if (bspp) {
+				passert(*bspp == bare_shunts);
+				free_bare_shunt(bspp);
+			}
+		}
     }
 
     pexpect(kernel_ops->remove_orphaned_holds != NULL);
@@ -360,12 +371,12 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 	char *p;
 	int   l;
 	strncat(srcip_str, "PLUTO_MY_SOURCEIP=", sizeof(srcip_str));
-	strncat(srcip_str, "'", sizeof(srcip_str));
+	strncat(srcip_str, "'", sizeof(srcip_str)-strlen(srcip_str)-1);
 	l = strlen(srcip_str);
 	p = srcip_str + l;
         
 	addrtot(&sr->this.host_srcip, 0, p, sizeof(srcip_str));
-	strncat(srcip_str, "'", sizeof(srcip_str));
+	strncat(srcip_str, "'", sizeof(srcip_str)-strlen(srcip_str)-1);
     }
     
     {
@@ -412,6 +423,13 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 		    "PLUTO_CONN_POLICY='%s' "
 		    "%s "           /* XAUTH username */
 		    "%s "           /* PLUTO_MY_SRCIP */
+		    "PLUTO_CISCO_DNS_INFO='%s' "
+		    "PLUTO_CISCO_DOMAIN_INFO='%s' "
+		    "PLUTO_PEER_BANNER='%s' "
+#ifdef HAVE_NM
+		    "PLUTO_NM_CONFIGURED='%u' "
+#endif
+
 		    , c->name
 		    , c->interface->ip_dev->id_vname
 		    , nexthop_str
@@ -434,7 +452,14 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 		    , metric_str
 		    , prettypolicy(c->policy)
 		    , secure_xauth_username_str
-		    , srcip_str);
+		    , srcip_str
+		    , c->cisco_dns_info
+		    , c->cisco_domain_info
+		    , c->server_banner
+#ifdef HAVE_NM
+		    , c->nmconfigured
+#endif
+		);
 	/* 
 	 * works for both old and new way of snprintf() returning
 	 * eiter -1 or the output length  -- by Carsten Schlote
@@ -497,7 +522,7 @@ could_route(struct connection *c)
 {
     struct spd_route *esr, *rosr;
     struct connection *ero      /* who, if anyone, owns our eroute? */
-        , *ro = route_owner(c, &rosr, &ero, &esr); /* who owns our route? */
+        , *ro = route_owner(c, &c->spd, &rosr, &ero, &esr); /* who owns our route? */
 
     DBG(DBG_CONTROL,
         DBG_log("could_route called for %s (kind=%s)"
@@ -562,8 +587,13 @@ could_route(struct connection *c)
     {
         loglog(RC_LOG_SERIOUS, "cannot route -- route already in use for \"%s\""
             , ro->name);
-        return route_impossible;  /* another connection already using the
-				     eroute. TODO: NETKEY can do this? */
+        /* We ignore this if the stack supports overlapping, and this
+         * connection was marked that overlapping is OK.  Below we will
+         * check the other eroute, ero. */
+        if (!kernel_ops->overlap_supported || !ero || ero == c
+                        || !LIN(POLICY_OVERLAPIP, c->policy))
+                return route_impossible;  /* another connection already using the
+                                             eroute. TODO: NETKEY can do this? */
     }
 
     /* if there is an eroute for another connection, there is a problem */
@@ -646,11 +676,18 @@ could_route(struct connection *c)
 
             fmt_conn_instance(ero, inst);
 
-            loglog(RC_LOG_SERIOUS
-                , "cannot install eroute -- it is in use for \"%s\"%s #%lu"
-                , ero->name, inst, esr->eroute_owner);
-            return FALSE;       /* another connection already using the eroute,
-				   TODO: NETKEY apparently can do this though */
+            if(!LIN(POLICY_OVERLAPIP, c->policy)
+                        || !LIN(POLICY_OVERLAPIP, ero->policy)) {
+                    loglog(RC_LOG_SERIOUS
+                        , "cannot install eroute -- it is in use for \"%s\"%s #%lu"
+                        , ero->name, inst, esr->eroute_owner);
+                    return FALSE;       /* another connection already using the eroute,
+                                           TODO: NETKEY apparently can do this though */
+            }
+
+            DBG(DBG_CONTROL,
+                DBG_log("overlapping permitted with \"%s\"%s #%lu"
+                        , ero->name, inst, esr->eroute_owner));
         }
     }
     return route_easy;
@@ -740,7 +777,7 @@ unroute_connection(struct connection *c)
         sr->routing = RT_UNROUTED;  /* do now so route_owner won't find us */
 
         /* only unroute if no other connection shares it */
-        if (routed(cr) && route_owner(c, NULL, NULL, NULL) == NULL)
+        if (routed(cr) && route_owner(c, sr, NULL, NULL, NULL) == NULL)
             (void) do_command(c, sr, "unroute", NULL);
     }
 }
@@ -839,7 +876,7 @@ raw_eroute(const ip_address *this_host
            , ipsec_spi_t spi
            , unsigned int proto
            , unsigned int transport_proto
-           , unsigned int satype
+           , enum eroute_type esatype
            , const struct pfkey_proto_info *proto_info
            , time_t use_lifetime
            , enum pluto_sadb_operations op
@@ -868,7 +905,7 @@ raw_eroute(const ip_address *this_host
                                   , that_host, that_client
                                   , spi, proto
                                   , transport_proto
-                                  , satype, proto_info
+                                  , esatype, proto_info
                                   , use_lifetime, op, text_said);
     
     if(result == FALSE || DBGP(DBG_CONTROL|DBG_KLIPS)) {
@@ -990,7 +1027,7 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
                                        , null_host, &that_broad_client
                                        , htonl(shunt_spi), SA_INT
                                        , transport_proto
-                                       , K_SADB_X_SATYPE_INT, null_proto_info
+                                       , ET_INT, null_proto_info
                                        , SHUNT_PATIENCE, ERO_REPLACE, why))
                             {
                                 struct bare_shunt *bs = alloc_thing(struct bare_shunt, "bare shunt");
@@ -1018,7 +1055,7 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
                        , htonl(shunt_spi)
                        , SA_INT
                        , transport_proto
-                       , K_SADB_X_SATYPE_INT, null_proto_info
+                       , ET_INT, null_proto_info
                        , SHUNT_PATIENCE, ERO_ADD, why))
             {
                 struct bare_shunt **bs_pp = bare_shunt_ptr(&this_client, &that_client
@@ -1041,7 +1078,7 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
         if (raw_eroute(null_host, &this_client, null_host, &that_client
                        , htonl(shunt_spi), SA_INT
                        , 0 /* transport_proto */
-                       , K_SADB_X_SATYPE_INT, null_proto_info
+                       , ET_INT, null_proto_info
                        , SHUNT_PATIENCE, op, why))
             {
                 struct bare_shunt **bs_pp = bare_shunt_ptr(&this_client
@@ -1079,7 +1116,7 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
 
 bool eroute_connection(struct spd_route *sr
 		       , ipsec_spi_t spi, unsigned int proto
-		       , unsigned int satype
+		       , enum eroute_type esatype
 		       , const struct pfkey_proto_info *proto_info
 		       , unsigned int op, const char *opname)
 {
@@ -1098,7 +1135,7 @@ bool eroute_connection(struct spd_route *sr
                       , spi
                       , proto
                       , sr->this.protocol
-                      , satype
+                      , esatype
                       , proto_info, 0, op, buf2);
 }
 
@@ -1168,7 +1205,7 @@ assign_hold(struct connection *c USED_BY_DEBUG
 	    }
 
             if(!eroute_connection(sr, htonl(SPI_HOLD)
-				  , SA_INT, K_SADB_X_SATYPE_INT
+				  , SA_INT, ET_INT
 				  , null_proto_info
 				  , op
 				  , reason)) {
@@ -1249,7 +1286,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
     ip_subnet src, dst;
     ip_subnet src_client, dst_client;
     ipsec_spi_t inner_spi = 0;
-    unsigned int proto = 0, satype = 0;
+    unsigned int proto = 0;
+    enum eroute_type esatype = ET_UNSPEC;
     bool replace;
     bool outgoing_ref_set = FALSE;
     bool incoming_ref_set = FALSE;
@@ -1300,7 +1338,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
     {
         inner_spi = 256;
         proto = SA_IPIP;
-        satype = SADB_SATYPE_UNSPEC;
+        esatype = ET_IPIP; /* XXX bart: used to be "UNSPEC" */
     }
     else if (encapsulation == ENCAPSULATION_MODE_TUNNEL)
     {
@@ -1333,7 +1371,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = ipip_spi;
-        said_next->satype = K_SADB_X_SATYPE_IPIP;
+        said_next->esatype = ET_IPIP;
         said_next->text_said = text_said;
 	said_next->sa_lifetime = c->sa_ipsec_life_seconds;
 
@@ -1392,7 +1430,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 
         inner_spi = ipip_spi;
         proto = SA_IPIP;
-        satype = K_SADB_X_SATYPE_IPIP;
+        esatype = ET_IPIP;
     }
 
     /* set up IPCOMP SA, if any */
@@ -1421,7 +1459,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = ipcomp_spi;
-        said_next->satype = K_SADB_X_SATYPE_COMP;
+        said_next->esatype = ET_IPCOMP;
         said_next->encalg = compalg;
         said_next->encapsulation = encapsulation;
         said_next->reqid = c->spd.reqid + 2;
@@ -1526,8 +1564,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
                 AES_CBC_BLOCK_SIZE, HMAC_SHA1_KEY_LEN,
                 SADB_X_EALG_AESCBC, SADB_AALG_SHA1HMAC },
         };
-	//static const int esp_max = elemsof(esp_info);
-	//int esp_count;
+	/* static const int esp_max = elemsof(esp_info); */
+	/* int esp_count; */
 
 #ifdef NAT_TRAVERSAL
         u_int8_t natt_type = 0;
@@ -1641,7 +1679,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = esp_spi;
-        said_next->satype = SADB_SATYPE_ESP;
+        said_next->esatype = ET_ESP;
         said_next->replay_window = kernel_ops->replay_window;
         said_next->authalg = ei->authalg;
         said_next->authkeylen = ei->authkeylen;
@@ -1698,8 +1736,19 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	    outgoing_ref_set  = TRUE;
 	}
 
+#ifdef HAVE_LIBNSS
+       if (!kernel_ops->add_sa(said_next, replace)) {
+            memset(said_next->enckey, 0, said_next->enckeylen);
+            memset(said_next->authkey, 0, said_next->authkeylen);
+#else
         if (!kernel_ops->add_sa(said_next, replace))
+#endif
             goto fail;
+#ifdef HAVE_LIBNSS
+       }
+            memset(said_next->enckey, 0, said_next->enckeylen);
+            memset(said_next->authkey, 0, said_next->authkeylen);
+#endif
 
 	/*
 	 * SA refs will have been allocated for this SA.
@@ -1755,7 +1804,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = ah_spi;
-        said_next->satype = SADB_SATYPE_AH;
+        said_next->esatype = ET_AH;
         said_next->replay_window = kernel_ops->replay_window;
         said_next->authalg = authalg;
         said_next->authkeylen = st->st_ah.keymat_len;
@@ -1790,8 +1839,18 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	    outgoing_ref_set  = TRUE;
 	}
 
+#ifdef HAVE_LIBNSS
+       if (!kernel_ops->add_sa(said_next, replace)) {
+            memset(said_next->authkey, 0, said_next->authkeylen);
+#else
         if (!kernel_ops->add_sa(said_next, replace))
+#endif
             goto fail;
+#ifdef HAVE_LIBNSS
+       }
+            memset(said_next->authkey, 0, said_next->authkeylen);
+#endif
+
 
 	/*
 	 * SA refs will have been allocated for this SA.
@@ -1877,7 +1936,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
                               , inner_spi              /* spi */
 			      , proto                  /* proto */
                               , c->spd.this.protocol   /* transport_proto */
-                              , satype                 /* satype */
+                              , esatype                /* esatype */
                               , proto_info             /* " */
 			      , 0                      /* lifetime */
                               , ERO_ADD_INBOUND        /* op */
@@ -1972,7 +2031,7 @@ teardown_half_ipsec_sa(struct state *st, bool inbound)
                           , 256
 			  , IPSEC_PROTO_ANY
                           , c->spd.this.protocol
-                          , SADB_SATYPE_UNSPEC
+                          , ET_UNSPEC
                           , null_proto_info, 0
                           , ERO_DEL_INBOUND, "delete inbound");
     }
@@ -2052,83 +2111,72 @@ void
 init_kernel(void)
 {
     struct utsname un;
+#if defined(NETKEY_SUPPORT) || defined(KLIPS) || defined(KLIPS_MAST)
+    struct stat buf;
+#endif
 
     /* get kernel version */
     uname(&un);
     strncpy(kversion, un.release, sizeof(kversion));
 
-    if (kern_interface == NO_KERNEL)
-    {
-        kernel_ops = &noklips_kernel_ops;
-        return;
-    }
-
-#if defined(KLIPS) && defined(NETKEY_SUPPORT)
-    if(kern_interface == AUTO_PICK)
-    {
-        struct stat buf;
-
+    switch(kern_interface) {
+    case AUTO_PICK:
+#if defined(NETKEY_SUPPORT) || defined(KLIPS) || defined(KLIPS_MAST)
 	/* If we detect NETKEY and KLIPS, we can't continue */
 	if(stat("/proc/net/pfkey", &buf) == 0 &&
 	   stat("/proc/net/pf_key", &buf) == 0) {
 	    /* we don't die, we just log and go to sleep */
 	    openswan_log("Can not run with both NETKEY and KLIPS in the kernel");
 	    openswan_log("Please check your kernel configuration, or specify a stack");
-	    openswan_log("using protostack={klips,netkey}");
+	    openswan_log("using protostack={klips,netkey,mast}");
 	    exit_pluto(0);
 	}
-	
-#if 0
-	/* for now, don't automatically pick MASTKLIPS */
-        if(stat("/proc/sys/net/ipsec/debug_mast",&buf)==0)
-	{
-	    kern_interface = USE_MASTKLIPS;
-	}
-        else
 #endif
-	    if (stat("/proc/net/pfkey", &buf) == 0)
-	{
-	    kern_interface = USE_NETKEY;
-	}
-	else
-	{
-	    kern_interface = USE_KLIPS;
-	}
-    }
-#endif
-
-    switch(kern_interface) {
-    case AUTO_PICK:
-	openswan_log("Kernel interface auto-pick fall-through");
+	openswan_log("Kernel interface auto-pick");
 	/* FALL THROUGH */
 
-#if defined(KLIPS) 
-    case USE_KLIPS:
-	openswan_log("Using KLIPS IPsec interface code on %s"
-		     , kversion);
-	kernel_ops = &klips_kernel_ops;
-	break;
+#if defined(NETKEY_SUPPORT)
+    case USE_NETKEY:
+	if (stat("/proc/net/pfkey", &buf) == 0) {
+	    kern_interface = USE_NETKEY;
+	    openswan_log("Using Linux 2.6 IPsec interface code on %s (experimental code)"
+			 , kversion);
+	    kernel_ops = &netkey_kernel_ops;
+	    break;
+	} else
+	    openswan_log("No Kernel NETKEY interface detected");
+	/* FALL THROUGH */
 #endif
 
 #if defined(KLIPS_MAST) 
     case USE_MASTKLIPS:
-	openswan_log("Using KLIPSng (mast) IPsec interface code on %s"
-		     , kversion);
-	kernel_ops = &mast_kernel_ops;
-	break;
+        if (stat("/proc/sys/net/ipsec/debug_mast", &buf) == 0) {
+	    kern_interface = USE_MASTKLIPS;
+	    openswan_log("Using KLIPSng (mast) IPsec interface code on %s"
+			 , kversion);
+	    kernel_ops = &mast_kernel_ops;
+	    break;
+	} else
+	    openswan_log("No Kernel MASTKLIPS interface detected");
+	/* FALL THROUGH */
 #endif
 
-#if defined(NETKEY_SUPPORT)
-    case USE_NETKEY:
-	openswan_log("Using Linux 2.6 IPsec interface code on %s (experimental code)"
-		     , kversion);
-	kernel_ops = &netkey_kernel_ops;
-	break;
+#if defined(KLIPS) 
+    case USE_KLIPS:
+	if (stat("/proc/net/pf_key", &buf) == 0) {
+	    kern_interface = USE_KLIPS;
+	    openswan_log("Using KLIPS IPsec interface code on %s"
+			 , kversion);
+	    kernel_ops = &klips_kernel_ops;
+	    break;
+	} else
+	    openswan_log("No Kernel KLIPS interface detected");
+	/* FALL THROUGH */
 #endif
 
-
 #if defined(BSD_KAME) 
     case USE_BSDKAME:
+	kern_interface = USE_BSDKAME;
 	openswan_log("Using BSD/KAME IPsec interface code on %s"
 			, kversion);
 	kernel_ops = &bsdkame_kernel_ops;
@@ -2137,22 +2185,25 @@ init_kernel(void)
 
 #if defined(WIN32) && defined(WIN32_NATIVE) 
     case USE_WIN32_NATIVE:
+	kern_interface = USE_WIN32_NATIVE;
 	openswan_log("Using Win2K native IPsec interface code on %s"
 		     , kversion);
 	kernel_ops = &win2k_kernel_ops;
 	break;
 #endif
 
-#if defined(__CYGWIN32__) || defined(linux) || (defined(macintosh) || (defined(__MACH__) && defined(__APPLE__)))
     case NO_KERNEL:
+	kern_interface = NO_KERNEL;
 	openswan_log("Using 'no_kernel' interface code on %s"
 		     , kversion);
 	kernel_ops = &noklips_kernel_ops;
 	break;
-#endif
 
     default:
-	openswan_log("kernel interface '%s' not available"
+	if(kern_interface == AUTO_PICK)
+		openswan_log("kernel interface auto-pick failed - no suitable kernel stack found");
+	else
+		openswan_log("kernel interface '%s' not available"
 		     , enum_name(&kern_interface_names, kern_interface));
 	exit_pluto(5);
     }
@@ -2164,7 +2215,10 @@ init_kernel(void)
 
     /* register SA types that we can negotiate */
     can_do_IPcomp = FALSE;  /* until we get a response from KLIPS */
-    kernel_ops->pfkey_register();
+    if (kernel_ops->pfkey_register)
+    {
+	kernel_ops->pfkey_register();
+    }
 
     if (!kernel_ops->policy_lifetime)
     {
@@ -2229,7 +2283,7 @@ install_inbound_ipsec_sa(struct state *st)
         for (;;)
         {
             struct spd_route *esr;
-            struct connection *o = route_owner(c, &esr, NULL, NULL);
+            struct connection *o = route_owner(c, &c->spd, &esr, NULL, NULL);
 
             if (o == NULL || c==o)
                 break;  /* nobody interesting has a route */
@@ -2244,11 +2298,18 @@ install_inbound_ipsec_sa(struct state *st)
                 break;  /* ??? is this good enough?? */
 #endif
 
-	    if(kernel_ops->overlap_supported
-	       && !LIN(POLICY_TUNNEL, c->policy)
-	       && !LIN(POLICY_TUNNEL, o->policy)) {
-		break;
-	    }
+            if(kernel_ops->overlap_supported) {
+		/* Both are transport mode, allow overlapping.
+		 * [bart] not sure if this is actually intended, but am
+		 *        leaving it in to make it behave like before */
+		if (!LIN(POLICY_TUNNEL, c->policy)
+		    && !LIN(POLICY_TUNNEL, o->policy))
+		    break;
+		/* Both declared that overlapping is OK. */
+		if (LIN(POLICY_OVERLAPIP, c->policy)
+		    && LIN(POLICY_OVERLAPIP, o->policy))
+		    break;
+            }
 		
 	    loglog(RC_LOG_SERIOUS, "route to peer's client conflicts with \"%s\" %s; releasing old connection to free the route"		
 		   , o->name, ip_str(&o->spd.that.host_addr));
@@ -2310,7 +2371,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
     struct spd_route *esr;
     struct spd_route *rosr;
     struct connection *ero      /* who, if anyone, owns our eroute? */
-        , *ro = route_owner(c, &rosr, &ero, &esr);
+        , *ro = route_owner(c, sr, &rosr, &ero, &esr);
     bool eroute_installed = FALSE
         , firewall_notified = FALSE
         , route_installed = FALSE;
@@ -2465,7 +2526,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
                 rosr->routing = RT_UNROUTED;
 
                 /* no need to keep old value */
-                ro = route_owner(c, &rosr, NULL, NULL);
+                ro = route_owner(c, sr, &rosr, NULL, NULL);
             } while (ro != NULL);
         }
     }
@@ -2560,7 +2621,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
                     , bs->said.spi      /* network order */
                     , SA_INT            /* proto */
                     , 0                 /* transport_proto */
-                    , K_SADB_X_SATYPE_INT
+                    , ET_INT
                     , null_proto_info
                     , SHUNT_PATIENCE
                     , ERO_REPLACE, "restore");
@@ -2650,7 +2711,14 @@ install_ipsec_sa(struct state *st, bool inbound_also USED_BY_KLIPS)
 	return TRUE;
     }
 
-    for (sr = &st->st_connection->spd; sr != NULL; sr = sr->next)
+
+    sr = &st->st_connection->spd;
+    if (st->st_connection->remotepeertype == CISCO) {
+    sr = sr->next;
+    }
+
+    /* for (sr = &st->st_connection->spd; sr != NULL; sr = sr->next) */
+    for (; sr != NULL; sr = sr->next)
     {
         DBG(DBG_CONTROL, DBG_log("sr for #%ld: %s"
                                  , st->st_serialno
@@ -2677,6 +2745,12 @@ install_ipsec_sa(struct state *st, bool inbound_also USED_BY_KLIPS)
         }
     }
 
+   if (st->st_connection->remotepeertype == CISCO) {
+	if(!do_command(st->st_connection, &st->st_connection->spd, "updateresolvconf", st)) {
+	DBG(DBG_CONTROL, DBG_log("Updating resolv.conf failed, you may need to update it manually"));
+	}
+   }
+
     return TRUE;
 }
 
@@ -2733,11 +2807,24 @@ delete_ipsec_sa(struct state *st USED_BY_KLIPS, bool inbound_only USED_BY_KLIPS)
 		    {
 			(void) shunt_eroute(c, sr, sr->routing, ERO_REPLACE, "replace with shunt");
 		    }
+
+#ifdef KLIPS_MAST
+		    /* in mast mode we must also delete the iptables rule */
+		    if (kern_interface == USE_MASTKLIPS)
+			    (void) sag_eroute(st, sr, ERO_DELETE, "delete");
+#endif
 		}
 	    }
 	    (void) teardown_half_ipsec_sa(st, FALSE);
 	}
 	(void) teardown_half_ipsec_sa(st, TRUE);
+
+	if (st->st_connection->remotepeertype == CISCO) {
+		if(!do_command(st->st_connection, &st->st_connection->spd, "restoreresolvconf", st)) {
+		DBG(DBG_CONTROL, DBG_log("Restoring resolv.conf failed, you may need to do it manually"));
+		}
+	}
+
 	break;
 #if defined(WIN32) && defined(WIN32_NATIVE)
     case USE_WIN32_NATIVE:
