@@ -10,6 +10,8 @@
  * Copyright (C) 2008 Neil Horman <nhorman@redhat.com>
  * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
  * Copyright (C) 2006-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
+ * Copyright (C) 2010 Mika Ilmaranta <ilmis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -60,6 +62,11 @@
 #include "klips-crypto/aes_cbc.h"
 #include "ike_alg.h"
 
+/* required for Linux 2.6.26 kernel and later */
+#ifndef XFRM_STATE_AF_UNSPEC
+#define XFRM_STATE_AF_UNSPEC	32
+#endif
+
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
@@ -83,7 +90,6 @@ static const struct pfkey_proto_info broad_proto_info[2] = {
         }
 };
 
-
 /* Minimum priority number in SPD used by pluto. */
 #define MIN_SPD_PRIORITY 1024
 
@@ -398,6 +404,7 @@ netlink_policy(struct nlmsghdr *hdr, bool enoent_ok, const char *text_said)
     struct {
 	struct nlmsghdr n;
 	struct nlmsgerr e;
+	char data[1024];
     } rsp;
     int error;
 
@@ -434,9 +441,9 @@ netlink_policy(struct nlmsghdr *hdr, bool enoent_ok, const char *text_said)
  * @param that_host ip_address
  * @param that_client ip_subnet
  * @param spi
- * @param proto int (Currently unused) Contains protocol (u=tcp, 17=udp, etc...)
- * @param transport_proto int (Currently unused) 0=tunnel, 1=transport
- * @param satype int
+ * @param proto int (4=tunnel, 50=esp, 108=ipcomp, etc ...)
+ * @param transport_proto int (Currently unused) Contains protocol (u=tcp, 17=udp, etc...)
+ * @param esatype int
  * @param pfkey_proto_info proto_info 
  * @param use_lifetime time_t (Currently unused)
  * @param pluto_sadb_opterations sadb_op (operation - ie: ERO_DELETE)
@@ -449,9 +456,9 @@ netlink_raw_eroute(const ip_address *this_host
 		   , const ip_address *that_host
 		   , const ip_subnet *that_client
 		   , ipsec_spi_t spi
-		   , unsigned int proto UNUSED
+		   , unsigned int proto
 		   , unsigned int transport_proto UNUSED
-		   , unsigned int satype
+		   , enum eroute_type esatype
 		   , const struct pfkey_proto_info *proto_info
 		   , time_t use_lifetime UNUSED
 		   , enum pluto_sadb_operations sadb_op
@@ -469,35 +476,103 @@ netlink_raw_eroute(const ip_address *this_host
     int dir;
     int family;
     int policy;
+    int satype;
     bool ok;
     bool enoent_ok;
+    ip_subnet local_that_client;
 
     policy = IPSEC_POLICY_IPSEC;
 
-    if (satype == K_SADB_X_SATYPE_INT)
-    {
+    switch(esatype) {
+    case ET_UNSPEC:
+	    satype = SADB_SATYPE_UNSPEC;
+	    break;
+
+    case ET_AH:
+	    satype = SADB_SATYPE_AH;
+	    break;
+
+    case ET_ESP:
+	    satype = SADB_SATYPE_ESP;
+	    break;
+
+    case ET_IPCOMP:
+	    satype = SADB_X_SATYPE_IPCOMP;
+	    break;
+
+    case ET_IPIP:
+	    satype = K_SADB_X_SATYPE_IPIP;
+	    break;
+
+    case ET_INT:
 	/* shunt route */
-	switch (ntohl(spi))
-	{
-	case SPI_PASS:
-	    policy = IPSEC_POLICY_NONE;
-	    break;
-	case SPI_DROP:
-	case SPI_REJECT:
-	default:
-	    policy = IPSEC_POLICY_DISCARD;
-	    break;
-	case SPI_TRAP:
-	case SPI_TRAPSUBNET:
-	  if (sadb_op == ERO_ADD_INBOUND || sadb_op == ERO_DEL_INBOUND)
+	    switch (ntohl(spi))
 	    {
-		return TRUE;
+	    case SPI_PASS:
+		    policy = IPSEC_POLICY_NONE;
+		    break;
+	    case SPI_DROP:
+	    case SPI_REJECT:
+	    default:
+		    policy = IPSEC_POLICY_DISCARD;
+		    break;
+	    case SPI_TRAP:
+	    case SPI_TRAPSUBNET:
+		    if (sadb_op == ERO_ADD_INBOUND || sadb_op == ERO_DEL_INBOUND)
+		    {
+			    return TRUE;
+		    }
+		    break;
+	    /* Do we really need %hold under NETKEY? Seems not so we just ignore. */
+	    case SPI_HOLD:
+	            return TRUE;
 	    }
 	    break;
-	/* Do we really need %hold under NETKEY? Seems not, so just ignore */
-	case SPI_HOLD:
-		return TRUE; 
+    }
+
+    /* Bug #1004 fix.
+     * There really isn't "client" with NETKEY and transport mode
+     * so eroute must be done to natted, visible ip. If we don't hide
+     * internal IP, communication doesn't work.
+     */
+    if((proto == ET_ESP || proto == ET_IPCOMP)
+	&& !addrinsubnet(that_host, that_client)
+	&& !isanyaddr(that_host))
+    {
+	addrtosubnet(that_host, &local_that_client);
+	DBG(DBG_NETKEY,
+	    {
+		char that_client_t[SUBNETTOT_BUF];
+		char local_that_client_t[SUBNETTOT_BUF];
+
+		subnettot(that_client, 0, that_client_t
+		    , sizeof(that_client_t));
+		subnettot(&local_that_client, 0, local_that_client_t
+		    , sizeof(local_that_client_t));
+		DBG_log(
+		    "netlink_raw_eroute: proto = %u,"
+		    " substituting %s with %s"
+		    , proto, that_client_t, local_that_client_t);
+	    });
+
+	/* We have a bug somewhere in the code where NATD port of remote
+	 * gets applied to protoport port of host. Happily it only seem
+	 * to affect natted transport mode so we can undo that corruption
+	 * here. This corruption of that_host port is random and we couldn't
+	 * find the place where it happens. Port of that_client is anyway the
+	 * port we should use here.
+	 * Bug #1101. Tuomo
+	 */
+	if(portof(&that_client->addr) != portof(that_host)) {
+	    openswan_log("%s: WARNING: that_client port %u and that_host"
+			 " port %u don't match. Using that_client port."
+			 , __FUNCTION__
+			 , ntohs(portof(&that_client->addr))
+			 , ntohs(portof(that_host)));
+	    setportof(portof(&that_client->addr), &local_that_client.addr);
 	}
+
+ 	that_client = &local_that_client;
     }
 
     memset(&req, 0, sizeof(req));
@@ -584,6 +659,7 @@ netlink_raw_eroute(const ip_address *this_host
 	    tmpl[i].optional =
 		proto_info[i].proto == IPPROTO_COMP && dir != XFRM_POLICY_OUT;
 	    tmpl[i].aalgos = tmpl[i].ealgos = tmpl[i].calgos = ~0;
+	    tmpl[i].family = that_host->u.v4.sin_family;
 	    tmpl[i].mode =
 		proto_info[i].encapsulation == ENCAPSULATION_MODE_TUNNEL;
 
@@ -627,7 +703,7 @@ netlink_raw_eroute(const ip_address *this_host
 	    break;
 	}
 	else if (proto_info[0].encapsulation != ENCAPSULATION_MODE_TUNNEL
-		 && satype != K_SADB_X_SATYPE_INT)
+	&& esatype != ET_INT)
 	{
 	    break;
 	}
@@ -667,9 +743,22 @@ netlink_add_sa(struct kernel_sa *sa, bool replace)
     ip2xfrm(sa->dst, &req.p.id.daddr);
 
     req.p.id.spi = sa->spi;
-    req.p.id.proto = satype2proto(sa->satype);
+    req.p.id.proto = esatype2proto(sa->esatype);
     req.p.family = sa->src->u.v4.sin_family;
-    req.p.mode = (sa->encapsulation == ENCAPSULATION_MODE_TUNNEL);
+    /*
+     * This requires ipv6 modules. It is required to support 6in4 and 4in6
+     * tunnels in linux 2.6.25+
+     */
+    if (sa->encapsulation == ENCAPSULATION_MODE_TUNNEL)
+    {
+	req.p.mode = XFRM_MODE_TUNNEL;
+	req.p.flags |= XFRM_STATE_AF_UNSPEC;
+    }
+    else
+    {
+	req.p.mode = XFRM_MODE_TRANSPORT;
+    }
+
     req.p.replay_window = sa->replay_window > 32 ? 32 : sa->replay_window; 
     req.p.reqid = sa->reqid;
     req.p.lft.soft_byte_limit = XFRM_INF;
@@ -752,7 +841,7 @@ netlink_add_sa(struct kernel_sa *sa, bool replace)
 	attr = (struct rtattr *)((char *)attr + attr->rta_len);
     }
 
-    if (sa->satype == SADB_X_SATYPE_IPCOMP)
+    if (sa->esatype == ET_IPCOMP)
     {
 	struct xfrm_algo algo;
 	const char *name;
@@ -1038,8 +1127,8 @@ netlink_acquire(struct nlmsghdr *n)
 
     if (!(ugh = xfrm_to_ip_address(family, srcx, &src))
 	&& !(ugh = xfrm_to_ip_address(family, dstx, &dst))
-	&& (ugh = add_port (family, &src, acquire->sel.sport))
-	&& (ugh = add_port (family, &dst, acquire->sel.dport))
+	&& !(ugh = add_port (family, &src, acquire->sel.sport))
+	&& !(ugh = add_port (family, &dst, acquire->sel.dport))
 	&& !(ugh = src_proto == dst_proto? NULL : "src and dst protocols differ")
 	&& !(ugh = addrtosubnet(&src, &ours))
 	&& !(ugh = addrtosubnet(&dst, &his)))
@@ -1310,9 +1399,8 @@ static bool
 netlink_sag_eroute(struct state *st, struct spd_route *sr
 		  , unsigned op, const char *opname)
 {
-    unsigned int
-        inner_proto,
-        inner_satype;
+    unsigned int inner_proto;
+    enum eroute_type inner_esatype;
     ipsec_spi_t inner_spi;
     struct pfkey_proto_info proto_info[4];
     int i;
@@ -1327,14 +1415,14 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
     tunnel = FALSE;
 
     inner_proto = 0;
-    inner_satype= 0;
+    inner_esatype = ET_UNSPEC;
     inner_spi = 0;
 
     if (st->st_ah.present)
     {
         inner_spi = st->st_ah.attrs.spi;
         inner_proto = SA_AH;
-        inner_satype = SADB_SATYPE_AH;
+        inner_esatype = ET_AH;
 
         i--;
         proto_info[i].proto = IPPROTO_AH;
@@ -1347,7 +1435,7 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_esp.attrs.spi;
         inner_proto = SA_ESP;
-        inner_satype = SADB_SATYPE_ESP;
+        inner_esatype = ET_ESP;
 
         i--;
         proto_info[i].proto = IPPROTO_ESP;
@@ -1360,7 +1448,7 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_ipcomp.attrs.spi;
         inner_proto = SA_COMP;
-        inner_satype = K_SADB_X_SATYPE_COMP;
+        inner_esatype = ET_IPCOMP;
 
         i--;
         proto_info[i].proto = IPPROTO_COMP;
@@ -1380,7 +1468,7 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
 
         inner_spi = st->st_tunnel_out_spi;
         inner_proto = SA_IPIP;
-        inner_satype = K_SADB_X_SATYPE_IPIP;
+        inner_esatype = ET_IPIP;
 
         proto_info[i].encapsulation = ENCAPSULATION_MODE_TUNNEL;
         for (j = i + 1; proto_info[j].proto; j++)
@@ -1390,7 +1478,8 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
     }
 
     return eroute_connection(sr
-        , inner_spi, inner_proto, inner_satype, proto_info + i
+        , inner_spi, inner_proto
+	, inner_esatype, proto_info + i
         , op, opname);
 }
 
@@ -1490,7 +1579,8 @@ netlink_shunt_eroute(struct connection *c
         {
             esr->routing = RT_ROUTED_PROSPECTIVE;
             return netlink_shunt_eroute(ue, esr
-                                , RT_ROUTED_PROSPECTIVE, ERO_REPLACE, "restoring eclipsed");
+                                , RT_ROUTED_PROSPECTIVE, ERO_REPLACE
+                                , "restoring eclipsed");
         }
     }
 
@@ -1512,7 +1602,7 @@ netlink_shunt_eroute(struct connection *c
 			      , htonl(spi)
 			      , SA_INT
 			      , sr->this.protocol
-			      , K_SADB_X_SATYPE_INT
+			      , ET_INT
 			      , null_proto_info, 0, op, buf2) )
       { return FALSE; }
 
@@ -1537,7 +1627,7 @@ netlink_shunt_eroute(struct connection *c
 			      , htonl(spi)
 			      , SA_INT
 			      , sr->this.protocol
-			      , K_SADB_X_SATYPE_INT
+			      , ET_INT
 			      , null_proto_info, 0, op, buf2);
     }
 }
@@ -1881,14 +1971,16 @@ const struct kernel_ops netkey_kernel_ops = {
     process_queue: NULL,
     grp_sa: NULL,
     get_spi: netlink_get_spi,
+    exceptsocket: NULL,
     docommand: netkey_do_command,
     process_ifaces: netlink_process_raw_ifaces,
-
-    /* XXX these needed to be added */
     shunt_eroute: netlink_shunt_eroute,
-    sag_eroute: netlink_sag_eroute,   /* pfkey_sag_eroute, */
-    eroute_idle: netlink_eroute_idle,  /* pfkey_was_eroute_idle,*/
+    sag_eroute: netlink_sag_eroute,
+    eroute_idle: netlink_eroute_idle,
     set_debug: NULL,    /* pfkey_set_debug, */
-    remove_orphaned_holds: NULL, /* pfkey_remove_orphaned_holds,*/
+    /* We should implement netlink_remove_orphaned_holds
+     * if netlink  specific changes are needed.
+     */
+    remove_orphaned_holds: pfkey_remove_orphaned_holds,
 };
 #endif /* linux && NETKEY_SUPPORT */
