@@ -149,6 +149,28 @@ static sparse_names pfkey_ext_names = {
 
 #undef NE
 
+/* convert ET_* enums to K_SADB_SATYPE_* numbers */
+static inline unsigned eroute_type_to_pfkey_satype(enum eroute_type esatype)
+{
+	switch(esatype) {
+	default:
+		bad_case(esatype);
+		return -1;
+	case ET_UNSPEC:
+		return K_SADB_SATYPE_UNSPEC;
+	case ET_AH:
+		return K_SADB_SATYPE_AH;
+	case ET_ESP:
+		return K_SADB_SATYPE_ESP;
+	case ET_IPCOMP:
+		return K_SADB_X_SATYPE_COMP;
+	case ET_INT:
+		return K_SADB_X_SATYPE_INT;
+	case ET_IPIP:
+		return K_SADB_X_SATYPE_IPIP;
+	}
+}
+
 void
 init_pfkey(void)
 {
@@ -806,6 +828,7 @@ static int kernelop2klips(enum pluto_sadb_operations op)
     return klips_op;
 }
 
+#ifdef KLIPS
 void klips_pfkey_register(void)
 {
     klips_register_proto(K_SADB_SATYPE_AH, "AH");
@@ -814,6 +837,7 @@ void klips_pfkey_register(void)
     klips_register_proto(K_SADB_X_SATYPE_COMP, "IPCOMP");
     klips_register_proto(K_SADB_X_SATYPE_IPIP, "IPIP");
 }
+#endif
 
 bool
 pfkey_raw_eroute(const ip_address *this_host
@@ -823,7 +847,7 @@ pfkey_raw_eroute(const ip_address *this_host
 		 , ipsec_spi_t spi
 		 , unsigned int proto UNUSED
 		 , unsigned int transport_proto
-		 , unsigned int satype
+		 , enum eroute_type esatype
 		 , const struct pfkey_proto_info *proto_info UNUSED
 		 , time_t use_lifetime UNUSED
 		 , enum pluto_sadb_operations op
@@ -839,6 +863,7 @@ pfkey_raw_eroute(const ip_address *this_host
 
     int sport = ntohs(portof(&this_client->addr));
     int dport = ntohs(portof(&that_client->addr));
+    int satype;
 
     networkof(this_client, &sflow_ska);
     maskof(this_client, &smask_ska);
@@ -848,7 +873,11 @@ pfkey_raw_eroute(const ip_address *this_host
     maskof(that_client, &dmask_ska);
     setportof(dport ? ~0:0, &dmask_ska);
 
-
+    satype = eroute_type_to_pfkey_satype(esatype);
+    if (satype < 0 || satype > K_SADB_SATYPE_MAX) {
+	impossible();
+	return FALSE;
+    }
 
     if (!pfkey_msg_start(klips_op & KLIPS_OP_MASK, satype
 		       , "pfkey_msg_hdr flow", text_said, extensions))
@@ -914,11 +943,18 @@ pfkey_raw_eroute(const ip_address *this_host
 
 bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 {
+    unsigned klips_satype;
     struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
     pfkey_buf pfb;
     bool success = FALSE;
 
-    success = pfkey_msg_start(replace ? K_SADB_UPDATE : K_SADB_ADD, sa->satype
+    klips_satype = eroute_type_to_pfkey_satype(sa->esatype);
+    if (klips_satype > K_SADB_SATYPE_MAX) {
+	impossible();
+	return FALSE;
+    }
+
+    success = pfkey_msg_start(replace ? K_SADB_UPDATE : K_SADB_ADD, klips_satype
 			      , "pfkey_msg_hdr Add SA"
 			      , sa->text_said, extensions);
 
@@ -1052,8 +1088,21 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 bool pfkey_grp_sa(const struct kernel_sa *sa0, const struct kernel_sa *sa1)
 {
     struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
+    unsigned klips_satype0, klips_satype1;
 
-    return pfkey_msg_start(K_SADB_X_GRPSA, sa1->satype
+    klips_satype0 = eroute_type_to_pfkey_satype(sa0->esatype);
+    if (klips_satype0 > K_SADB_SATYPE_MAX) {
+	impossible();
+	return FALSE;
+    }
+
+    klips_satype1 = eroute_type_to_pfkey_satype(sa1->esatype);
+    if (klips_satype1 > K_SADB_SATYPE_MAX) {
+	impossible();
+	return FALSE;
+    }
+
+    return pfkey_msg_start(K_SADB_X_GRPSA, klips_satype1
 	, "pfkey_msg_hdr group", sa1->text_said, extensions)
 
     && pfkey_build(pfkey_sa_build(&extensions[K_SADB_EXT_SA]
@@ -1066,7 +1115,7 @@ bool pfkey_grp_sa(const struct kernel_sa *sa0, const struct kernel_sa *sa1)
 	, "pfkey_addr_d group", sa1->text_said, extensions)
 
     && pfkey_build(pfkey_x_satype_build(&extensions[K_SADB_X_EXT_SATYPE2]
-	    , sa0->satype)
+	    , klips_satype0)
 	, "pfkey_satype group", sa0->text_said, extensions)
 
     && pfkey_build(pfkey_sa_build(&extensions[K_SADB_X_EXT_SA2]
@@ -1221,7 +1270,7 @@ pfkey_shunt_eroute(struct connection *c
 			      , htonl(spi)
 			      , SA_INT
 			      , 0 /* transport_proto is not relevant */
-			      , SADB_X_SATYPE_INT, null_proto_info
+			      , ET_INT, null_proto_info
 			      , 0      /* use lifetime */
 			      , inop
 			      , opname);
@@ -1247,7 +1296,7 @@ pfkey_shunt_eroute(struct connection *c
 			      , htonl(spi)
 			      , SA_INT
 			      , sr->this.protocol
-			      , K_SADB_X_SATYPE_INT
+			      , ET_INT
 			      , null_proto_info, 0, op, buf2);
     }
 }
@@ -1257,9 +1306,8 @@ bool
 pfkey_sag_eroute(struct state *st, struct spd_route *sr
 		 , unsigned op, const char *opname)
 {
-    unsigned int
-        inner_proto,
-        inner_satype;
+    unsigned int inner_proto;
+    enum eroute_type inner_esatype;
     ipsec_spi_t inner_spi;
     struct pfkey_proto_info proto_info[4];
     int i;
@@ -1274,14 +1322,14 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
     tunnel = FALSE;
 
     inner_proto = 0;
-    inner_satype= 0;
+    inner_esatype = ET_UNSPEC;
     inner_spi = 0;
 
     if (st->st_ah.present)
     {
         inner_spi = st->st_ah.attrs.spi;
         inner_proto = SA_AH;
-        inner_satype = SADB_SATYPE_AH;
+        inner_esatype = ET_AH;
 
         i--;
         proto_info[i].proto = IPPROTO_AH;
@@ -1294,7 +1342,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_esp.attrs.spi;
         inner_proto = SA_ESP;
-        inner_satype = SADB_SATYPE_ESP;
+        inner_esatype = ET_ESP;
 
         i--;
         proto_info[i].proto = IPPROTO_ESP;
@@ -1307,7 +1355,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_ipcomp.attrs.spi;
         inner_proto = SA_COMP;
-        inner_satype = K_SADB_X_SATYPE_COMP;
+        inner_esatype = ET_IPCOMP;
 
         i--;
         proto_info[i].proto = IPPROTO_COMP;
@@ -1327,7 +1375,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
 
         inner_spi = st->st_tunnel_out_spi;
         inner_proto = SA_IPIP;
-        inner_satype = K_SADB_X_SATYPE_IPIP;
+        inner_esatype = ET_IPIP;
 
         proto_info[i].encapsulation = ENCAPSULATION_MODE_TUNNEL;
         for (j = i + 1; proto_info[j].proto; j++)
@@ -1338,7 +1386,7 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
 
     return eroute_connection(sr
 			     , inner_spi, inner_proto
-			     , inner_satype, proto_info + i
+			     , inner_esatype, proto_info + i
 			     , op, opname);
 }
 
