@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * RCSID $Id: kernel_pfkey.h,v 1.1 2003/07/31 18:14:23 mcr Exp $
  */
 
 #ifdef KLIPS
@@ -44,7 +42,7 @@ extern bool pfkey_raw_eroute(const ip_address *this_host
 			     , ipsec_spi_t spi
 			     , unsigned int proto UNUSED
 			     , unsigned int transport_proto
-			     , unsigned int satype
+			     , enum eroute_type esatype
 			     , const struct pfkey_proto_info *proto_info UNUSED
 			     , time_t use_lifetime UNUSED
 			     , enum pluto_sadb_operations op
