@@ -35,7 +35,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
+#include <resolv.h>
 
 #include <glob.h>
 #ifndef GLOB_ABORTED
