@@ -2,9 +2,9 @@
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  * Copyright (C) 2005-2007 Michael Richardson
- * Copyright (C) 2006 Bart Trojanowski
- * Copyright (C) 2008 Paul Wouters
- * Copyright (C) 2008 David McCullough.
+ * Copyright (C) 2006-2010 Bart Trojanowski
+ * Copyright (C) 2008-2010 Paul Wouters
+ * Copyright (C) 2008-2010 David McCullough.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -301,7 +301,7 @@ ensure_writeable_parent_directory(char *path)
 static void
 open_peerlog(struct connection *c)
 {
-    //syslog(LOG_INFO, "opening log file for conn %s", c->name);
+    /* syslog(LOG_INFO, "opening log file for conn %s", c->name); */
 
     if (c->log_file_name == NULL)
     {
@@ -331,12 +331,14 @@ open_peerlog(struct connection *c)
 	    + 1;
 	c->log_file_name = alloc_bytes(lf_len, "per-peer log file name");
 
-	//fprintf(stderr, "base dir |%s| dname |%s| peername |%s|"
-	//	, base_perpeer_logdir, dname, peername);
+#if 0
+	fprintf(stderr, "base dir |%s| dname |%s| peername |%s|"
+		, base_perpeer_logdir, dname, peername);
+#endif
 	snprintf(c->log_file_name, lf_len, "%s/%s/%s.log"
 		 , base_perpeer_logdir, dname, peername);
 
-	//syslog(LOG_DEBUG, "conn %s logfile is %s", c->name, c->log_file_name);
+	/* syslog(LOG_DEBUG, "conn %s logfile is %s", c->name, c->log_file_name); */
     }
 
     /* now open the file, creating directories if necessary */
@@ -630,7 +632,8 @@ passert_fail(const char *pred_str, const char *file_str, unsigned long line_no)
 	dying_breath = TRUE;
 	show_status();
     }
-    osw_abort();	/* exiting correctly doesn't always work */
+    /* exiting correctly doesn't always work */
+    openswan_log_abort(file_str, line_no);
 }
 
 void
@@ -887,6 +890,7 @@ struct log_conn_info {
 		p1_encrypt,
 		p1_auth,
 		p1_up,
+		p1_down
 	} phase1;
 
 	enum {
@@ -908,9 +912,16 @@ connection_state(struct state *st, void *data)
 {
 	struct log_conn_info *lc = data;
 
-	if (!st || st == lc->ignore || st->st_connection != lc->conn)
+	if (!st || st == lc->ignore || !st->st_connection || !lc->conn)
 		return;
 
+	if (st->st_connection != lc->conn) {
+		if (lc->conn->host_pair != st->st_connection->host_pair ||
+			!same_peer_ids(lc->conn, st->st_connection, NULL))
+		    return;
+		/* phase1 is shared with another connnection */
+	}
+
 	/* ignore undefined states (ie., just deleted) */
 	if (st->st_state == STATE_UNDEFINED)
 		return;
@@ -930,8 +941,11 @@ connection_state(struct state *st, void *data)
 			if (IS_ISAKMP_AUTHENTICATED(st->st_state) && lc->phase1 < p1_auth)
 				lc->phase1 = p1_auth;
 		}
-	}
+	} else lc->phase1 = p1_down;
 
+	/* only phase one shares across connections, so we can quit now */
+	if (st->st_connection != lc->conn)
+		return;
 
 	if (IS_PHASE15(st->st_state)) {
 		if (lc->tunnel < tun_phase15)
@@ -961,6 +975,7 @@ log_state(struct state *st, enum state_kind new_state)
 	const char *tun = NULL, *p1 = NULL, *p2 = NULL;
 	enum state_kind save_state;
 
+
 	if (!st || !st->st_connection || !st->st_connection->name)
 		return;
 
@@ -985,31 +1000,47 @@ log_state(struct state *st, enum state_kind new_state)
 	case tun_phase15: tun = "phase15"; break;
 	case tun_phase2:  tun = "phase2";  break;
 	case tun_up:      tun = "up";      break;
-	default:          tun = "unknown"; break;
+	case tun_down:    tun = "down";    break; /* not set anywhere, default */
+	default:          tun = "unchanged"; break;
 	}
 
 	switch (lc.phase1) {
-	case p1_init:     p1 = "init";    break;
-	case p1_encrypt:  p1 = "encrypt"; break;
-	case p1_auth:     p1 = "auth";    break;
-	default:          p1 = "unknown"; break;
+	case p1_init:     p1 = "init";     break;
+	case p1_encrypt:  p1 = "encrypt";  break;
+	case p1_auth:     p1 = "auth";     break;
+	case p1_up:       p1 = "up";       break;
+	case p1_down:       p1 = "down";   break;
+	default:          p1 = "unchanged";  break;
 	}
 
 	switch (lc.phase2) {
-	case p2_neg:      p2 = "neg";     break;
-	default:          p2 = "unknown";  break;
+	case p2_neg:      p2 = "neg";      break;
+	case p2_up:       p2 = "up";       break;
+	default:          p2 = "down";     break;
 	}
 
-	snprintf(buf, sizeof(buf), "/bin/statsd "
+
+	snprintf(buf, sizeof(buf), "/bin/openswan-statsd "
 			"%s ipsec-tunnel-%s if_stats /proc/net/dev/%s \\; "
 			"%s ipsec-tunnel-%s tunnel %s \\; "
 			"%s ipsec-tunnel-%s phase1 %s \\; "
-			"%s ipsec-tunnel-%s phase2 %s",
+			"%s ipsec-tunnel-%s phase2 %s \\; "
+			"%s ipsec-tunnel-%s nfmark-me/him 0x%x/0x%x",
+
 			conn->interface ? "push" : "drop", conn->name,
-		   			conn->interface ? conn->interface->ip_dev->id_vname : "",
+					conn->interface ? conn->interface->ip_dev->id_vname : "",
 			tun ? "push" : "drop", conn->name, tun ? tun : "",
 			p1  ? "push" : "drop", conn->name, p1  ? p1  : "",
-			p2  ? "push" : "drop", conn->name, p2  ? p2  : "");
+			p2  ? "push" : "drop", conn->name, p2  ? p2  : "",
+			(st->st_ref || st->st_refhim) ? "push" : "drop", conn->name,
+				st->st_ref == IPSEC_SAREF_NA ? IPSEC_SAREF_NA
+				: st->st_ref == IPSEC_SAREF_NULL ? 0u
+				: IPsecSAref2NFmark(st->st_ref) | IPSEC_NFMARK_IS_SAREF_BIT
+				,
+				st->st_refhim == IPSEC_SAREF_NA ? IPSEC_SAREF_NA
+				: st->st_refhim == IPSEC_SAREF_NULL ? 0u
+				: IPsecSAref2NFmark(st->st_refhim) | IPSEC_NFMARK_IS_SAREF_BIT 
+		);
 	system(buf);
 }
 
