@@ -34,7 +34,7 @@
 #include <sys/un.h>
 #include <fcntl.h>
 #include <getopt.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
+#include <resolv.h>
 
 #include <openswan.h>
 
@@ -139,6 +139,8 @@ usage(const char *mess)
 	    " [--use-auto]"
 	    " [--use-klips]"
 	    " [--use-netkey]"
+	    " [--use-mast]"
+	    " [--use-bsdkame]"
 	    " [--use-nostack]"         /* old --no_klips */
 	    " \\\n\t"
 	    "[--interface <ifname|ifaddr>]"
@@ -321,6 +323,10 @@ main(int argc, char **argv)
 	capng_updatev(CAPNG_ADD, CAPNG_EFFECTIVE|CAPNG_PERMITTED,
 			CAP_NET_BIND_SERVICE, CAP_NET_ADMIN, CAP_NET_RAW,
 			CAP_IPC_LOCK, -1);
+	/* our children must be able to CAP_NET_ADMIN to change routes.
+	 */
+	capng_updatev(CAPNG_ADD, CAPNG_BOUNDING_SET,
+			CAP_NET_ADMIN, -1);
 	capng_apply(CAPNG_SELECT_BOTH);
 #endif
 
@@ -363,6 +369,8 @@ main(int argc, char **argv)
 	    { "usenetkey", no_argument, NULL, 'K' },
 	    { "use-netkey", no_argument, NULL, 'K' },
 	    { "use-mast",   no_argument, NULL, 'M' },
+	    { "use-mastklips",   no_argument, NULL, 'M' },
+	    { "use-bsdkame",   no_argument, NULL, 'F' },
 	    { "interface", required_argument, NULL, 'i' },
 	    { "ikeport", required_argument, NULL, 'p' },
 	    { "ctlbase", required_argument, NULL, 'b' },
@@ -500,6 +508,10 @@ main(int argc, char **argv)
 	    kern_interface = USE_MASTKLIPS;
 	    continue;
 
+	case 'F':       /* --use-bsdkame */
+	    kern_interface = USE_BSDKAME;
+	    continue;
+
 	case 'K':       /* --use-netkey */
 	    kern_interface = USE_NETKEY;
 	    continue;
@@ -778,7 +790,7 @@ main(int argc, char **argv)
 
 #ifdef HAVE_LIBNSS
 	char buf[100];
-	snprintf(buf, sizeof(buf), "sql:%s",oco->confddir);
+	snprintf(buf, sizeof(buf), "%s",oco->confddir);
 	loglog(RC_LOG_SERIOUS,"nss directory plutomain: %s",buf);
 	SECStatus nss_init_status= NSS_InitReadWrite(buf);
 	if (nss_init_status != SECSuccess) {
@@ -817,14 +829,11 @@ main(int argc, char **argv)
 					IPSECLIBDIR"/spi",
 					IPSECLIBDIR"/spigrp",
 					IPSECLIBDIR"/_startklips",
-					IPSECLIBDIR"/_startklips.old",
 					IPSECLIBDIR"/_startnetkey",
 					IPSECLIBDIR"/tncfg",
 					IPSECLIBDIR"/_updown",
 					IPSECLIBDIR"/_updown.klips",
-					IPSECLIBDIR"/_updown.klips.old",
 					IPSECLIBDIR"/_updown.mast",
-					IPSECLIBDIR"/_updown.mast.old",
 					IPSECLIBDIR"/_updown.netkey", 
 					IPSECLIBDIR"/verify",
 					IPSECLIBDIR"/whack",
@@ -845,24 +854,19 @@ main(int argc, char **argv)
      * ipsec barf was one, but it no longer does.
      */
     {
-#ifdef PLUTO_SENDS_VENDORID
-# ifdef HAVE_LIBNSS
-	if(Pluto_IsFIPS()) {
-	openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
-		, ipsec_version_code() , compile_time_interop_options, getpid());
-	} else {
-# endif
-        const char *v = init_pluto_vendorid();
 	const char *vc = ipsec_version_code();
-
-        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s) pid:%u"
-		     , vc
-		     , compile_time_interop_options
-		     , v, getpid());
+#ifdef PLUTO_SENDS_VENDORID
+	const char *v = init_pluto_vendorid();
+	openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s) pid:%u"
+		     , vc, compile_time_interop_options, v, getpid());
 #else
-        openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
-		     , ipsec_version_code()
-		     , compile_time_interop_options, getpid());
+	openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
+		     , vc, compile_time_interop_options, getpid());
+#endif
+#ifdef HAVE_LIBNSS
+	if(Pluto_IsFIPS()) {
+		openswan_log("Pluto is running in FIPS mode");
+	}
 #endif
 
 	if((vc[0]=='c' && vc[1]=='v' && vc[2]=='s') ||
@@ -876,12 +880,6 @@ main(int argc, char **argv)
 	     */
 	    openswan_log("@(#) built on "__DATE__":" __TIME__ " by " BUILDER);
 	}
-#ifdef HAVE_LIBNSS
-#ifdef PLUTO_SENDS_VENDORID
-	}
-#endif
-#endif
-
 #if defined(USE_1DES)
 	openswan_log("WARNING: 1DES is enabled");
 #endif
@@ -891,6 +889,35 @@ main(int argc, char **argv)
 	openswan_log("core dump dir: %s", coredir);
     }
 
+   /* Check for SAREF support */
+#ifdef KLIPS_MAST
+#include <ipsec_saref.h>
+    {
+	int e, sk, saref;
+	saref = 1;
+	errno=0;
+
+	sk = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+	e = setsockopt(sk, IPPROTO_IP, IP_IPSEC_REFINFO, &saref, sizeof(saref));
+	if (e == -1 ) {
+		openswan_log("SAref support [disabled]: %s" , strerror(errno));
+	}
+	else {
+		openswan_log("SAref support [enabled]");
+	}
+	errno=0;
+	e = setsockopt(sk, IPPROTO_IP, IP_IPSEC_BINDREF, &saref, sizeof(saref));
+	if (e == -1 ) {
+		openswan_log("SAbind support [disabled]: %s" , strerror(errno));
+	}
+	else {
+		openswan_log("SAbind support [enabled]");
+	}
+
+
+	close(sk);
+}
+#endif
 /** Initialize all of the various features */
 
 #ifdef NAT_TRAVERSAL
