@@ -26,7 +26,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <resolv.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
 
 #include <openswan.h>
 
@@ -313,7 +312,9 @@ info_handle(int infoctlfd)
 	{
 	case IPSEC_CMD_QUERY_HOSTPAIR:
 	    info_lookuphostpair(&ipcq);
-	    write(infofd, &ipcq, ipcq.head.ipm_msg_len);
+	    if(write(infofd, &ipcq, ipcq.head.ipm_msg_len) == -1 ) {
+            plog("info_handle: write error");
+	    }
 	    break;
 	    
 	default:
