@@ -30,7 +30,7 @@
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
+#include <resolv.h>
 #include <fcntl.h>
 
 #include <openswan.h>
@@ -154,10 +154,15 @@ static bool writewhackrecord(char *buf, int buflen)
     time(&n);
     header[2]=n;
 
-    //DBG_log("buflen: %u abuflen: %u\n", header[0], abuflen);
+    /* DBG_log("buflen: %u abuflen: %u\n", header[0], abuflen); */
     
-    fwrite(header, sizeof(u_int32_t)*3, 1, whackrecordfile);
-    fwrite(buf, abuflen, 1, whackrecordfile);
+    if(fwrite(header, sizeof(u_int32_t)*3, 1, whackrecordfile) < 1) {
+	DBG_log("writewhackrecord: fwrite error when writing header");
+    }
+
+    if(fwrite(buf, abuflen, 1, whackrecordfile) < 1) {
+	DBG_log("writewhackrecord: fwrite error when writing buf");
+    }
     
     return TRUE;
 }
@@ -625,7 +630,7 @@ void whack_process(int whackfd, struct whack_message msg)
 	if (!listening)
 	    whack_log(RC_DEAF, "need --listen before opportunistic initiation");
 	else
-	    initiate_ondemand(&msg.oppo_my_client, &msg.oppo_peer_client, 0
+	    (void)initiate_ondemand(&msg.oppo_my_client, &msg.oppo_peer_client, 0
 		, FALSE
 		, msg.whack_async? NULL_FD : dup_any(whackfd), "whack");
     }
@@ -659,7 +664,7 @@ whack_handle(int whackctlfd)
     int whackfd = accept(whackctlfd, (struct sockaddr *)&whackaddr, &whackaddrlen);
     /* Note: actual value in n should fit in int.  To print, cast to int. */
     ssize_t n;
-    //static int msgnum=0;
+    /* static int msgnum=0; */
 
     if (whackfd < 0)
     {
@@ -685,7 +690,7 @@ whack_handle(int whackctlfd)
 
     msg_saved = msg;
 
-    //DBG_log("msg %d size=%u\n", ++msgnum, n);
+    /* DBG_log("msg %d size=%u\n", ++msgnum, n); */
 
     /* sanity check message */
     {
