@@ -41,9 +41,9 @@
 #include <fcntl.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
-#include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
 #include <sys/resource.h>
 #include <sys/wait.h>
+#include <resolv.h>
 
 #if defined(IP_RECVERR) && defined(MSG_ERRQUEUE)
 #  include <asm/types.h>	/* for __u8, __u32 */
@@ -446,6 +446,14 @@ create_socket(struct raw_iface *ifp, const char *v_name, int port)
     udpfromto_init(fd);
 #endif
 
+    /* poke a hole for IKE messages in the IPsec layer */
+    if(kernel_ops->exceptsocket) {
+	if(!(*kernel_ops->exceptsocket)(fd, AF_INET)) {
+	    close(fd);
+	    return -1;
+	}
+    }
+
     return fd;
 }
 
