@@ -11,15 +11,17 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: server.h,v 1.31 2005-06-14 22:38:06 mcr Exp $
+ * RCSID $Id: server.h,v 1.32 2005/09/19 00:40:48 mcr Exp $
  */
 
 #ifndef _SERVER_H
 #define _SERVER_H
 
-#include <sys/queue.h>
+#include <sysqueue.h>
 
 extern bool no_retransmits;
+extern bool log_to_stderr_desired;
+
 extern int ctl_fd;	/* file descriptor of control (whack) socket */
 extern struct sockaddr_un ctl_addr;	/* address of control (whack) socket */
 
