@@ -469,6 +469,29 @@ delete_state(struct state *st)
     freeanychunk(st->st_shared);
     freeanychunk(st->st_ni);
     freeanychunk(st->st_nr);
+#ifdef HAVE_LIBNSS
+    free_osw_nss_symkey(st->st_skeyid);
+    free_osw_nss_symkey(st->st_skey_d);
+    free_osw_nss_symkey(st->st_skey_ai);
+    free_osw_nss_symkey(st->st_skey_ar);
+    free_osw_nss_symkey(st->st_skey_ei);
+    free_osw_nss_symkey(st->st_skey_er);
+    free_osw_nss_symkey(st->st_skey_pi);
+    free_osw_nss_symkey(st->st_skey_pr);
+    free_osw_nss_symkey(st->st_enc_key);
+
+    if(st->st_ah.our_keymat!=NULL)
+    memset(st->st_ah.our_keymat, 0, st->st_ah.keymat_len);
+
+    if(st->st_ah.peer_keymat!=NULL)
+    memset(st->st_ah.peer_keymat, 0, st->st_ah.keymat_len);
+
+    if(st->st_esp.our_keymat!=NULL)
+    memset(st->st_esp.our_keymat, 0, st->st_esp.keymat_len);
+
+    if(st->st_esp.peer_keymat!=NULL)
+    memset(st->st_esp.peer_keymat, 0, st->st_esp.keymat_len);
+#endif
     freeanychunk(st->st_skeyid);
     freeanychunk(st->st_skey_d);
     freeanychunk(st->st_skey_ai);
@@ -864,6 +887,19 @@ duplicate_state(struct state *st)
     clone_chunk(st_skeyid_a, "st_skeyid_a in duplicate_state");
     clone_chunk(st_skeyid_e, "st_skeyid_e in duplicate_state");
 #endif
+
+#ifdef HAVE_LIBNSS
+    dup_osw_nss_symkey(st->st_skeyseed);
+    dup_osw_nss_symkey(st->st_skey_d);
+    dup_osw_nss_symkey(st->st_skey_ai);
+    dup_osw_nss_symkey(st->st_skey_ar);
+    dup_osw_nss_symkey(st->st_skey_ei);
+    dup_osw_nss_symkey(st->st_skey_er);
+    dup_osw_nss_symkey(st->st_skey_pi);
+    dup_osw_nss_symkey(st->st_skey_pr);
+    dup_osw_nss_symkey(st->st_enc_key);
+#endif
+
     clone_chunk(st_enc_key,  "st_enc_key in duplicate_state");
 
     /* v2 duplication of state */
@@ -1207,9 +1243,9 @@ state_eroute_usage(ip_subnet *ours, ip_subnet *his
 	});
 }
 
-void fmt_state(struct state *st, time_t n
-, char *state_buf, size_t state_buf_len
-, char *state_buf2, size_t state_buf2_len)
+void fmt_state(struct state *st, const time_t n
+, char *state_buf, const size_t state_buf_len
+, char *state_buf2, const size_t state_buf2_len)
 {
     /* what the heck is interesting about a state? */
     const struct connection *c = st->st_connection;
@@ -1382,7 +1418,7 @@ state_compare(const void *a, const void *b)
 void
 show_states_status(void)
 {
-    time_t n = now();
+    const time_t n = now();
     int i;
     char state_buf[LOG_WIDTH];
     char state_buf2[LOG_WIDTH];
