@@ -58,6 +58,14 @@
 #include "whack.h"      /* for RC_LOG_SERIOUS */
 #include "keys.h"
 
+#ifdef HAVE_BROKEN_POPEN
+/*
+ * including this may be acceptable on a system without a working popen
+ * but a normal system should not need this, <errno.h> should cover it ;-)
+ */
+#include <asm-generic/errno.h>
+#endif
+
 /* invoke the updown script to do the routing and firewall commands required
  *
  * The user-specified updown script is run.  Parameters are fed to it in
@@ -130,9 +138,18 @@ bool invoke_command(const char *verb, const char *verb_suffix, char *cmd)
 
         if (f == NULL)
         {
-            loglog(RC_LOG_SERIOUS, "unable to popen %s%s command", verb, verb_suffix);
-	    signal(SIGCHLD, savesig);
-            return FALSE;
+#ifdef HAVE_BROKEN_POPEN
+	   /* See bug #1067  Angstrom Linux on a arm7 has no popen() */
+	   if (errno == ENOSYS) {
+		/* Try system(), though it will not give us output */
+		system(cmd);
+		DBG_log("unable to popen(), falling back to system()");
+		return TRUE;
+	   }
+#endif
+	   loglog(RC_LOG_SERIOUS, "unable to popen %s%s command", verb, verb_suffix);
+	   signal(SIGCHLD, savesig);
+	   return FALSE;
         }
 
         /* log any output */
