@@ -64,7 +64,7 @@ enum known_vendorid {
   VID_VISTA_AUTHIP2	    =52,
   VID_VISTA_AUTHIP3	    =53,
 
-  /* 101 - 200 : NAT-Traversal */
+  /* 101 - 150 : NAT-Traversal */
   VID_NATT_STENBERG_01       =101,
   VID_NATT_STENBERG_02       =102,
   VID_NATT_HUTTUNEN          =103,
@@ -77,6 +77,71 @@ enum known_vendorid {
   VID_NATT_DRAFT_IETF_IPSEC_NAT_T_IKE =110,
   VID_NATT_IETF_05           =111,
 
+/* 125 - 200 Strongswan */
+
+  VID_STRONGSWAN		= 126,
+  VID_STRONGSWAN_2_2_0		= 127,
+  VID_STRONGSWAN_2_2_1		= 128,
+  VID_STRONGSWAN_2_2_2		= 129,
+  VID_STRONGSWAN_2_3_0		= 130,
+  VID_STRONGSWAN_2_3_1		= 131,
+  VID_STRONGSWAN_2_3_2		= 132,
+  VID_STRONGSWAN_2_4_0		= 133,
+  VID_STRONGSWAN_2_4_1		= 134,
+  VID_STRONGSWAN_2_4_2		= 135,
+  VID_STRONGSWAN_2_4_3		= 136,
+  VID_STRONGSWAN_2_4_4		= 137,
+  VID_STRONGSWAN_2_5_0		= 138,
+  VID_STRONGSWAN_2_5_1		= 139,
+  VID_STRONGSWAN_2_5_2		= 140,
+  VID_STRONGSWAN_2_5_3		= 141,
+  VID_STRONGSWAN_2_5_4		= 142,
+  VID_STRONGSWAN_2_5_5		= 143,
+  VID_STRONGSWAN_2_5_6		= 144,
+  VID_STRONGSWAN_2_5_7		= 145,
+  VID_STRONGSWAN_2_6_0		= 146,
+  VID_STRONGSWAN_2_6_1		= 147,
+  VID_STRONGSWAN_2_6_2		= 148,
+  VID_STRONGSWAN_2_6_3		= 149,
+  VID_STRONGSWAN_2_6_4		= 150,
+  VID_STRONGSWAN_2_7_0		= 151,
+  VID_STRONGSWAN_2_7_1		= 152,
+  VID_STRONGSWAN_2_7_2		= 153,
+  VID_STRONGSWAN_2_7_3		= 154,
+  VID_STRONGSWAN_2_8_0		= 155,
+  VID_STRONGSWAN_2_8_1		= 156,
+  VID_STRONGSWAN_2_8_2		= 157,
+  VID_STRONGSWAN_2_8_3		= 158,
+  VID_STRONGSWAN_2_8_4		= 159,
+  VID_STRONGSWAN_2_8_5		= 160,
+  VID_STRONGSWAN_2_8_6		= 161,
+  VID_STRONGSWAN_2_8_7		= 162,
+  VID_STRONGSWAN_2_8_8		= 163,
+
+  VID_STRONGSWAN_4_0_0		= 170,
+  VID_STRONGSWAN_4_0_1		= 171,
+  VID_STRONGSWAN_4_0_2		= 172,
+  VID_STRONGSWAN_4_0_3		= 173,
+  VID_STRONGSWAN_4_0_4		= 174,
+  VID_STRONGSWAN_4_0_5		= 175,
+  VID_STRONGSWAN_4_0_6		= 176,
+  VID_STRONGSWAN_4_0_7		= 177,
+  VID_STRONGSWAN_4_1_0		= 178,
+  VID_STRONGSWAN_4_1_1		= 179,
+  VID_STRONGSWAN_4_1_2		= 180,
+  VID_STRONGSWAN_4_1_3		= 181,
+  VID_STRONGSWAN_4_1_4		= 182,
+  VID_STRONGSWAN_4_1_5		= 183,
+  VID_STRONGSWAN_4_1_6		= 184,
+  VID_STRONGSWAN_4_1_7		= 185,
+  VID_STRONGSWAN_4_1_8		= 186,
+  VID_STRONGSWAN_4_1_9		= 187,
+  VID_STRONGSWAN_4_1_10		= 188,
+  VID_STRONGSWAN_4_1_11		= 189,
+  VID_STRONGSWAN_4_2_0		= 190,
+  VID_STRONGSWAN_4_2_1		= 191,
+  VID_STRONGSWAN_4_2_2		= 192,
+  VID_STRONGSWAN_4_2_3		= 193,
 /* 
   While searching (strings) in /usr/sbin/racoon on Max OS X 10.3.3, I found it :
   # echo -n "draft-ietf-ipsec-nat-t-ike" | md5sum
