@@ -121,6 +121,10 @@ help(void)
 	    " \\\n   "
 	    " [--esp <esp-algos>]"
 	    " \\\n   "
+	    " [--remote_peer_type <cisco>]"
+	    " \\\n   "
+	    "[--nm_configured]"
+	    " \\\n   "
 	    " [--dontrekey]"
 	    " [--aggrmode]"
 	    " [--forceencaps]"
@@ -485,6 +489,8 @@ enum option_enums {
     CD_FORCEENCAPS,
     CD_IKE,
     CD_PFSGROUP,
+    CD_REMOTEPEERTYPE,
+    CD_NMCONFIGURED,
     CD_ESP	
 #   define CD_LAST CD_ESP	/* last connection description */
 
@@ -698,6 +704,10 @@ static const struct option long_opts[] = {
     { "ikealg", required_argument, NULL, CD_IKE + OO },
     { "pfsgroup", required_argument, NULL, CD_PFSGROUP + OO },
     { "esp", required_argument, NULL, CD_ESP + OO },
+    { "remote_peer_type", required_argument, NULL, CD_REMOTEPEERTYPE + OO},
+#ifdef HAVE_NM
+    { "nm_configured", no_argument, NULL, CD_NMCONFIGURED + OO},
+#endif
 #ifdef DEBUG
     { "debug-none", no_argument, NULL, DBGOPT_NONE + OO },
     { "debug-all]", no_argument, NULL, DBGOPT_ALL + OO },
@@ -904,6 +914,13 @@ main(int argc, char **argv)
     msg.ike = NULL;
     msg.pfsgroup = NULL;
 
+    msg.remotepeertype = NON_CISCO;
+
+    /*Network Manager support*/
+#ifdef HAVE_NM
+    msg.nmconfigured = NO;
+#endif
+
     msg.sa_ike_life_seconds = OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT;
     msg.sa_ipsec_life_seconds = PLUTO_SA_LIFE_DURATION_DEFAULT;
     msg.sa_rekey_margin = SA_REPLACEMENT_MARGIN_DEFAULT;
@@ -1505,6 +1522,26 @@ main(int argc, char **argv)
 	    msg.esp = optarg;
 	    continue;
 
+	case CD_REMOTEPEERTYPE: /* --remote_peer_type  <cisco> */
+	    if ( strcmp(optarg, "cisco" ) == 0) {
+		msg.remotepeertype = CISCO;
+	    }
+	    else {
+		msg.remotepeertype = NON_CISCO;
+	    }
+	    continue;
+
+#ifdef HAVE_NM
+	case CD_NMCONFIGURED: /* --nm_configured */
+	    if ( strcmp(optarg, "yes" ) == 0) {
+		msg.nmconfigured = YES;
+	    }
+	    else {
+		msg.nmconfigured = NO;
+	    }
+		continue;
+#endif
+
 	case CD_CONNIPV4:
 	    if (LHAS(cd_seen, CD_CONNIPV6 - CD_FIRST))
 		diag("--ipv4 conflicts with --ipv6");
@@ -1841,6 +1878,10 @@ main(int argc, char **argv)
             msg.dpd_action = DPD_ACTION_HOLD;
     }
 
+    if (msg.remotepeertype != CISCO && msg.remotepeertype != NON_CISCO) {
+            diag("remote_peer_type can only be \"CISCO\" or \"NON_CISCO\" - defaulting to non-cisco mode");
+            msg.remotepeertype = NON_CISCO; /*NON_CISCO=0*/
+    }
 
     /* pack strings for inclusion in message */
     wp.msg = &msg;
@@ -1957,8 +1998,10 @@ main(int argc, char **argv)
 		    }
 
 		    le++;	/* include NL in line */
-		    write(1, ls, le - ls);
-		    fsync(1);
+		    if(write(STDOUT_FILENO, ls, le - ls) != (le-ls)) {
+			int e = errno;
+			fprintf(stderr, "whack: write() failed to stdout(%d %s)\n", e, strerror(e));
+		    }
 
 		    /* figure out prefix number
 		     * and how it should affect our exit status
