@@ -19,7 +19,7 @@
  * Porting to 2.x by Sean Mathews
  */
 
-//#ifdef XAUTH
+/* #ifdef XAUTH */
 
 #include <stdio.h>
 #include <string.h>
@@ -716,6 +716,33 @@ stf_status modecfg_send_request(struct state *st)
 	attr.isaat_lv = 0;
 	out_struct(&attr, &isakmp_xauth_attribute_desc, &strattr, NULL);
 
+	if(st->st_connection->remotepeertype == CISCO) {
+	/* ISAKMP attr out (INTERNAL_IP4_DNS) */
+	attr.isaat_af_type = INTERNAL_IP4_DNS;
+	attr.isaat_lv = 0;
+	out_struct(&attr, &isakmp_xauth_attribute_desc, &strattr, NULL);
+
+	/* ISAKMP attr out (INTERNAL_IP4_NBNS) */
+	attr.isaat_af_type = INTERNAL_IP4_NBNS;
+	attr.isaat_lv = 0;
+	out_struct(&attr, &isakmp_xauth_attribute_desc, &strattr, NULL);
+
+	/* ISAKMP attr out (CISCO_BANNER) */
+	attr.isaat_af_type = CISCO_BANNER;
+	attr.isaat_lv = 0;
+	out_struct(&attr, &isakmp_xauth_attribute_desc, &strattr, NULL);
+
+        /* ISAKMP attr out (CISCO_DEF_DOMAIN) */
+        attr.isaat_af_type = CISCO_DEF_DOMAIN;
+        attr.isaat_lv = 0;
+        out_struct(&attr, &isakmp_xauth_attribute_desc, &strattr, NULL);
+
+	/* ISAKMP attr out (CISCO_SPLIT_INC) */
+	attr.isaat_af_type = CISCO_SPLIT_INC;
+	attr.isaat_lv = 0;
+	out_struct(&attr, &isakmp_xauth_attribute_desc, &strattr, NULL);
+	}
+
 	close_message(&strattr);
     }
 
@@ -1168,7 +1195,9 @@ xauth_inR0(struct msg_digest *md)
         struct isakmp_attribute attr;
         pb_stream strattr;
 
-	if (md->chain[ISAKMP_NEXT_ATTR]->payload.attribute.isama_type != ISAKMP_CFG_REPLY)
+	/* XXX This needs checking with the proper RFC's - ISAKMP_CFG_ACK got added for Cisco interop */
+	if ( (md->chain[ISAKMP_NEXT_ATTR]->payload.attribute.isama_type != ISAKMP_CFG_REPLY) &&
+	     (md->chain[ISAKMP_NEXT_ATTR]->payload.attribute.isama_type != ISAKMP_CFG_ACK) )
 	{
 	    openswan_log("Expecting MODE_CFG_REPLY, got %s instead."
 		 , enum_name(&attr_msg_type_names, md->chain[ISAKMP_NEXT_ATTR]->payload.attribute.isama_type));
@@ -1610,6 +1639,7 @@ modecfg_inR1(struct msg_digest *md)
     pb_stream *attrs = &md->chain[ISAKMP_NEXT_ATTR]->pbs;
     int resp = LEMPTY;
     struct payload_digest *p;
+    bool first_dns_flag = TRUE;
 
     DBG(DBG_CONTROL, DBG_log("modecfg_inR1"));
     openswan_log("received mode cfg reply");
@@ -1733,11 +1763,160 @@ modecfg_inR1(struct msg_digest *md)
 		break;
 		
 		case INTERNAL_IP4_NETMASK:
+		{
+		    ip_address a;
+		    char caddr[SUBNETTOT_BUF];
+
+		    u_int32_t *ap = (u_int32_t *)(strattr.cur);
+		    a.u.v4.sin_family = AF_INET;
+		    memcpy(&a.u.v4.sin_addr.s_addr, ap
+			, sizeof(a.u.v4.sin_addr.s_addr));
+
+		    addrtot(&a, 0, caddr, sizeof(caddr));
+		    openswan_log("Received IP4 NETMASK %s", caddr);
+		}
+		resp |= LELEM(attr.isaat_af_type);
+		break;
+
 		case INTERNAL_IP4_DNS:
+                {
+                    ip_address a;
+                    char caddr[SUBNETTOT_BUF];
+                
+                    u_int32_t *ap = (u_int32_t *)(strattr.cur);
+                    a.u.v4.sin_family = AF_INET;
+                    memcpy(&a.u.v4.sin_addr.s_addr, ap
+                        , sizeof(a.u.v4.sin_addr.s_addr));
+                    
+                    addrtot(&a, 0, caddr, sizeof(caddr));
+                    openswan_log("Received DNS %s, len=%zd", caddr, strlen(caddr));
+
+                    if (first_dns_flag) {
+                    strcpy(st->st_connection->cisco_dns_info, caddr);
+                    first_dns_flag = 0;
+                    }
+                    else {
+                    strcat(st->st_connection->cisco_dns_info, " ");
+                    strcat(st->st_connection->cisco_dns_info, caddr);
+                    }
+
+                    DBG_log("Cisco DNS info: %s, len=%zd", st->st_connection->cisco_dns_info, strlen(st->st_connection->cisco_dns_info));
+                }
+                resp |= LELEM(attr.isaat_af_type);
+                break;
+
+
 		case INTERNAL_IP4_SUBNET:
 		case INTERNAL_IP4_NBNS:
 		    resp |= LELEM(attr.isaat_af_type);
 		    break;
+
+		case CISCO_BANNER:
+                {
+                DBG_dump("Received cisco banner: ", strattr.cur, pbs_left(&strattr));
+		strncpy(st->st_connection->server_banner, strattr.cur, pbs_left(&strattr));
+		st->st_connection->server_banner[pbs_left(&strattr)]='\0';
+		DBG_log("Cisco banner: %s", st->st_connection->server_banner);
+                resp |= LELEM(attr.isaat_af_type);
+                }
+                break;
+
+
+		case CISCO_DEF_DOMAIN:
+                {
+                char tmp[50];
+                DBG_dump("Received cisco def domain: ", strattr.cur, pbs_left(&strattr));
+                strncpy(tmp, strattr.cur, pbs_left(&strattr));
+                tmp[pbs_left(&strattr)]='\0';
+                DBG_log("Cisco defined domain: %s", tmp);
+                strcpy(st->st_connection->cisco_domain_info, tmp);
+                DBG_log("Cisco defined domain: %s", st->st_connection->cisco_domain_info);
+                resp |= LELEM(attr.isaat_af_type);
+                }
+                break;
+
+		case CISCO_SPLIT_INC:
+                {
+                    struct spd_route *tmp_spd;
+                    ip_address a;
+                    char caddr[SUBNETTOT_BUF];
+                    size_t len = pbs_left(&strattr);
+                    struct connection *c = st->st_connection;
+                    struct spd_route *tmp_spd2 = &c->spd;
+                    
+                    if ( FALSE == tmp_spd2->that.has_client ) {
+                    ttosubnet("0.0.0.0/0.0.0.0", 0, AF_INET, &tmp_spd2->that.client);
+                    tmp_spd2->that.has_client = TRUE;
+                    tmp_spd2->that.has_client_wildcard = FALSE;
+                    }
+
+                    while (len > 0) {
+                    tmp_spd = clone_thing(c->spd, "remote subnets policies");
+
+                    tmp_spd->this.id.name.ptr = NULL;
+                    tmp_spd->this.id.name.len = 0;
+                    tmp_spd->that.id.name.ptr = NULL;
+                    tmp_spd->that.id.name.len = 0; 
+
+                    tmp_spd->this.host_addr_name = NULL;
+                    tmp_spd->that.host_addr_name = NULL;
+
+                    u_int32_t *ap = (u_int32_t *)(strattr.cur);
+                    a.u.v4.sin_family = AF_INET;
+                    memcpy(&a.u.v4.sin_addr.s_addr, ap
+                           , sizeof(a.u.v4.sin_addr.s_addr));
+
+                    addrtosubnet(&a, &tmp_spd->that.client);
+
+                    len -= sizeof(a.u.v4.sin_addr.s_addr);
+                    strattr.cur += sizeof(a.u.v4.sin_addr.s_addr);
+
+                    ap = (u_int32_t *)(strattr.cur);
+                    a.u.v4.sin_family = AF_INET;
+                    memcpy(&a.u.v4.sin_addr.s_addr, ap
+                           , sizeof(a.u.v4.sin_addr.s_addr));
+
+                    tmp_spd->that.client.maskbits = masktocount(&a);
+                    len -= sizeof(a.u.v4.sin_addr.s_addr);
+                    strattr.cur += sizeof(a.u.v4.sin_addr.s_addr);
+
+                    setportof(0, &tmp_spd->that.client.addr);
+
+                    len -= 6;
+                    strattr.cur += 6;
+
+                    subnettot(&tmp_spd->that.client, 0
+                              , caddr, sizeof(caddr));
+
+                    openswan_log("Received subnet %s, maskbits %d", caddr, tmp_spd->that.client.maskbits);
+
+                    tmp_spd->this.updown = clone_str(tmp_spd->this.updown, "updown");
+                    tmp_spd->that.updown = clone_str(tmp_spd->that.updown, "updown");
+
+                    tmp_spd->this.cert_filename = NULL;
+                    tmp_spd->that.cert_filename = NULL;
+
+                    tmp_spd->this.cert.type = 0;
+                    tmp_spd->that.cert.type = 0;
+
+                    tmp_spd->this.ca.ptr = NULL;
+                    tmp_spd->this.ca.ptr = NULL;
+
+                    tmp_spd->this.groups = NULL;
+                    tmp_spd->this.groups = NULL;
+
+                    tmp_spd->this.virt = NULL;
+                    tmp_spd->this.virt = NULL;
+
+                    tmp_spd->next = NULL;
+                    tmp_spd2->next = tmp_spd;
+                    tmp_spd2 = tmp_spd;
+                    }
+
+                }
+                resp |= LELEM(attr.isaat_af_type);
+                break;
+
 		default:
 		    openswan_log("unsupported mode cfg attribute %s received."
 				 , enum_show(&modecfg_attr_names, (attr.isaat_af_type & ISAKMP_ATTR_RTYPE_MASK )));
@@ -1819,6 +1998,7 @@ stf_status xauth_client_resp(struct state *st
 	    dont_advance = FALSE;
 	    if(xauth_resp & 1)
 	    {
+		bool password_read_from_prompt = FALSE;
 		/* ISAKMP attr out */
 		switch(attr_type)
 		{
@@ -1911,14 +2091,26 @@ stf_status xauth_client_resp(struct state *st
 			clonereplacechunk(st->st_xauth_password
 					  , xauth_password, strlen(xauth_password)
 					  , "XAUTH password");
+			password_read_from_prompt = TRUE;
 		    }
 		    
 		    out_raw(st->st_xauth_password.ptr
 			    , st->st_xauth_password.len
 			    , &attrval, "XAUTH password");
+
+		    /*
+		     * Do not store the password read from the prompt. The password
+		     * could have be read from a one-time token device (like SecureID)
+		     * or the password could have been entereted wrong,
+		     */
+		    if (password_read_from_prompt) {
+			freeanychunk (st->st_xauth_password);
+			st->st_xauth_password.len = 0;
+			password_read_from_prompt = FALSE;
+		    }
 		    close_output_pbs(&attrval);
 		    break;
-		    
+
 		default:
 		    openswan_log("trying to send XAUTH reply, sending %s instead."
 			 , enum_show(&modecfg_attr_names, attr_type));
