@@ -36,6 +36,7 @@
 # include <prerror.h>
 # include <prinit.h>
 # include <prmem.h>
+# include <plstr.h>
   /* nss */
 # include <key.h>
 # include <keyt.h>
@@ -288,11 +289,11 @@ char *GetModulePassword(PK11SlotInfo *slot, PRBool retry, void *arg)
 		*/
 		pw = GetFilePasswd(slot, retry, pwdata->data);
 		pwdata->source = PW_PLAINTEXT;
-		pwdata->data = PL_strdup(pw);
+		pwdata->data = strdup(pw);
 		/* it's already been dup'ed */
 		return pw;
 	case PW_PLAINTEXT:
-		return PL_strdup(pwdata->data);
+		return strdup(pwdata->data);
 	default: /* cases PW_NONE and PW_EXTERNAL not supported */
 		fprintf(stderr, "Unknown or unsupported case in GetModulePassword");
 		break;
@@ -568,7 +569,7 @@ rsasigkey(int nbits, char *configdir, char *password)
 	}
 
 	PR_Init(PR_USER_THREAD, PR_PRIORITY_NORMAL, 1);
-	snprintf(buf, sizeof(buf), "sql:%s",configdir);
+	snprintf(buf, sizeof(buf), "%s",configdir);
 	if ((rv = NSS_InitReadWrite(buf)) != SECSuccess) {
 		fprintf(stderr, "%s: NSS_InitReadWrite returned %d\n", me, PR_GetError());
 		break;
@@ -588,11 +589,11 @@ rsasigkey(int nbits, char *configdir, char *password)
 	PK11_SetPasswordFunc(GetModulePassword);
 	nss_initialized = PR_TRUE;
 
-	/* Good for now but someone may want to use a hardware token*/
-	 slot = PK11_GetInternalKeySlot();
-	 /* In which case this may be better */
-	//slot = PK11_GetBestSlot(CKM_RSA_PKCS_KEY_PAIR_GEN, password ? &pwdata : NULL);
-	/*or the user may specify the name of a token.*/
+	/* Good for now but someone may want to use a hardware token */
+	slot = PK11_GetInternalKeySlot();
+	/* In which case this may be better */
+	/* slot = PK11_GetBestSlot(CKM_RSA_PKCS_KEY_PAIR_GEN, password ? &pwdata : NULL); */
+	/* or the user may specify the name of a token. */
 
 	/*if (PK11_IsFIPS() || !PK11_IsInternal(slot)) {
 		rv = PK11_Authenticate(slot, PR_FALSE, &pwdata);
