@@ -27,7 +27,7 @@ char showpolicy_version[] = "RCSID $Id: showpolicy.c,v 1.5 2004/04/04 01:50:56 k
 #include "openswan/ipsec_policy.h"
 #include "sysdep.h"
 #include "socketwrapper.h"
-#include "rcvinfo.h"
+#include "ipsec_saref.h"
 
 char *program_name;
 
@@ -122,7 +122,7 @@ int udp_recv_loop(int udpsock)
 	int packetcount =0;
 	
 	do {
-		unsigned int pktref[1];
+		unsigned int pktref[2] = {0};
 
 		memset(&from, 0, sizeof(from));
 		memset(&to,   0, sizeof(to));
