@@ -9,7 +9,7 @@ HOST_START=$POOLSPACE/$TESTHOST/startmodule.sh
 EXITONEMPTY=--exitonempty
 PRIV_INPUT=../inputs/01-sunrise-sunset-ping.pcap
 
-THREEEIGHT=true
+#THREEEIGHT=true
 REF_PUB_OUTPUT=spi1-output.txt
 REF_CONSOLE_OUTPUT=spi1-console.txt
 REF26_CONSOLE_OUTPUT=spi1-console.txt
