@@ -1,4 +1,5 @@
 #!/bin/sh
+: ==== start ====
 
 TZ=GMT export TZ
 
@@ -28,3 +29,4 @@ ipsec look
 # magic route command
 route add -host 192.0.2.1 gw 192.1.2.23 dev ipsec0
 
+: ==== end ====
