@@ -7,5 +7,8 @@ TESTNAME=aggr-pluto-01
 source /testing/pluto/bin/eastlocal.sh
 
 ipsec setup start
+/testing/pluto/bin/wait-until-pluto-started
+ipsec whack --whackrecord /var/tmp/aggr.record 
+
 ipsec auto --add westnet-eastnet-aggr
-/testing/pluto/basic-pluto-01/eroutewait.sh trap
+echo done
