@@ -14,9 +14,8 @@ TESTNAME=aggr-pluto-01
 source /testing/pluto/bin/westlocal.sh
 
 ipsec setup start
+/testing/pluto/bin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-aggr
-/testing/pluto/basic-pluto-01/eroutewait.sh trap
-ipsec auto --up  westnet-eastnet-aggr
 
 echo done
 
