@@ -1,3 +1,4 @@
+: ==== start ====
 TESTNAME=basic-pluto-01
 source /testing/pluto/bin/westlocal.sh
 
@@ -11,7 +12,7 @@ ping -n -c 4 192.0.2.254
 ipsec setup start
 ipsec auto --add westnet-eastnet
 ipsec whack --debug-control --debug-controlmore --debug-parsing --debug-crypt
-/testing/pluto/basic-pluto-01/eroutewait.sh trap
+/testing/pluto/bin/wait-until-pluto-started
 
 echo done
 
