@@ -22,12 +22,13 @@ fi
 OPENSWANSRCDIR=`cd ${OPENSWANSRCDIR}; pwd`
 export OPENSWANSRCDIR
 
+FIXUPDIR=`cd ${OPENSWANSRCDIR}/testing/scripts/fixups && pwd`
+FIXUPDIR2=`cd ${OPENSWANSRCDIR}/testing/klips/fixups && pwd`
 TESTINGROOT=${OPENSWANSRCDIR}/testing
 UTILS=`cd ${TESTINGROOT}/utils && pwd`
-FIXUPDIR=`cd ${OPENSWANSRCDIR}/testing/klips/fixups && pwd`
 NJ=${UTILS}/uml_netjig/uml_netjig
 
-REGRESSRESULTS=${REGRESSRESULTS-results}
+TESTSUBDIR=scripts
 
 
 
