@@ -24,44 +24,46 @@ $unexpectedcolour="#009999"; # YELLOW
 $expectedcolour="#007700"; # dark green
 $roguecolour="#999900";    # purple?
 
+$fastrate=20;    # when test are running, how often to refresh
+$slowrate=300;   # when tests are not running, how often to refresh.
+$maxpieceperpage=20;
+
 $failed=0;
 $passed=0;
 $missed=0;
 $total=0;
+$skipped=0;
 
 #$gnatsurl="http://gnats.freeswan.org/bugs/gnatsweb.pl?database=freeswan&amp;cmd=view+audit-trail&amp;pr=";
 $gnatsurl=undef;
 
 @faillist=();
 
+sub printit {
+  print HTMLFILE @_;
+  print HTMLPART @_;
+}
+
 sub htmlize_test {
   local($testname)=@_;
 
   my($expected, $verdict, $packetstat, $consolestat, $file);
 
-  if(++$linecount > 40) {
-	print HTMLFILE "</TABLE>\n";
-	print HTMLFILE "<TD>";
-	print HTMLFILE "<TABLE>\n";
+  &openpiecefile;
 
-	print HTMLFILE "<TR><TH COLSPAN=3>$testtypename tests</TH></TR>\n";
-	print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
- 	$linecount=4;
-  }	
-
-  print HTMLFILE "<TR><TD>";
+  &printit("<TR><TD>");
 
   if(-f "$testname/description.txt") {
-    print HTMLFILE "<A HREF=\"$testname/description.txt\">$testname</A>";
+    &printit("<A HREF=\"$testname/description.txt\">$testname</A>");
   } else {
-    print HTMLFILE "$testname";
+    &printit("$testname");
   }
 
-  print HTMLFILE "</TD>";
+  &printit("</TD>");
 
   if($testname eq $runningtest) {
       $verdict="<BLINK>**RUNNING**</BLINK>";
-      print HTMLFILE "<TD>$verdict</TD></TR>";
+      &printit("<TD>$verdict</TD></TR>");
       return;
   } 
 
@@ -108,7 +110,8 @@ sub htmlize_test {
       if(defined($expected) &&
         $expected ne 'missing') {
 	$verdict .= " <FONT COLOR=\"$unexpectedcolour\">AWOL</FONT>";
-      }	
+      }
+      $skipped++;	
     } else {
       $verdict = "${old}FAILED";
       if(!defined($expected) ||
@@ -223,7 +226,7 @@ sub htmlize_test {
     $missed++;
   }
 
-  print HTMLFILE "<TD>$verdict</TD>";
+  &printit("<TD>$verdict</TD>");
 
   if(-f "$testname/regress.txt") {
     open(PROBREPORT, "$testname/regress.txt") || die "$testname/regress.txt: $!\n";
@@ -231,7 +234,7 @@ sub htmlize_test {
     close(PROBREPORT);
 
     if($prnum > 0 && defined($gnatsurl)) {
-	    print HTMLFILE "<TD><A HREF=\"$gnatsurl$prnum\">PR#$prnum</A></TD>";
+	    &printit("<TD><A HREF=\"$gnatsurl$prnum\">PR#$prnum</A></TD>");
     }
 
   } elsif(-f "$testname/goal.txt") {
@@ -240,7 +243,7 @@ sub htmlize_test {
     close(GOALREQ);
 
     $goalnum=sprintf("%03d", $goalnum);
-    #print HTMLFILE "<TD><A HREF=\"http://www.freeswan.org/freeswan_snaps/CURRENT-SNAP/klips/doc/klipsNGreq/requirements/$goalnum\">Requirement $goalnum</A></TD>";
+    #&printit("<TD><A HREF=\"http://www.freeswan.org/freeswan_snaps/CURRENT-SNAP/klips/doc/klipsNGreq/requirements/$goalnum\">Requirement $goalnum</A></TD>";
 
   } elsif(-f "$testname/exploit.txt") {
     open(EXPLOIT, "$testname/exploit.txt") || die "$testname/exploit.txt: $!\n";
@@ -251,7 +254,7 @@ sub htmlize_test {
     # test not categorized, output nothing.
   }
 
-  print HTMLFILE "</TR>\n";
+  &printit("</TR>\n");
 }
 
 # the test names are sorted.
@@ -285,21 +288,23 @@ if(defined($runningtest)) {
 
 @testnames=sort @tests;
 
-#
-# make pass through the tests, categorizing them.
-#
-@regresstests=();
-@goaltests=();
-@exploittests=();
-foreach $testname (@testnames) {
-  if(-f "$testname/regress.txt") {
-    push(@regresstests,$testname);
-  } elsif(-f "$testname/goal.txt") {
-    push(@goaltests, $testname);
-  } elsif(-f "$testname/exploit.txt") {
-    push(@exploittests, $testname);
-  } else {
-    push(@regresstests,$testname);
+if($wanttestcategories) {
+  #
+  # make pass through the tests, categorizing them.
+  #
+  @regresstests=();
+  @goaltests=();
+  @exploittests=();
+  foreach $testname (@testnames) {
+    if(-f "$testname/regress.txt") {
+      push(@regresstests,$testname);
+    } elsif(-f "$testname/goal.txt") {
+      push(@goaltests, $testname);
+    } elsif(-f "$testname/exploit.txt") {
+      push(@exploittests, $testname);
+    } else {
+      push(@regresstests,$testname);
+    }
   }
 }
 
@@ -315,67 +320,122 @@ $hostname=`uname -n`;
 
 open(HTMLFILE, ">testresults.html") || die "Can not open testresults.html: $!\n";
 
-print HTMLFILE "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n";
-print HTMLFILE "<HTML>  <HEAD>\n";
-print HTMLFILE "<META http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">\n";
+$piece = 1;
+$piececount=0;
 
-if(defined($runningtest)) {
-  print HTMLFILE "<META http-equiv=\"Refresh\" content=\"15,testresults.html\">\n";
+sub openpiecefile {
+  if($piececount == 0) {
+    $file = sprintf("testresult%02d.html", $piece);
+
+    print HTMLPART "</TABLE>  \n";
+    print HTMLPART "\n<BR><PRE>TOTAL tests: $total SKIPPED: $skipped   PASSED: $passed   FAILED: $failed   MISSED: $missed SKIPPED: $skipped SUCCESS RATE: $testrate%</PRE><BR>\n";
+    print HTMLPART "<A HREF=\"$file\">next page</A><BR>\n";
+    print HTMLPART "<A HREF=\"stdout.txt\">stdout</A><BR>\n";
+    print HTMLPART "<A HREF=\"stderr.txt\">stderr</A><BR>\n";
+    print HTMLPART "</BODY></HTML>\n";
+    close(HTMLPART);
+
+    open(HTMLPART, ">$file") || die "Can not open $file: $!\n";
+
+    print HTMLPART "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n";
+    print HTMLPART "<HTML>  <HEAD>\n";
+    print HTMLPART "<META http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">\n";
+    
+    if(defined($runningtest)) {
+      print HTMLPART "<META http-equiv=\"Refresh\" content=\"$fastrate,$file\">\n";
+    } else {
+      print HTMLPART "<META http-equiv=\"Refresh\" content=\"$slowrate,$file\">\n";
+    }
+    
+    print HTMLPART "<TITLE>Openswan nightly testing results for $runtime</TITLE>\n";
+    print HTMLPART "</HEAD>  <BODY>\n";
+    print HTMLPART "<H1>Openswan nightly testing results for $runtime on $hostname</H1>\n";
+    
+    if(defined($runningtest)) {
+      print HTMLPART "Currently running $runningtest<P>\n";
+    }
+    
+    
+    print HTMLPART "<TABLE border>\n";
+    print HTMLPART "<TD>";
+    
+    print HTMLPART "<TR><TH COLSPAN=3>Regression tests</TH></TR>\n";
+    print HTMLPART "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
+    $piece++;
+    $piececount=$maxpieceperpage;
+  }
+  $piececount--;
 }
 
-print HTMLFILE "<TITLE>Openswan nightly testing results for $runtime</TITLE>\n";
-print HTMLFILE "</HEAD>  <BODY>\n";
-print HTMLFILE "<H1>Openswan nightly testing results for $runtime on $hostname</H1>\n";
+&printit("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n");
+&printit("<HTML>  <HEAD>\n");
+&printit("<META http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">\n");
 
 if(defined($runningtest)) {
-  print HTMLFILE "Currently running $runningtest<P>\n";
+  &printit("<META http-equiv=\"Refresh\" content=\"$fastrate,testresults.html\">\n");
+} else {
+  &printit("<META http-equiv=\"Refresh\" content=\"$slowrate,testresults.html\">\n");
+}
+
+&printit("<TITLE>Openswan nightly testing results for $runtime</TITLE>\n");
+&printit("</HEAD>  <BODY>\n");
+&printit("<H1>Openswan nightly testing results for $runtime on $hostname</H1>\n");
+
+if(defined($runningtest)) {
+  &printit("Currently running $runningtest<P>\n");
 }
 
 
-print HTMLFILE "<TABLE border>\n";
-print HTMLFILE "<TD>";
+&printit("<TABLE border>\n");
+&printit("<TD>");
 
 $testtypename="Regression";
-print HTMLFILE "<TABLE>\n";
-print HTMLFILE "<TR><TH COLSPAN=3>Regression tests</TH></TR>\n";
-print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
+&printit("<TR><TH COLSPAN=3>Regression tests</TH></TR>\n");
+&printit("<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n");
 $linecount=3;
 
-foreach $testname (@regresstests) {
-  next if($testname =~ /^\./);
-  next unless(-d $testname || $testname eq $runningtest);
-
-  &htmlize_test($testname);
+if($wanttestcategories) {
+  foreach $testname (@regresstests) {
+    next if($testname =~ /^\./);
+    next unless(-d $testname || $testname eq $runningtest);
+    
+    &htmlize_test($testname);
+  }
+  
+  $testtypename="Goal";
+  &printit("<TR><TH COLSPAN=3>Goal tests</TH></TR>\n");
+  &printit("<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n");
+  $linecount+=3;
+  
+  foreach $testname (@goaltests) {
+    next if($testname =~ /^\./);
+    next unless(-d $testname || $testname eq $runningtest);
+    
+    &htmlize_test($testname);
+  }
+  
+  $testtypename="Exploit ";
+  &printit("<TR><TH COLSPAN=3>Exploit tests</TH></TR>\n");
+  &printit("<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n");
+  $linecount+=3;
+  
+  foreach $testname (@exploittests) {
+    next if($testname =~ /^\./);
+    next unless(-d $testname || $testname eq $runningtest);
+    
+    &htmlize_test($testname);
+  }
+} else {
+  foreach $testname (@testnames) {
+    next if($testname =~ /^\./);
+    next unless(-d $testname || $testname eq $runningtest);
+    
+    &htmlize_test($testname);
+  }
 }
-
-$testtypename="Goal";
-print HTMLFILE "</TABLE>\n";
-print HTMLFILE "<TABLE>\n";
-print HTMLFILE "<TR><TH COLSPAN=3>Goal tests</TH></TR>\n";
-print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
-$linecount+=3;
-
-foreach $testname (@goaltests) {
-  next if($testname =~ /^\./);
-  next unless(-d $testname || $testname eq $runningtest);
-
-  &htmlize_test($testname);
-}
-
-$testtypename="Exploit ";
-print HTMLFILE "</TABLE>\n";
-print HTMLFILE "<TABLE>\n";
-print HTMLFILE "<TR><TH COLSPAN=3>Exploit tests</TH></TR>\n";
-print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
-$linecount+=3;
-
-foreach $testname (@exploittests) {
-  next if($testname =~ /^\./);
-  next unless(-d $testname || $testname eq $runningtest);
-
-  &htmlize_test($testname);
-}
-
+  
+  
+  
 $subtotal = $passed + $failed;
 $skipped = $total - $subtotal;
 
@@ -385,12 +445,11 @@ if($subtotal > 0) {
   $testrate="inf";
 }
 
-print HTMLFILE "</TABLE>  \n";
-print HTMLFILE "</TABLE>  \n";
-print HTMLFILE "\n<BR><PRE>TOTAL tests: $total SKIPPED: $skipped   PASSED: $passed   FAILED: $failed   MISSED: $missed  SUCCESS RATE: $testrate%</PRE><BR>\n";
-print HTMLFILE "<A HREF=\"stdout.txt\">stdout</A><BR>\n";
-print HTMLFILE "<A HREF=\"stderr.txt\">stderr</A><BR>\n";
-print HTMLFILE "</BODY></HTML>\n";
+&printit("</TABLE>  \n");
+&printit("\n<BR><PRE>TOTAL tests: $total SKIPPED: $skipped   PASSED: $passed   FAILED: $failed   MISSED: $missed  SUCCESS RATE: $testrate%</PRE><BR>\n");
+&printit("<A HREF=\"stdout.txt\">stdout</A><BR>\n");
+&printit("<A HREF=\"stderr.txt\">stderr</A><BR>\n");
+&printit("</BODY></HTML>\n");
 close(HTMLFILE);
 
 if(!defined($runningtest)) {
@@ -400,7 +459,7 @@ if(!defined($runningtest)) {
   close(FAILLIST);
 
   open(STATS, ">stats.txt") || die "failed to write to stats.txt: $!\n";
-  print STATS "$timestamp $total $passed $failed $missed\n";
+  print STATS "$timestamp $total $passed $failed $missed $skipped\n";
   close(STATS);
 }
 
