@@ -44,7 +44,9 @@ setup_make() {
     fi
 
     # now describe how to build the initrd.
-    echo "initrd.uml: ${OPENSWANSRCDIR}/testing/utils/initrd.list"
+    # echo "initrd.uml: ${OPENSWANSRCDIR}/testing/utils/initrd-x86_64.list"
+    # echo "$TAB fakeroot ${OPENSWANSRCDIR}/testing/utils/buildinitrd ${OPENSWANSRCDIR}/testing/utils/initrd-x86_64.list ${OPENSWANSRCDIR} ${BASICROOT}" 
+    echo "initrd.cpio: ${OPENSWANSRCDIR}/testing/utils/initrd.list"
     echo "$TAB fakeroot ${OPENSWANSRCDIR}/testing/utils/buildinitrd ${OPENSWANSRCDIR}/testing/utils/initrd.list ${OPENSWANSRCDIR} ${BASICROOT}" 
 }
 
@@ -163,6 +165,7 @@ setup_host_make() {
     echo "$TAB echo none	   /usr/obj		     hostfs   defaults,ro,\${OBJDIRTOP} 0 0 >>$hostroot/etc/fstab"
     echo "$TAB echo none	   /usr/local		     hostfs   defaults,rw,${POOLSPACE}/${hostroot}/usr/local 0 0 >>$hostroot/etc/fstab"
     echo "$TAB echo none	   /var/tmp		     hostfs   defaults,rw,${POOLSPACE}/${hostroot}/var/tmp 0 0 >>$hostroot/etc/fstab"
+    echo "$TAB echo /var/lib/swapfile none  swap    sw              0       0  >>$hostroot/etc/fstab"
     depends="$depends $hostroot/etc/fstab"
 
     # split Debian "interfaces" file into RH ifcfg-* file
@@ -196,14 +199,15 @@ setup_host_make() {
 
 	    # make module startup script
 	    startscript=$POOLSPACE/$host/startmodule.sh
-	    echo "$startscript : $OPENSWANSRCDIR/umlsetup.sh $hostroot/ipsec.o initrd.uml"
+	    echo "$startscript : $OPENSWANSRCDIR/umlsetup.sh $hostroot/ipsec.o initrd.cpio"
 	    echo "$TAB echo '#!/bin/sh' >$startscript"
 	    echo "$TAB echo ''          >>$startscript"
 	    echo "$TAB echo '# get $net value from baseconfig'          >>$startscript"
 	    echo "$TAB echo . ${TESTINGROOT}/baseconfigs/net.$host.sh   >>$startscript"
 	    echo "$TAB echo ''          >>$startscript"
 	    echo "$TAB # the umlroot= is a local hack >>$startscript"
-	    echo "$TAB echo '$POOLSPACE/plain${KERNVER}/linux initrd=$POOLSPACE/initrd.uml umlroot=$POOLSPACE/$hostroot testname=$TESTNAME root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  init=/linuxrc gim\$\$*' >>$startscript"
+	    # echo "$TAB echo '$POOLSPACE/plain${KERNVER}/linux load_ramdisk=1 ramdisk_size=98304 mem=256M initrd=$POOLSPACE/initrd.uml umlroot=$POOLSPACE/$hostroot testname=$TESTNAME root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  selinux=0 init=/linuxrc gim\$\$*' >>$startscript"
+	    echo "$TAB echo '$POOLSPACE/plain${KERNVER}/linux initrd=$POOLSPACE/initrd.cpio umlroot=$POOLSPACE/$hostroot root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  rdinit=/linuxrc gim\$\$*' >>$startscript"
 	    echo "$TAB chmod +x $startscript"
 	    echo
 	    depends="$depends $startscript"
@@ -221,7 +225,7 @@ setup_host_make() {
      echo "$TAB echo . ${TESTINGROOT}/baseconfigs/net.$host.sh   >>$startscript"
      echo "$TAB echo ''          >>$startscript"
      echo "$TAB # the umlroot= is a local hack >>$startscript"
-     echo "$TAB echo '$NETKEY_KERNEL initrd=$POOLSPACE/initrd.uml umlroot=$POOLSPACE/$hostroot testname=$TESTNAME root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  init=/linuxrc \$\$*' >>$startscript"
+     echo "$TAB echo '$NETKEY_KERNEL load_ramdisk=1 ramdisk_size=98304 mem=256M initrd=$POOLSPACE/initrd.uml umlroot=$POOLSPACE/$hostroot testname=$TESTNAME root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  selinux=0 init=/linuxrc \$\$*' >>$startscript"
      echo "$TAB echo 'if [ -n \"\$\$UML_SLEEP\" ]; then eval \$\$UML_SLEEP; fi'  >>$startscript"
      echo "$TAB chmod +x $startscript"
      echo
@@ -229,14 +233,15 @@ setup_host_make() {
     fi
     # make startup script for KLIPS uml (no modules)
     startscript=$POOLSPACE/$host/start.sh
-    echo "$startscript : $OPENSWANSRCDIR/umlsetup.sh initrd.uml"
+    echo "$startscript : $OPENSWANSRCDIR/umlsetup.sh initrd.cpio"
     echo "$TAB echo '#!/bin/sh' >$startscript"
     echo "$TAB echo ''          >>$startscript"
     echo "$TAB echo '# get $net value from baseconfig'          >>$startscript"
     echo "$TAB echo . ${TESTINGROOT}/baseconfigs/net.$host.sh   >>$startscript"
     echo "$TAB echo ''          >>$startscript"
     echo "$TAB # the umlroot= is a local hack >>$startscript"
-    echo "$TAB echo '$KERNEL initrd=$POOLSPACE/initrd.uml umlroot=$POOLSPACE/$hostroot testname=$TESTNAME root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  init=/linuxrc \$\$*' >>$startscript"
+    #echo "$TAB echo '$KERNEL load_ramdisk=1 ramdisk_size=98304 mem=256M initrd=$POOLSPACE/initrd.uml umlroot=$POOLSPACE/$hostroot testname=$TESTNAME root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT selinux=0  init=/linuxrc \$\$*' >>$startscript"
+    echo "$TAB echo '$KERNEL initrd=$POOLSPACE/initrd.cpio umlroot=$POOLSPACE/$hostroot root=/dev/ram0 rw ssl=pty umid=$host \$\$net \$\$UML_DEBUG_OPT \$\$UML_"${host}"_OPT  rdinit=/linuxrc \$\$*' >>$startscript"
     echo "$TAB echo 'if [ -n \"\$\$UML_SLEEP\" ]; then eval \$\$UML_SLEEP; fi'  >>$startscript"
     echo "$TAB chmod +x $startscript"
     echo
@@ -298,6 +303,7 @@ setup_host() {
 
     # setup the mount of /usr/share
     echo "none	   /usr/share		     hostfs   defaults,ro,$SHAREROOT 0 0" >>$hostroot/etc/fstab
+    echo "$TAB echo /var/lib/swapfile  none  swap    sw              0       0  >>$hostroot/etc/fstab"
 
     # split Debian "interfaces" file into RH ifcfg-* file
     mkdir -p $hostroot/etc/sysconfig/network-scripts
