@@ -13,7 +13,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: uml_netjig.c,v 1.27.26.1 2005-08-12 16:32:01 ken Exp $
+ * RCSID $Id: uml_netjig.c,v 1.27 2003/04/02 20:25:53 mcr Exp $
  *
  * @(#) based upon uml_router from User-Mode-Linux tools package
  *
@@ -97,7 +97,7 @@ void *xmalloc1(size_t size, char *file, int linenum)
 
 	space = malloc(size);
 	if(space == NULL) {
-		fprintf(stderr, "no memory allocating %d bytes at %s:%d\n",
+		fprintf(stderr, "no memory allocating %zu bytes at %s:%d\n",
 			size, file, linenum);
 		exit(1);
 	}
@@ -107,7 +107,7 @@ void *xmalloc1(size_t size, char *file, int linenum)
 static void Usage(void)
 {
   fprintf(stderr, "Usage : uml_netjig \n"
-	  "Version $Revision: 1.27.26.1 $ \n\n"
+	  "Version $Revision: 1.27 $ \n\n"
       "\t--cmdproto (-C)             go into the command protocol prompt\n"
       "\t--exitonempty (-e)          exit when no more packets to read\n"
       "\t--playpublic (-p) <file>    pcap(3) file to feed into public side\n"
@@ -119,7 +119,7 @@ static void Usage(void)
 #ifdef NETDISSECT
 	  "\t--tcpdump (-t)           dump packets with tcpdump-dissector\n"
 #else
-	  "\t--tcpdump (-t)           (not available - dissector not built in)\n"
+	  "\t--tcpdump (-t)           hexdump packets (dissector not built in)\n"
 #endif
 #ifdef ARP_PROCESS
 	  "\t--arpreply (-a)          respond to ARP requests\n"
@@ -315,13 +315,7 @@ int main(int argc, char **argv)
 	    break;
 	    
     case 't':
-#ifdef NETDISSECT
 	    tcpdump_print = 1;
-#else
-	    if(ns.verbose) {
-		    fprintf(stderr, "tcpdump dissector not available\n");
-	    }
-#endif
 	    break;
 
     case 'p':
@@ -474,8 +468,8 @@ int main(int argc, char **argv)
     }
 
     if(n < 0 && errno!=EINTR) {
-      perror("poll");
-      ns.done = 1;
+	    perror("poll");
+	    ns.done = 1;
     }
 
     if((timeout!=-1 && n == 0) || ns.forcetick) {
@@ -510,7 +504,7 @@ int main(int argc, char **argv)
 			    /* advance the pointer once */
 			    onh = onh->nh_link.tqe_next;
 			    
-			    assert(onh!=onh2);  /* is there a loop? */
+			    assert(onh!=onh2);  /*oh shit, is there a loop? */
 		    }
 	    }
 	    /*
@@ -527,7 +521,17 @@ int main(int argc, char **argv)
 	
 		    packet = pcap_next(nh->nh_input, &ph);
 		    if(packet == NULL) {
-			    nh->nh_input=NULL;
+			    if(ns.forcetick) {
+				    char errbuf[PCAP_ERRBUF_SIZE];
+				    /*
+				     * when we force a tick, we should wrap
+				     * the inputs too!
+				     */
+				    pcap_close(nh->nh_input);
+				    nh->nh_input = pcap_open_offline(nh->nh_inputFile, errbuf);
+			    } else {
+				    nh->nh_input=NULL;
+			    }
 		    } else {
 			    gotinput = 1;
 			    if(ns.verbose) {
@@ -658,9 +662,6 @@ int main(int argc, char **argv)
 
 /*
  * $Log: uml_netjig.c,v $
- * Revision 1.27.26.1  2005-08-12 16:32:01  ken
- * Make tree PG13
- *
  * Revision 1.27  2003/04/02 20:25:53  mcr
  * 	quiet down uml_netjig unless -v or $NETJIGVERBOSE is set.
  *
